Package: kSamples
Type: Package
Title: K-Sample Rank Tests and their Combinations
Version: 1.2-12
Date: 2025-08-25
Authors@R: c(person("Fritz", "Scholz",role=c("aut","cre"),email =
        "fscholz@u.washington.edu"), person("Angie", "Zhu", role=c("aut"),
        email= "a.zhu.stat@gmail.com"))
Author: Fritz Scholz [aut, cre],
  Angie Zhu [aut]
Maintainer: Fritz Scholz <fscholz@u.washington.edu>
Depends: SuppDists
Imports: methods, graphics, stats
Description: Compares k samples using the Anderson-Darling test, Kruskal-Wallis type tests 
	with different rank score criteria, Steel's multiple comparison test, and the 
        Jonckheere-Terpstra (JT) test. It computes asymptotic, simulated or (limited) exact 
        P-values, all valid under randomization, with or without ties, or conditionally 
        under random sampling from populations, given the observed tie pattern.  Except for 
        Steel's test and the JT test it also combines these tests across several blocks of 
	samples.  Also analyzed are 2 x t contingency tables and their blocked combinations 
	using the Kruskal-Wallis criterion.  Steel's test is inverted to provide simultaneous 
	confidence bounds for shift parameters.  A plotting function compares tail probabilities
	obtained under asymptotic approximation with those obtained via simulation or exact 
	calculations.
License: GPL (>= 2)
LazyLoad: yes
NeedsCompilation: yes
Packaged: 2025-08-26 19:04:09 UTC; fritz
Repository: CRAN
Date/Publication: 2025-08-26 23:20:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 12:59:38 UTC; unix
Archs: kSamples.so.dSYM

Package: kSamples
Type: Package
Title: K-Sample Rank Tests and their Combinations
Version: 1.2-12
Date: 2025-08-25
Authors@R: c(person("Fritz", "Scholz",role=c("aut","cre"),email =
        "fscholz@u.washington.edu"), person("Angie", "Zhu", role=c("aut"),
        email= "a.zhu.stat@gmail.com"))
Author: Fritz Scholz [aut, cre],
  Angie Zhu [aut]
Maintainer: Fritz Scholz <fscholz@u.washington.edu>
Depends: SuppDists
Imports: methods, graphics, stats
Description: Compares k samples using the Anderson-Darling test, Kruskal-Wallis type tests 
	with different rank score criteria, Steel's multiple comparison test, and the 
        Jonckheere-Terpstra (JT) test. It computes asymptotic, simulated or (limited) exact 
        P-values, all valid under randomization, with or without ties, or conditionally 
        under random sampling from populations, given the observed tie pattern.  Except for 
        Steel's test and the JT test it also combines these tests across several blocks of 
	samples.  Also analyzed are 2 x t contingency tables and their blocked combinations 
	using the Kruskal-Wallis criterion.  Steel's test is inverted to provide simultaneous 
	confidence bounds for shift parameters.  A plotting function compares tail probabilities
	obtained under asymptotic approximation with those obtained via simulation or exact 
	calculations.
License: GPL (>= 2)
LazyLoad: yes
NeedsCompilation: yes
Packaged: 2025-08-26 19:04:09 UTC; fritz
Repository: CRAN
Date/Publication: 2025-08-26 23:20:02 UTC

useDynLib("kSamples", .registration = TRUE)

export(ad.pval,ad.test,ad.test.combined,contingency2xt,contingency2xt.comb)
export(conv,pp.kSamples,qn.test,qn.test.combined)
export(Steel.test,SteelConfInt,djt,pjt,qjt,jt.test)
S3method(print,kSamples)
importFrom("SuppDists","normOrder")
importFrom("graphics","abline","par","plot","text")
importFrom("methods","is")
importFrom("stats","dnorm","integrate","model.response","pchisq",
   "pnorm","predict","qnorm","smooth.spline","uniroot","var")

na.remove <- function(x){
#
# This function removes NAs from a list and counts the total 
# number of NAs in na.total.
# Returned is a list with the cleaned list x.new and with 
# the count na.total of NAs.
#
	na.status <- lapply(x,is.na) # changed sapply to lapply
	k <- length(x)
	x.new <- list()
	na.total <- 0
	for( i in 1:k ){
		x.new[[i]] <- x[[i]][!na.status[[i]]]
		na.total <- na.total + sum(na.status[[i]])
	}
	list(x.new=x.new,na.total=na.total)
}

qn.test <-
function (..., data = NULL,
		test = c("KW","vdW","NS"),
		method=c("asymptotic","simulated","exact"),
		dist=FALSE,Nsim=10000) 
{
#############################################################################
# This function "qn.test" tests whether k samples (k>1) come from a common
# continuous distribution, using the QN rank test. See Lehmann (2006),
# Nonparametrics, Statistical Methods Based on Ranks, Appendix Corollary 10.
# Ties are handled by using average rank scores.
# While the asymptotic P-value is always returned, there is the option 
# to get an estimate based on Nsim simulations or an exact value based 
# on the full enumeration distribution, provided method = "exact" is chosen
# and the number of full enumerations is <= the Nsim specified.
# If the latter is not the case, simulation is used with the indicated Nsim.
# These simulated or exact P-values are appropriate under the continuity 
# assumption or, when ties are present, they are still appropriate
# conditionally on the tied rank pattern, provided randomization took 
# place in allocating subjects to the respective samples, i.e., also
# under random sampling from a common discrete parent population.
#
#
# 
# Inputs:
#       	...:	can either be a sequence of k (>1) sample vectors,
#
#				or a list of k (>1) sample vectors,
#
#                               or y, g, where y contains the concatenated
#				samples and g is a factor which by its levels
#				identifies the samples in y,
#
#                               or a formula y ~ g with y and g as in previous case.
#
#           test:	specifies the ranks scores to be used, averaging the scores
#                   of tied observations. 
#                   test = "KW" uses scores 1:N ( ==> Kruskal-Wallis test)
#                   test = "vdW" uses the van der Waerden scores qnorm(1:N/(N+1))
#                   test = "NS" uses normal scores, expected standard normal order 
#                   statistics, uses function normOrder of package SuppDists.
#                   Other scores could easily be added to this function.
#				
#			method: takes values "asymptotic", "simulated", or "exact".
#					The value "asymptotic" causes calculation of P-values
#            		using the asymptotic chi-square approximation, always done.
#
#					The value "simulated" causes estimation of P-values
#					by randomly splitting the the pooled data into
#					samples of sizes ns[1], ..., ns[k], where
#  					ns[i] is the size of the i-th sample vector,
#					and n = ns[1] + ... + ns[k] is the pooled sample size.
#					For each such random split the QN statistic is 
#					computed. This is repeated Nsim times and the proportions
#					of simulated values >= the actually observed QN value
#					is reported as P-value estimate.
#
#                   The value "exact" enumerates all n!/(ns[1]! * ... * ns[k])
#                   splits of the pooled sample and computes the QN statistic.
#					The proportion of all enumerated QN statistics
# 					that are >= the actually observed QN value
#					is reported as exact (conditional) P-value.
#
#			dist: 	= FALSE (default) or TRUE, TRUE causes the simulated
#					or fully enumerated vector of the QN statstic to be returned
#     				as null.dist.
#
#			Nsim: 	number of simulations to perform, 
#					for method = "exact" to take hold, it needs to be at least
#					equal the number of all possible splits of the pooled
#					data into samples of sizes ns[1], ..., ns[k], where
#  					ns[i] is the size of the i-th sample vector.
#
# When there are NA's among the sample values they are removed,
# with a warning message indicating the number of NA's.
# It is up to the user to judge whether such removals make sense.
#
# An example:
# z1 <- c(0.824, 0.216, 0.538, 0.685)
# z2 <- c(0.448, 0.348, 0.443, 0.722)
# z3 <- c(0.403, 0.268, 0.440, 0.087)
# qn.test(z1,z2,z3,method="exact",dist=T,Nsim=100000)
# or 
# qn.test(list(z1,z2,z3),test="KW",method="exact",dist=T,Nsim=100000)
# which produces the output below.
#############################################################################
#
# Kruskal-Wallis  k-sample test.
# 
# Number of samples:  3
# Sample sizes: 4 4 4
# Total number of values: 12
# Number of unique values: 12
# 
# Null Hypothesis: All samples come from a common population.
# 
#               QN  asympt. P-value    exact P-Value 
#        3.5769231        0.1672172        0.1729870 
# 
# 
# Warning: At least one sample size is less than 5.
# asymptotic p-values may not be very accurate.
#
#############################################################################
# In order to get the output list, call 
# qn.out <- qn.test(list(z1,z2,z3),test="KW",method="exact",dist=T,Nsim=100000)
# then qn.out is of class ksamples and has components 
# > names(qn.out)
# [1] "test.name" "k"         "ns"        "N"         "n.ties"    "qn"       
# [7] "warning"   "null.dist" "method"    "Nsim"    
#
# where
# test.name = "Kruskal-Wallis", "van der Waerden", or "normal scores"
# k = number of samples being compared
# ns = vector of the k sample sizes ns[1],...,ns[k]
# N = ns[1] + ... + ns[k] total sample size
# n.ties = number of ties in the combined set of all n observations
# qn =  2 (or 3) vector containing the QN statistics, its asymptotic P-value,
#      	(and its exact or simulated P-value). 
# warning = logical indicator, warning = TRUE indicates that at least  
#		one of the sample sizes is < 5.    
# null.dist is a vector of simulated values of the QN statistic
# 		or the full enumeration of such values.
#		This vector is given when dist = TRUE is specified, 
# 		otherwise null.dist = NULL is returned.
# method = one of the following values: "asymptotic", "simulated", "exact"
# 			as it was ultimately used.
# Nsim = number of simulations used, when applicable.
#
# The class ksamples causes qn.out to be printed in a special output
# format when invoked simply as: > qn.out
# An example was shown above.
#
# Fritz Scholz, August 2012
#
#################################################################################

ave.score <- function(z, scores){
# This function takes a data vector z and a vector scores
# of same length and returns a vector av.scores of scores 
# using average scores for each group of tied 
# observations in z. av.scores and scores have same length.
   N <- length(z)
   rz <- rank(z)
   r.rz <- rank(z,ties.method="random")
   rz.u <- unique(rz)
   av.scores <- rep(0,N)
   for(rz.ui in rz.u){
      av.scores[rz==rz.ui] <- mean(scores[r.rz[rz
                                      ==rz.ui]])
   }
  av.scores
}
	samples <- io(...,data = data)
	test <- match.arg(test)
	method <- match.arg(method)
	out <- na.remove(samples)
	na.t <- out$na.total
	if( na.t > 1) print(paste("\n",na.t," NAs were removed!\n\n"))
	if( na.t == 1) print(paste("\n",na.t," NA was removed!\n\n"))
	samples <- out$x.new
	k <- length(samples)
	if (k < 2) stop("Must have at least two samples.")
	ns <- sapply(samples, length)
	n <- sum(ns)
	if (any(ns == 0)) stop("One or more samples have no observations.")
	x <- unlist(samples)
	if(test == "KW"){ scores.vec <- 1:n }
	if (test == "NS") {
		if (!requireNamespace("SuppDists", quietly = TRUE)){
		# if (!exists("normOrder")) library(SuppDists)
			stop("SuppDists (>= 1.1-9.4) needed for this function to work. Please install it.",
				call. = FALSE)
		}
		scores.vec <- normOrder(n)
	}
	if(test == "vdW") {
		scores.vec <- qnorm((1:n)/(n + 1))
	}  
	QNobs <- 0
	pval <- 0
	rx <- ave.score(x,scores.vec)
	svar <- var(rx)
	smean <- mean(rx)
	L <- length(unique(rx))
	if(dist == TRUE) Nsim <- min(Nsim,1e8) 
	ncomb <- 1
	if( method == "exact"){
		np <- n
    		for(i in 1:(k-1)){
			ncomb <- ncomb * choose(np,ns[i])
        		np <- np-ns[i]
		}
	# it is possible that ncomb overflows to Inf
		if(!(ncomb < Inf)) stop('ncomb = Inf, method = "exact" not possible\n')
	}

	if( method == "exact" & Nsim < ncomb) {
		method <- "simulated"
	}
	if( method == "exact" & dist == TRUE ) nrow <- ncomb
   	if( method == "simulated" & dist == TRUE ) nrow <- Nsim

	if( method == "simulated" ) ncomb <- 1 # don't need ncomb anymore
	if(method == "asymptotic"){
		Nsim <- 1
		dist <- FALSE
	}
	useExact <- FALSE
	if(method == "exact") useExact <- TRUE
	if(dist==T){
		QNvec <- numeric(nrow)
	}else{
    		QNvec <- 0
	}	
	out <- .C("QNtest", pval=as.double(pval),
		Nsim=as.integer(Nsim), k=as.integer(k), 
		rx=as.double(rx), ns=as.integer(ns), 
            	useExact=as.integer(useExact),
		getQNdist=as.integer(dist),
		ncomb=as.double(ncomb),QNobs=as.double(QNobs),
		QNvec = as.double(QNvec), PACKAGE = "kSamples")
	QNobs <- (out$QNobs - n*smean^2)/svar
	pval <- out$pval
	if(dist){
		QNvec <- round((out$QNvec- n*smean^2)/svar,8)
	}
	pval.asympt <- 1-pchisq(QNobs,k-1)
	if(method=="asymptotic"){
		qn <- c(QNobs,pval.asympt)
	}else{
		qn <- c(QNobs,pval.asympt,pval)
	}
	if(method=="asymptotic"){
		names(qn) <- c("test statistic"," asympt. P-value")
	}
	if(method=="exact"){
		names(qn) <- c("test statistic"," asympt. P-value","exact P-Value")
	}
	if(method=="simulated"){
		names(qn) <- c("test statistic"," asympt. P-value","sim. P-Value")
	}
	warning <- FALSE
	if(min(ns) < 5) warning <- TRUE
	if(dist == FALSE | method == "asymptotic") QNvec <- NULL
	if(test == "vdW") test.name <- "van der Waerden scores"
	if(test == "NS") test.name <- "normal scores"
	if(test == "KW") test.name <- "Kruskal-Wallis"
	object <- list(test.name = test.name,
		k = k, ns = ns, N = n, n.ties = n - L,
		qn = qn, warning = warning, null.dist = QNvec,
		method=method, Nsim=Nsim)
    	class(object) <- "kSamples"
    	object
}

Steel.test <-
function (..., data = NULL, method=c("asymptotic","simulated","exact"),
		alternative = c("greater","less","two-sided"), continuity.corr = TRUE,
		dist=FALSE,Nsim=10000) 
{
#############################################################################
# This function "Steel.test" tests whether k-1 samples (k>1) come from the
# same continuous distribution as the control sample, taking as test statistic
# the maximum standardized Wilcoxon test statistic, or the
# minimum standardized Wilcoxon test statistic, or the
# maximum absolute standardized Wilcoxon test statistic,
# (the Steel statistic) for all two sample 
# comparisons with the control sample. See Lehmann (2006),
# Nonparametrics, Statistical Methods Based on Ranks, Chap. 5.5.
# While the asymptotic P-value is always returned, there is the option 
# to get an estimated P-value based on Nsim simulations or an exact P-value
# value based on the full enumeration distribution, provided method = "exact" 
# is chosen and the number of full enumerations is <= Nsim, as specified.
# The latter makes sure that the user is aware of the computation effort involved.
# If the number of full enumerations is > Nsim, simulation is used with the 
# indicated Nsim.
# These asymptotic, simulated or exact P-values are appropriate under the 
# continuity assumption or, when ties are present, they are still appropriate
# conditionally given the tied rank pattern, provided randomization took 
# place in allocating subjects to the respective samples, i.e., also
# under random sampling from a common discrete parent population.
#
#
# 
# Inputs:
#       	...:	can either be a sequence of k (>1) sample vectors,
#
#				or a list of k (>1) sample vectors,
#
#                               or y, g, where y contains the concatenated
#				samples and g is a factor which by its levels
#				identifies the samples in y,
#
#                               or a formula y ~ g with y and g as in previous case.
#				
#		method: takes values "asymptotic", "simulated", or "exact".
#			The value "asymptotic" causes calculation of P-values based
#                       on the multivariate normal approximation for the correlated 
#                       rank sums. This is always done, but when "asymptotic" is
#                       is specified, that is all that's done. Useful for large samples,
#                  	or when a fast P-value assessment is desired.
#
#			The value "simulated" causes estimation of P-values
#			by randomly splitting the pooled data into
#			samples of sizes ns[1], ..., ns[k], where
#  			ns[i] is the size of the i-th sample vector,
#			and n = ns[1] + ... + ns[k] is the pooled sample size.
#			For each such random split the Steel statistic is 
#			computed. This is repeated Nsim times and the proportions
#		      	of simulated values exceeding the actually observed Steel value
#			in the appropriate direction is reported as P-value estimate.
#
#                   	The value "exact" enumerates all ncomb = n!/(ns[1]! * ... * ns[k]!)
#                   	splits of the pooled sample and computes the Steel statistic.
#			The proportion of all enumerated Steel statistics exceeding
# 			the actually observed Steel statistic value in the appropriate 
#			direction is reported as exact (conditional) P-value.
#   			This is only done when ncomb <= Nsim.
#
#		alternative: takes values "greater", "less", and "two-sided".
#
#                     	For the value "greater" the maximum standardized treatment 
#			rank sum is used as test statistic, using conditional means and
# 			standard deviations given the overall tie pattern among all
#			n observations for standardization. The test rejects for 
#			large values of this maximum.
#
#                     	For the value "less" the minimum standardized treatment 
#			rank sum is used as test statistic. The test rejects 
#                       for low values of minimum.
#
#                     	For the value "two-sided" the maximum absolute standardized 
#			treatment rank sum is used as test statistic. The test rejects 
#                       for large values of this maximum.
#
#   continuity.corr:   = TRUE (default) a step .25 or .5 is added/subtracted to the observed 
#                      Mann-Whitney statistic depending on existing or no ties in the 
#                      compared samples.
#
#		dist: 	= FALSE (default) or TRUE, TRUE causes the simulated
#			or fully enumerated vector of the Steel statistic 
#			to be returned as null.dist. TRUE should be used 
#			judiciously, keeping in mind the size ncomb or Nsim
#			of the returned vector.
#
#		Nsim: 	number of simulations to perform, 
#
#
# When there are NA's among the sample values they are removed,
# with a warning message indicating the number of NA's.
# It is up to the user to judge whether such removals make sense.
#
# An example: using the data from Steel's paper on the effect of 
# birth conditions on IQ
# z1 <- c(103, 111, 136, 106, 122, 114)
# z2 <- c(119, 100,  97,  89, 112,  86)
# z3 <- c( 89, 132,  86, 114, 114, 125)
# z4 <- c( 92, 114,  86, 119, 131,  94)
#set.seed(27)
#Steel.test(z1,z2,z3,z4, method = "simulated", 
#         alternative = "less", Nsim = 100000)
#or
#Steel.test(list(z1,z2,z3,z4), method = "simulated", 
#         alternative = "less", Nsim = 100000)
#
#produces the output below.
#
#Steel Multiple Comparison Wilcoxon Test:
#k treatments against a common control (1st sample)
#
#Steel.test(list(z1,z2,z3,z4), method = "simulated", alternative = "less", Nsim = 100000,dist=TRUE)
#
#Number of samples:  4
#Sample sizes:  6, 6, 6, 6
#Number of ties: 7
#
#Null Hypothesis: All samples come from a common population.
#
#Based on Nsim = 1e+05 simulations
#
#  test statistic  asympt. P-value     sim. P-Value 
#       -1.771300         0.094596         0.104740 
#
#
#############################################################################
# In order to get the output list, call 
# st.out <- Steel.test(list(z1,z2,z3,z4),method="simulated",
#   alternative="less",Nsim=100000)
# then st.out is of class ksamples and has components 
# names(st.out)
# > names(st.out)
# [1] "test.name" "k"         "ns"        "N"         "n.ties"    "st"       
# [7] "warning"   "null.dist" "method"    "Nsim"      "mu"        "sig0"     
# [13] "sig"       "tau"       "W"  
#
# where
# test.name = "Steel"
# k = number of samples being compared, including the control sample
# alternative = "greater","less", or "twosided")
# ns = vector of the k sample sizes ns[1],...,ns[k]
# N = ns[1] + ... + ns[k] total sample size
# n.ties = number of ties in the combined set of all N observations
# st =  2 (or 3) vector containing the Steel statistics, its asymptotic P-value,
#      	(and its exact or simulated P-value). 
# warning = logical indicator, warning = TRUE indicates that at least  
#		one of the sample sizes is < 5.    
# null.dist is a vector of simulated values of the Steel statistic
# 		or the full enumeration of such values.
#		This vector is given when dist = TRUE is specified, 
# 		otherwise null.dist = NULL is returned.
# method = one of the following values: "asymptotic", "simulated", "exact"
# 			as it was ultimately used.
# Nsim = number of simulations used, when applicable.
# W = vector of Mann-Whitney statistics W.X1X2, ..., W.X1Xk
# Wstand = (W-mu)/tau
# pval.asympt.adj = adjusted asymptotic p-values based on the asymptotic distribution of the 
#      Steel statistic.
# Wstand = vector of standardized W, 
# pval.asympt.adj = adjusted p-values for all observed Wstand, based on normal approximation 
#                   to the Steel distribution.
# pval.adj = adjusted p-values for all observed Wstand, based on simulated or 
# exact Steel distribution, if present.
#     The smallest of pval.asympt.adj and pval.adj serves also as overall p-value 
#     for the Steel test.
# mu = the vector of means for the Mann-Whitney statistics W.XY
# tau = vector of standard deviations of  W.X1X2, ..., W.X1Xk
# sig0 = standard deviation of V.0, when W.X1Xi are viewed as n.i^2 * V.0 + V.i
#        with V.0, V.1, ..., V.(k-1) independent with means zero.
# sig = vector of standard deviations ofV.1, ..., V.(k-1)
#    all these means and standard deviations are conditional on the tie
#    pattern and are either used in the standardization of the W.X1Xi
#    and/or in the normal approximation. 
# The class ksamples causes st.out to be printed in a special output
# format when invoked simply as: > st.out
# An example was shown above.
#
# Fritz Scholz, August 2025
#
#################################################################################


samples <- io(..., data = data)

method <- match.arg(method)
alternative <- match.arg(alternative)
if(alternative=="greater") alt <- 1
if(alternative=="less") alt <- -1
if(alternative=="two-sided") alt <- 0
out <- na.remove(samples)
na.t <- out$na.total
if( na.t > 1) print(paste("\n",na.t," NAs were removed!\n\n"))
if( na.t == 1) print(paste("\n",na.t," NA was removed!\n\n"))
samples <- out$x.new
k <- length(samples)
if (k < 2) stop("Must have at least two samples.")
ns <- sapply(samples, length)



n <- sum(ns)
if (any(ns == 0)) stop("One or more samples have no observations.")
x <- unlist(samples)

# reverse the sign of x to change testing alt = -1 to alt = 1
if(alt==-1){
	x <- -x
	alt <- 1
}

Wvec <- numeric(k-1)
corr <- numeric(k-1)
xst <- 0
for(i in 2:k){
xst <- xst + ns[i-1]
Wvec[i-1] <- sum( rank( c(x[1:ns[1]],x[xst+1:ns[i]]) )[ns[1]+1:ns[i]])-ns[i]*(ns[i]+1)/2
if(length(unique(c(x[1:ns[1]],x[xst+1:ns[i]]))) < ns[1]+ns[i]){
	corr[i-1] <- .25
	}else{
	corr[i-1] <- .5
	}
}
Steelobs <- 0
pval <- 0
rx <- rank(x)




dvec <- as.vector(table(rx))
d2 <- sum(dvec*(dvec-1))
d3 <- sum(dvec*(dvec-1)*(dvec-2))
n2 <- n*(n-1)
n3 <- n2*(n-2)
n0 <- ns[1]
ni <- ns[-1]
sig02 <- (n0/12)*(1-d3/n3)
sig0 <- sqrt(sig02)
sig2 <- (n0*ni/12)*(n0+1-3*d2/n2-(n0-2)*d3/n3)
sig <- sqrt(sig2)
tau <- sqrt(ni^2 * sig02 + sig2)
mu <- n0*ni/2
L <- length(unique(rx))
# computing total number of combination splits
if(dist == TRUE) Nsim <- min(Nsim,1e8) 
# limits the size of null.dist
# whether method = "exact" or = "simulated"
ncomb <- 1
np <- n
for(i in 1:(k-1)){
	ncomb <- ncomb * choose(np,ns[i])
      	np <- np-ns[i]
}
# it is possible that ncomb overflows to Inf
if( method == "exact" & Nsim < ncomb) {
	method <- "simulated"
}
if( method == "exact" & dist == TRUE ) nrow <- ncomb
if( method == "simulated" & dist == TRUE ) nrow <- Nsim
if(method == "asymptotic"){
	Nsim <- 1
	dist <- FALSE
}
useExact <- FALSE
if( method == "exact") useExact <- TRUE
if(useExact){nrow <- ncomb}
if(dist==TRUE){
	Steelvec <- numeric(nrow)
}else{
    	Steelvec <- 0
}

out <- .C("Steeltest", pval=as.double(pval),
		Nsim=as.integer(Nsim), k=as.integer(k), 
		rx=as.double(rx), ns=as.integer(ns), 
            	useExact=as.integer(useExact),
		getSteeldist=as.integer(dist),
		ncomb=as.double(ncomb), alt=as.integer(alt),
                mu=as.double(mu), tau=as.double(tau),
		Steelobs=as.double(Steelobs),
		Steelvec = as.double(Steelvec), PACKAGE = "kSamples")
Steelobs <- out$Steelobs
# changes Steelobs back to make up for earlier sign change
if(alternative == "less"){ 
	Steelobs <- -Steelobs
	Wvec <- ns[1]*ns[2:k]-Wvec
	}

Wvec.stand <- (Wvec-mu)/tau
pval <- out$pval
if(dist){
	Steelvec <- out$Steelvec
	if(alternative == "less") Steelvec <- - Steelvec
}
if(dist == F | method == "asymptotic") Steelvec <- NULL
pval.asympt.adj <- Steelnormal(mu,sig0,sig,tau,Wvec,ni,alternative,continuity.corr,corr)
pval.asympt <- min(pval.asympt.adj)
if(method=="asymptotic"){
	st <- c(Steelobs,pval.asympt)
}else{
	st <- c(Steelobs,pval.asympt,pval)
}
if(method=="asymptotic"){
	names(st) <- c("test statistic"," asympt. P-value")
}
if(method=="exact"){
	names(st) <- c("test statistic"," asympt. P-value","exact P-Value")
}
if(method=="simulated"){
	names(st) <- c("test statistic"," asympt. P-value","sim. P-Value")
}
warning <- FALSE
if(min(ns) < 5) warning <- TRUE
if(dist == FALSE) null.dist <- NULL
pval.adj <- NULL
if(method == "simulated" | method == "exact"){
    pval.adj <- numeric(k-1)
    if(alternative =="two-sided"){   Wvec.standa <- abs(Wvec.stand)
    				      Steelveca <- abs(Steelvec)
    		                     for(j in 1:(k-1)){
    			                pval.adj[j] <- mean(Steelveca >= Wvec.standa[j])
    		                     }
    }			            
    if(alternative == "less"){
                                    for(j in 1:(k-1)){
    			                pval.adj[j] <- mean(Steelvec <= Wvec.stand[j])
    		              }
    }
    if(alternative == "greater"){ 
    		                     for(j in 1:(k-1)){
    			                pval.adj[j] <- mean(Steelvec >= Wvec.stand[j])
    		                     }
    		                 }      
}

object <- list(test.name = "Steel", k = k, alternative = alternative,
		ns = ns, N = n, n.ties = n - L,
		st = st, warning = warning, null.dist = Steelvec,
		method=method, Nsim=Nsim, W=Wvec, Wstand = Wvec.stand, pval.asympt.adj = pval.asympt.adj,
		pval.adj = pval.adj, mu=mu, tau=tau, sig0=sig0, sig=sig)
    class(object) <- "kSamples"
    object

}

io2 <- function(...,data=NULL) {
# ... can be a sequence of lists of numeric sample vectors 
# with at least 2 sample vectors per list.
# Each list corresponds to a block level, and the samples
# within a block correspond to different (treatment) 
# group levels, although the same group levels (treatments) 
# may be used within different blocks.
# Or ... can be a list of such lists.
# Or ... can be a formula that specifies a
# a response (e.g., y), grouped by a treatment factor, e.g., g, 
# with a blocking factor, e.g., b, via formula y ~ g | b. 
# The variables y, g, b may be names of columns in a supplied
# data frame dat via data = dat (default NULL).
# When data = NULL is used the variables y, g, b should exist 
# in the calling environment.
# This breaks down y into blocks of responses via the block
# levels and within each block the responses are broken down into 
# different samples corresponding to the different levels of g.
# Or ... can be a data.frame with three column y, g, and b, 
# as described above.
xlist <- list(...)
cl <- match.call()  # gets a copy of the current call
if(is(xlist[[1L]], "formula")) {
	clstr <- unlist (strsplit(as.character(eval(cl[[2]])),NULL))
	if ("~" %in% clstr & "|" %in% clstr) {
			
			Y <- 	eval(as.name(strsplit(
				as.character(eval(cl[[2]]))," ")[[2]]),
				envir = data)
			G <- eval(as.name(strsplit(
				as.character(eval(cl[[2]]))," ")[[3]][1]),
				envir = data)
			B <- eval(as.name(strsplit(
				as.character(eval(cl[[2]]))," ")[[3]][3]),
				envir = data)
			data.sets <- listmake(
						Y,as.factor(G),as.factor(B))
	}
} else {	if(length(xlist) == 1 && is.list(...[[1]])){	
			data.sets <- list(...)[[1]]
		} else {
			if( length(list(...)) > 1 && 
				is.list(list(...)[[1]])){
				data.sets <- list(...)
			} 
		}
 	}
b <- length(data.sets)
if(b < 2) stop("less than 2 blocks\n")
num <- NULL
for( i in 1:b ){
	num <- c(num,unlist(lapply(data.sets[[i]],FUN=is.numeric)))
        if(length(data.sets[[i]]) < 2) stop("at least one block with less than 2 samples\n")
}
if( all(num) == FALSE ) stop("improper data in ... \n")
data.sets
}

Steelnormal <- function(mu,sig0,sig,tau,Wvec,ni,
		alternative=c("greater","less","two-sided"),
		continuity.corr=TRUE,corr){
#
# This function evalutates the normal approximation significance probability
# of the Steel test statistic for any of the Mann-Whitney statistics in k comparisons 
# for each of k treatment samples Y with a common control sample X, based on the 
# mutiple comparison Steel test distribution,thus giving the adjusted p-values for 
# each of the Mann-Whitney statistics.
# The minimum of these adjusted p-values then serves as the overall p-value of the 
# Steel test.
# Wvec contains the k Mann-Whitney statistics, 
# mu,sig0,sig,tau contain parameters required for the normal approximation and 
# ni contains the sizes of the treatment samples.
#
    alternative <- match.arg(alternative)
    k <- length(ni)
    pval <- numeric(k)
    if(continuity.corr == FALSE) corr <- corr*0
    if (alternative == "greater") {
        for (j in 1:k) {
            S <- (Wvec[j] -corr[j] - mu[j])/tau[j]
            funz <- function(z, k, sig0, sig, tau, S, ni) {
                fac <- 1
                for (i in 1:k) {
                  fac <- fac * pnorm((S * tau[i] - ni[i] * sig0 *
                    z)/sig[i])
                }
                dnorm(z) * fac
            }
            pval[j] <- 1 - integrate(funz, -10, 10, k, sig0,
                sig, tau, S, ni)$value
        }
    }
    if (alternative == "less") {
        for (j in 1:k) {
            S <- (Wvec[j] + corr[j]- mu[j])/tau[j]
            funz <- function(z, k, sig0, sig, tau, S, ni) {
                fac <- 1
                for (i in 1:k) {
                  fac <- fac * (1 - pnorm((S * tau[i] - ni[i] *
                    sig0 * z)/sig[i]))
                }
                dnorm(z) * fac
            }
            pval[j] <- 1 - integrate(funz, -10, 10, k, sig0,
                sig, tau, S, ni)$value
        }
    }
    if (alternative == "two-sided") {
        for (j in 1:k) {
            S <- (abs(Wvec[j] -corr[j] - mu[j]))/tau[j]
            funz <- function(z, k, sig0, sig, tau, S, ni) {
                fac <- 1
                for (i in 1:k) {
                  fac <- fac * (pnorm((S * tau[i] - ni[i] * sig0 *
                    z)/sig[i]) - pnorm((-S * tau[i] - ni[i] *
                    sig0 * z)/sig[i]))
                }
                dnorm(z) * fac
            }
            pval[j] <- 1 - integrate(funz, -10, 10, k, sig0,
                sig, tau, S, ni)$value
        }
    }
    pval
}
# end of Steelnormal

qn.test.combined <-
function (...,data = NULL, test = c("KW","vdW","NS"),
	method=c("asymptotic","simulated","exact"),
	dist=FALSE,Nsim=10000) 
{
#############################################################################
# This function qn.test.combined combines several QN K-sample test statistics 
# QN.m, m = 1,...,M, into one overall test 
# statistic QN.combined as suggested for the Kruskal-Wallis test in 
# Lehmann, E.L. (2006), Nonparametrics, Statistical Methods Based on Ranks, 
# Ch. 6, Sec. 5D.
# See also the documentation of qn.test for the comparison of a single 
# set of K samples.
# Each application of the QN K-sample test can be based on a different K > 1. 
# This combined version tests the hypothesis that all the hypotheses 
# underlying the individual K-sample tests are 
# true simultaneously.
# The individual K-sample test hypothesis is that all samples from 
# the m-th group come from a common population. However, that population 
# may be different from one individual K-sample situation to the next. 
# Such a combined test is useful in 
#
# 1) examining intra-laboratory measurement equivalence, when samples from 
#    the same material or batch are compared for M laboratories and such
#    comparisons are made for samples from several different materials or
#    batches and one assessment over all materials/batches is desired.
#
# 2) analyzing treatment effects in randomized complete or incomplete 
#    block designs.
#
# When there are NA's among the sample values they are removed,
# with a warning message indicating the number of NA's.
# It is up to the user to judge whether such removals make sense.
#
# Input: ...  
#        can take the form of several lists, 
#        say L.1,...,L.M, where list L.i contains 
#        K.i sample vectors of respective sizes n.i[1], ..., n.i[K.i] 
#        (n.i[j] > 4 is recommended)
#
#        or a single list of such lists
#
#        or a data frame with first column representing the responses y,
#           the second column a factor g the levels of which are used to 
#           indicate the samples within each block, and the third column 
#           a factor b indicating the block
#
#        or a formula y ~ g | b with y, g, b as in the previous situation
#
#        or just the three vectors y, g, b in this order with same meaning.
#
# 		test:	specifies the ranks scores to be used, averaging the scores
#               of tied observations. 
#               test = "KW" uses scores 1:N ( ==> Kruskal-Wallis test)
#               test = "vdW" uses the van der Waerden scores qnorm(1:N/(N+1))
#               test = "NS" uses normal scores, expected standard normal order 
#               statistics, uses function normOrder of package SuppDists.
#               Other scores could easily be added to this function and
#               to qn.test.
#
#		method 
#       can take one of three values "asymptotic", "simulated",
#		and "exact", which determines the mode of P-value calculation.
#       The asymptotic P-value based on the chi-square approximation
#       is always returned.
#       The simulated P-value simulates splits of the pooled samples in 
#		the i-th list L.i into K.i samples of sizes n.i[1], ..., n.i[K.i],
#       computing the corresponding QN statistic QN.i,
#       doing this independently for i = 1,...,M and adding the QN.i
#       to get the combined statistic QN.comb. This is repeated Nsim 
#       times and the simulated P-value is the proportion of these 
#       values that are >= the observed combined value.
#       The exact P-value should only be attempted for small M and small
#       sample sizes and requires that Nsim be set to >= the total number
#       of QN.comb enumerations. Otherwise Nsim simulations are run
#       to get a simulated P-value, as described above.
#       As example consider: M=2 with K.1 = 2, n.1[1] = 5, n.1[2] = 6,
#		K.2 = 2, n.2[1] = 5, n.2[2] = 7, then we would have
#       choose(11,5) = 462 splits of the first set of pooled samples
#       and choose(12,5) = 792 splits of the second set of pooled samples
#       and thus 462 * 792 = 365904 combinations of QN.1+QN.2 = QN.comb.
#       Thus we would need to set Nsim >= 365904 to enable exact 
#       exact enumeration evaluation of the P-value. Since these enumerated
# 		values of QN.comb need to be held inside R in a single vector,
#  		we are limited by the object size in R. In simulations the length
#       of the simulated vector of QN.comb is only Nsim and is manageable.
#
#       dist
#       takes values FALSE (default) or TRUE, where TRUE enables the 
#	return of the simulated or exact vectors of generated values
#       of QN.comb.
#       Otherwise NULL is returned for both versions
#
#       Nsim 
#       = 10000 (default), number of simulations as discussed above.
#       
#       
#
# An example:
# x1 <- c(1, 3, 2, 5, 7)
# x2 <- c(2, 8, 1, 6, 9, 4) 
# x3 <- c(12,  5,  7,  9, 11)
# y1 <- c(51, 43, 31, 53, 21, 75) 
# y2 <- c(23, 45, 61, 17, 60)
# then 
# set.seed(2627)
# qn.test.combined(list(x1,x2,x3),list(y1,y2),test="KW", method="simulated",Nsim=100000) 
# or equivalently qn.test.combined(list(list(x1,x2,x3),list(y1,y2)),
#	test="KW",method="simulated",Nsim=100000)
# produces the outcome below.
##########################################################################
# 
# Combination of Kruskal-Wallis K-Sample Tests.
# 
# Number of data sets = 2 
# 
# Sample sizes within each data set:
# Data set 1 :  5 6 5
# Data set 2 :  6 5
# Total sample size per data set: 16 11 
# Number of unique values per data set: 11 11 
# 
# Null Hypothesis:
# All samples within a data set come from a common distribution.
# The common distribution may change between data sets.
# 
# Based on Nsim = 1e+05 simulations
# for data set 1 we get
#               QN  asympt. P-value     sim. P-Value 
#       5.64851852       0.05935261       0.05156000 
# 
# for data set 2 we get
#               QN  asympt. P-value     sim. P-Value 
#       0.03333333       0.85513214       0.93001000 
# 
# Combined Kruskal-Wallis Criterion: QN.combined = QN.1+QN.2 
# 
# Based on Nsim = 1e+05 simulations
#          QN.comb  asympt. P-value     sim. P-Value 
#        5.6818519        0.1281575        0.1216000 
#
###############################################################################
# For out.qn.combined <- qn.test.combined(list(x1,x2,x3),list(y1,y2))
# or out.qn.combined <- qn.test.combined(list(list(x1,x2,x3),list(y1,y2)))
# we get the object out.qn.combined of class ksamples with the following 
# components
# > names(qn.combined.out)
# [1] "test.name" "M"         "n.samples" "nt"        "n.ties"    "qn.list"  
# [7] "qn.c"      "warning"   "null.dist" "method"    "Nsim" 
# where 
# test.name 	"Kruskal-Wallis", "van der Waerden scores", or "normal scores"
# M 			number of sets of samples being compared
# n.samples		= is a list of the vectors giving the sample sizes for each 
#             	set of samples being compared
# nt  			vector of total sample sizes involved in each of the M comparisons
# n.ties  		vector of lenth M giving the number of ties in each comparison group
# qn.list 	 	list of M data frames for the results for each of the test results
#           	corresponding to the M block
# qn.c  		2 (or 3) vector containing QN.obs, asymptotic P-value, 
#          		(and simulated or exact P-value) for the combined test.
#         		Here qn.obs is the observed value of QN.comb and 
#				P-value = P(QN.comb >= qn.obs).
# warning  	logical indicator, warning = TRUE when at least one of 
#           	the sample sizes is < 5.
# null.dist 	vector of simulated or fully enumerated QN statistics, if requested,
#               otherwise it is NULL
# method  		one of the following values: "asymptotic", "simulated", "exact"
# 				as it was ultimately used.
# Nsim  		number of simulations used, when applicable.
# Nsim
#
# Fritz Scholz, April 2012
#####################################################################################
convvec <- function(x1,x2){
#----------------------------------------------------
# R routine for calling convvec in conv.c
# created 02.05.2012  Fritz Scholz
#----------------------------------------------------
n1 <- length(x1)
n2 <-length(x2)
n <- n1*n2
x <- numeric(n)
 out <- .C("convvec",  x1=as.double(x1), n1=as.integer(n1), 
					x2=as.double(x2),n2=as.integer(n2), 
					x=as.double(x),n=as.integer(n), PACKAGE = "kSamples")
out$x
}


# the following converts individual data sets into a list of such,
# if not already in this form. It drops blocks (sublist) with at
# most one sample in it.
data.sets <- io2(...,data = data)
data.sets <- test.list(data.sets)
# end of data.sets list conversion
test <- match.arg(test)
method <- match.arg(method)
n.sizes <- NULL
M <- length(data.sets) # number of data sets

n.data <- sapply(data.sets, length) 
		# gets vector of number of samples in each component of data.sets
n.samples <- list() # intended to contain vectors of sample sizes for 
                    # for each respective data set.
na.t <- 0 # intended to tally total of NA cases
ncomb <- 1 # intended to hold the total number of evaluations 
           # of the full convolution distribution, to check
           # whether exact computation is reasonable. 

for(i in 1:M){
	out <- na.remove(data.sets[i])
	na.t<- na.t + out$na.total
	data.sets[i] <- out$x.new # replace data set i with the one that has
                              # NA's removed
	n.sample <- sapply(data.sets[[i]], length) 
				# contains sample sizes for i-th data set
	n.sizes <- c(n.sizes, n.sample) 
				# accumulates all sample size, warning purpose only
	if(any(n.sample==0))
		stop(paste("one or more samples in data set", i,
                       "has no observations"))
	n.samples[[i]] <- n.sample 
	N <- sum(n.sample) # total observations in i-th data set
    k <- length(n.sample) # number of samples in i-th data set

    # updating ncomb
   	ncomb <- ncomb * choose(N,n.sample[1])
	for(j in 2:k){
		N <- N-n.sample[j-1]
		ncomb <- ncomb * choose(N,n.sample[j])
	} # end of ncomb update for data set i
}
if(ncomb > Nsim & method == "exact") method <- "simulated"	

if( na.t > 1) cat(paste("\n",na.t," NAs were removed!\n\n"))
if( na.t == 1) cat(paste("\n",na.t," NA was removed!\n\n"))

warning <- min(n.sizes) < 5 # set warning flag for caution on
                            # trusting asymptotic p-values

# Initializing output objects
QNobs <- 0
sig <- NULL
n.ties <- NULL
nt <- NULL
mu <- NULL
qn.list <- list()
mu.c <- 0
null.dist <- NULL
if(method == "asymptotic"){dist0 <- FALSE}else{dist0 <- TRUE}
# the following loop aggregates the (estimated or exact)
# convolution distribution of the combined QN statistic versions
for(i in 1:M){
	out <- qn.test(data.sets[[i]],test=test,method=method,dist=dist0,Nsim=Nsim)
	if(dist0==T){
	    if(i == 1){
			null.dist <- out$null.dist
		}else{
			if(method=="simulated"){
				null.dist <- null.dist+out$null.dist
			}else{
				null.dist <- convvec(null.dist,out$null.dist)
			}
		}
	}
	qn.list[[i]] <- out$qn
	mu <- c(mu, length(data.sets[[i]])-1)
	QNobs <- QNobs + out$qn[1] # aggregates combined QN stats 
	n.ties <- c(n.ties, out$n.ties)
	nt <- c(nt, sum(out$ns)) # accumulates total observations in data sets
}
# get exact or simulated P-value
if(dist0==T){
    nrow <- length(null.dist)
	pval <- sum(null.dist >= QNobs)/nrow
}

# get asymptotic P-value
pval.asympt <- 1-pchisq(QNobs, sum(mu))

    if(method=="asymptotic"){
     	qn.c <- c(QNobs,pval.asympt)
	}else{
		qn.c <- c(QNobs,pval.asympt,pval)
	}
    if(method=="asymptotic"){
		names(qn.c) <- c("comb.statistic"," asympt. P-value")
    }
    if(method=="exact"){
		names(qn.c) <- c("comb.statistic"," asympt. P-value","exact P-Value")
    }
    if(method=="simulated"){
		names(qn.c) <- c("comb.statistic"," asympt. P-value","sim. P-Value")
    }
if(dist==FALSE){
	null.dist <- NULL

}
    if(test == "vdW") test.name <- "van der Waerden scores"
    if(test == "NS") test.name <- "normal scores"
    if(test == "KW") test.name <- "Kruskal-Wallis"


object <- list(test.name =test.name, M = M,
				n.samples=n.samples, nt=nt, n.ties=n.ties, qn.list=qn.list,
           		qn.c = qn.c, warning=warning,null.dist=null.dist,
				method=method,Nsim=Nsim)
class(object) <-  "kSamples"
object
}

ad.test <-
function (..., data = NULL,
  method=c("asymptotic","simulated","exact"),dist=FALSE,Nsim=10000) 
{
#############################################################################
# This function "ad.test" tests whether k samples (k>1) come from a common
# continuous distribution, using the nonparametric (rank) test described in
# Scholz F.W. and Stephens M.A. (1987), K-sample Anderson-Darling Tests,
# Journal of the American Statistical Association, Vol 82, No. 399, 
# pp. 918-924. 
# This test is consistent against all alternatives. 
# Ties are handled by using midranks, and according to the above
# reference two versions of the test statistic are returned.
# They are labeled version 1 and version 2, in the order introduced
# in the above reference.
# While the asymptotic P-value is always returned, there is the option 
# to get an estimate based on Nsim simulations or an exact value based 
# on the full enumeration distribution, provided method = "exact" is chosen
# and the number of full enumerations is <= the Nsim specified.
# If the latter is not the case, simulation is used with the indicated Nsim.
# These simulated or exact P-values are appropriate under the continuity 
# assumption or, when ties are present, they are still appropriate
# conditionally on the tied rank pattern, provided randomization took 
# place in allocating subjects to the respective samples, i.e., also
# under random sampling from a common discrete parent population.
#
#
# 
# Inputs:
#       	...:	can either be a sequence of k (>1) sample vectors,
#
#				or a list of k (>1) sample vectors,
#
#                               or y, g, where y contains the concatenated
#				samples and g is a factor which by its levels
#				identifies the samples in y,
#
#                               or a formula y ~ g with y and g as in previous case.
#
#				
#               data:   data frame with variables usable in formula input, default = NULL.
#
#			method: takes values "asymptotic", "simulated", or "exact".
#					The value "asymptotic" causes calculation of P-values
#            		using the asymptotic approximation, always done.
#
#					The value "simulated" causes estimation of P-values
#					by randomly splitting the the pooled data into
#					samples of sizes ns[1], ..., ns[k], where
#  					ns[i] is the size of the i-th sample vector,
#					and n = ns[1] + ... + ns[k] is the pooled sample size.
#					For each such random split the AD statistics are 
#					computed. This is repeated Nsim times and the proportions
#					of simulated values >= the respective actually 
#					observed AD values are reported as P-value estimates.
#
#                   The value "exact" enumerates all n!/(ns[1]! * ... * ns[k])
#                   splits of the pooled sample and computes the respective 
#					AD statistics. The proportion of all enumerated AD statistics
# 					which are >= the respective actually observed AD values
#					are reported as exact P-values.
#
#			dist: 	= FALSE (default) or TRUE, TRUE causes the simulated
#					or fully enumerated vectors of both AD statstics to be returned
#     					as null.dist1 and null.dist2.
#
#			Nsim: 	number of simulations to perform, 
#					for method = "exact" to take hold, it needs to be at least
#					equal the number of all possible splits of the pooled
#					data into samples of sizes ns[1], ..., ns[k], where
#  					ns[i] is the size of the i-th sample vector.
#
# When there are NA's among the sample values they are removed,
# with a warning message indicating the number of NA's.
# It is up to the user to judge whether such removals make sense.
#
# An example:
# z1 <- c(0.824, 0.216, 0.538, 0.685)
# z2 <- c(0.448, 0.348, 0.443, 0.722)
# z3 <- c(0.403, 0.268, 0.440, 0.087)
# ad.test(z1,z2,z3,method="exact",dist=T,Nsim=100000)
# or 
# ad.test(list(z1,z2,z3),method="exact",dist=T,Nsim=100000) 
# which produces the output below.
#############################################################################
#  Anderson-Darling k-sample test.
#
# Number of samples:  3
# Sample sizes:  4, 4, 4
# Number of ties: 0
#
# Mean of  Anderson-Darling  Criterion: 2
# Standard deviation of  Anderson-Darling  Criterion: 0.88133
#
# T.AD = ( Anderson-Darling  Criterion - mean)/sigma
#
# Null Hypothesis: All samples come from a common population.
#
#                AD    T.AD  asympt. P-value  exact P-value
# version 1: 2.6367 0.72238          0.18525        0.20924
# version 2: 2.6200 0.70807          0.18819        0.21703
#
#
# Warning: At least one sample size is less than 5.
# asymptotic p-values may not be very accurate.
#
#############################################################################
# In order to get the output list, call 
# ad.out <- ad.test(z1,z2,z3,method="exact",dist=T,Nsim=100000)
# then ad.out is of class ksamples and has components 
# > names(ad.out)
#  [1] "test.name"  "k"          "ns"         "N"          "n.ties"    
#  [6] "sig"        "ad"         "warning"    "null.dist1" "null.dist2"
# [11] "method"     "Nsim"
#
# where
# test.name = "Anderson-Darling"
# k = number of samples being compared
# ns = vector of the k sample sizes ns[1],...,ns[k]
# N = ns[1] + ... + ns[k] total sample size
# n.ties = number of ties in the combined set of all n observations
# sig = standard deviation of the AD statistic (for continuous population case)
# ad =  2 x 3 (or 2 x 4) matrix containing the AD statistics, 
#		standardized AD statistics, its asymptotic P-value, 
#      	(and its exact or simulated P-value), for version 1 in the first row 
#		and for version 2 in the second row.
# warning = logical indicator, warning = TRUE indicates that at least  
#		one of the sample sizes is < 5.
# null.dist1 is a vector of simulated values of the AD statistic (version 1)
# 		or the full enumeration of such values.
#		This vector is given when dist = TRUE is specified, 
# 		otherwise null.dist1 = NULL is returned.
# null.dist2 is the corresponding vector for the 2nd AD statistic version.
# method = one of the following values: "asymptotic", "simulated", "exact"
# 			as it was ultimately used.
# Nsim = number of simulations used, when applicable.
#
# The class ksamples causes ad.out to be printed in a special output
# format when invoked simply as: > ad.out
# An example was shown above.
#
# Fritz Scholz, August 2012
#
#################################################################################
	samples <- io(...,data = data)
	method <- match.arg(method)
	out <- na.remove(samples)
	na.t <- out$na.total
	if( na.t > 1) print(paste("\n",na.t," NAs were removed!\n\n"))
	if( na.t == 1) print(paste("\n",na.t," NA was removed!\n\n"))
	samples <- out$x.new
	k <- length(samples)
	if (k < 2) stop("Must have at least two samples.")
	ns <- sapply(samples, length)
	if (any(ns == 0)) stop("One or more samples have no observations.")
	x <- unlist(samples)
	n <- length(x)
	Z.star <- sort(unique(x))
	L <- length(Z.star)
	if(dist == TRUE) Nsim <- min(Nsim,1e8) 
	# limits the size of null.dist1 and null.dist2
	# whether method = "exact" or = "simulated"
	ncomb <- 1
	np <- n
    	for(i in 1:(k-1)){
		ncomb <- ncomb * choose(np,ns[i])
       		np <- np-ns[i]
	}
	# it is possible that ncomb overflows to Inf
	if( method == "exact" & Nsim < ncomb) {
		method <- "simulated"
	}
	if( method == "exact" & dist == TRUE ) nrow <- ncomb
   	if( method == "simulated" & dist == TRUE ) nrow <- Nsim
	if( method == "simulated" ) ncomb <- 1 # don't need ncomb anymore
	if(method == "asymptotic"){
		Nsim <- 1
		dist <- FALSE
	}
	dist1 <- NULL
	dist2 <- NULL
	pv <- c(NA,NA)
	getA2mat <- dist
	useExact <- FALSE
	if(method == "exact") useExact <- TRUE
	if(getA2mat){
		a2mat <- matrix(0,nrow=nrow,ncol=2)}else{
		a2mat <- 0
	}
	ans <- numeric(2)
        pval <- numeric(2)
	out0 <- .C("adkTestStat0",ans=as.double(ans),k=as.integer(k),x=as.double(x),
		ns=as.integer(ns),Z.star=as.double(Z.star),L=as.integer(L), PACKAGE = "kSamples")
	if(method != "asymptotic"){	
		out1 <- .C("adkPVal0",pval=as.double(pval), Nsim=as.integer(Nsim),k=as.integer(k),
				x=as.double(x),ns=as.integer(ns),
				zstar=as.double(Z.star),L=as.integer(L),
				useExact=as.integer(useExact),getA2mat=as.integer(getA2mat),
				ncomb=as.double(ncomb),a2mat=as.double(a2mat), PACKAGE= "kSamples")
	    	pv <- out1$pval
  		if(getA2mat){
    			a2mat <- matrix(out1$a2mat, nrow=nrow, ncol=2, byrow=FALSE, 
			 		dimnames=list(NULL, c("AkN2", "AakNk2")))
    			dist1 <- round(a2mat[,1],8)
    			dist2 <- round(a2mat[,2],8)
		}
	}
	AkN2 <- out0[[1]][1]
	AakN2 <- out0[[1]][2]
	if(n > 3){
		coef.d <- 0
		coef.c <- 0
		coef.b <- 0
		coef.a <- 0
		H <- sum(1/ns)
		h <- sum(1/(1:(n - 1)))
		g <- 0
		for (i in 1:(n - 2)) {
	        	g <- g + (1/(n - i)) * sum(1/((i + 1):(n - 1)))
		}
		coef.a <- (4 * g - 6) * (k - 1) + (10 - 6 * g) * H
		coef.b <- (2 * g - 4) * k^2 + 8 * h * k + (2 * g - 14 * h - 
	       	 4) * H - 8 * h + 4 * g - 6
		coef.c <- (6 * h + 2 * g - 2) * k^2 + (4 * h - 4 * g + 6) * 
	   	     k + (2 * h - 6) * H + 4 * h
		coef.d <- (2 * h + 6) * k^2 - 4 * h * k
		sig2 <- (coef.a * n^3 + coef.b * n^2 + coef.c * n + coef.d)/((n - 
	        1) * (n - 2) * (n - 3))
		sig <- sqrt(sig2)
		TkN <- (AkN2 - (k - 1))/sig
		TakN <- (AakN2 - (k - 1))/sig
		pvalTkN <- ad.pval(TkN, k - 1,1)
		pvalTakN <- ad.pval(TakN, k - 1,2)
	}
	if(n == 3 && k == 3 | n == 2){
		sig <- 0
		TkN <- NA
		TakN <- NA
		pvalTkN <- 1
		pvalTakN <- 1
	}
	if(n == 3 && k == 2){
		sig <- .3535534
		TkN <- (AkN2 - (k - 1))/sig
		TakN <- (AakN2 - (k - 1))/sig
		pvalTkN <- ad.pval(TkN, k - 1,1)
		pvalTakN <- ad.pval(TakN, k - 1,2)
	}
	warning <- min(ns) < 5
	if(method=="asymptotic"){
	     	ad.mat <- matrix(c(signif(AkN2,5), signif(TkN, 5), 
				signif(pvalTkN, 5), signif(AakN2,3) , 
				signif(TakN, 5), signif(pvalTakN, 5)), 
				byrow = TRUE, ncol = 3)
	}else{
  	   	ad.mat <- matrix(c(signif(AkN2,5), signif(TkN, 5), 
				signif(pvalTkN, 5),  signif(pv[1],5),signif(AakN2,3) , 
				signif(TakN, 5), signif(pvalTakN, 5), 
				signif(pv[2],5)), byrow = TRUE, ncol = 4)
	}
    	if(method=="asymptotic"){
		dimnames(ad.mat) <- list(c("version 1:","version 2:"),
						c("AD","T.AD"," asympt. P-value"))
    	}
    	if(method=="exact"){
		dimnames(ad.mat) <- list(c("version 1:","version 2:"),
						c("AD","T.AD"," asympt. P-value"," exact P-value"))
    	}
    	if(method=="simulated"){
		dimnames(ad.mat) <- list(c("version 1:","version 2:"),
						c("AD","T.AD"," asympt. P-value"," sim. P-value"))
    	}
	object <- list(test.name ="Anderson-Darling",
				k = k, ns = ns, N = n, n.ties = n - L, sig = round(sig, 5), 
				ad = ad.mat, warning = warning, null.dist1 = dist1, 
				null.dist2 = dist2, method=method, Nsim=Nsim)
	class(object) <- "kSamples"
	object
}

print.kSamples <-
function (x, ...) 
{
######################################################
#
# This is a print function for objects of class kSamples,
# as they are produced by ad.test, kw.test,
# ad.combined.test, kw.combined.test, contingency2xt,
# contingency2xt.comb, Steel.test, SteelConfInt,
# and JT.test.
#
# Fritz Scholz, August 2015
#
#######################################################
	if(names(x)[2]=="k"){# checking whether the object x 
                         #came from ad.test or qn.test of JT.test
		if(x$test.name=="Steel"){
			cat("\nSteel Multiple Comparison Wilcoxon Test:\nk treatments against a common control (1st sample)\n\n")
		}else{
	    	 	cat(paste("\n\n",x$test.name,"k-sample test.\n"))
		}
	    	cat(paste("\nNumber of samples: ", x$k))
	    	cat("\nSample sizes: ",paste(x$ns,collapse=", "))
	    	cat(paste("\nNumber of ties:", x$n.ties))
	    	if(x$test.name == "Anderson-Darling"){
	    		cat(paste("\n\nMean of ",x$test.name," Criterion:", 
	        		x$k-1))
	    		cat(paste("\nStandard deviation of ",x$test.name," Criterion:", 
	        		x$sig))
	    		cat(paste("\n\nT.AD = (",x$test.name," Criterion - mean)/sigma"))
		}
		if(x$test.name != "Jonckheere-Terpstra" ){
			cat("\n\nNull Hypothesis: All samples come from a common population.\n\n")
		}else{
			cat("\n\nNull Hypothesis: All samples come from a common population.\n")
			cat("Alternative: Samples indicate a positive trend.\n\n")
		}
    		
    		if(x$method=="simulated") cat(paste("Based on Nsim =",x$Nsim,"simulations\n\n"))
    		if(x$test.name == "Anderson-Darling" ){print(signif(x$ad,5))}
		if(x$test.name == "van der Waerden scores" ) print(signif(x$qn,5))
		if(x$test.name == "Kruskal-Wallis" ) print(signif(x$qn,5))
		if(x$test.name == "normal scores" ) print(signif(x$qn,5))
 		if(x$test.name == "Jonckheere-Terpstra" ){
			print(signif(x$JT,5))
		}
		if (x$warning) {
        		cat("\n\nWarning: At least one sample size is less than 5,\n")
			cat("  asymptotic p-values may not be very accurate.\n")
    		}
    		invisible(x)
	}
  	if(names(x)[2]=="M"){# checking whether the object x came from ad.combined.test
   		cat(paste("Combination of",x$test.name,"K-Sample Tests.\n"))
    		cat(paste("\nNumber of data sets =", x$M,"\n"))
    		cat("\nSample sizes within each data set:\n")
    		ns <- NULL
    		k <- length(x$n.samples)
    		d.sets <- paste("Data set",1:k)
    		for(i in 1:k){
      			cat(d.sets[i],": ",x$n.samples[[i]])
       			cat("\n")
    		}
    		if(k>3) AD.name=paste("AD.1","...",paste("AD.",k,sep=""),sep="+")
    		if(k==2)AD.name=paste("AD.1+AD.2")
    		if(k==3)AD.name=paste("AD.1+AD.2+AD.3")
    		if(k>3) QN.name=paste("QN.1","...",paste("QN.",k,sep=""),sep="+")
    		if(k==2)QN.name=paste("QN.1+QN.2")
    		if(k==3)QN.name=paste("QN.1+QN.2+QN.3")
    		cat("Total sample size per data set: ")
    		cat(x$nt,"\n")
    		cat("Number of unique values per data set: ")
    		cat(x$nt-x$n.ties,"\n")
		if(x$test.name=="Anderson-Darling"){ 
    			cat(paste("\nAD.i =",x$test.name,"Criterion for i-th data set\n"))
    			cat("Means:",x$mu,"\n")
    			cat("Standard deviations:", x$sig,"\n")
			cat("\nT.i = (AD.i - mean.i)/sigma.i\n")
		}
    		cat("\nNull Hypothesis:\nAll samples within a data set come from a common distribution.\n")
    		cat("The common distribution may change between data sets.\n\n")
    		if(x$test.name=="Anderson-Darling"){ 
     			nx <- length(x$ad.list)
        		if(x$method=="simulated") cat(paste("Based on Nsim =",x$Nsim,"simulations\n\n"))
	    		for(i in 1:nx){
				cat(paste("for data set",i,"we get\n"))
    				print(signif(x$ad.list[[i]],5))
            			cat("\n")
    			}
			cat("Combined Anderson-Darling Criterion: AD.comb =",AD.name,"\n")
	    		cat("Mean =",x$mu.c,"   Standard deviation =",round(x$sig.c,5),"\n")
    			cat("\nT.comb = (AD.comb - mean)/sigma\n")
	    		cat("\n")
        		if(x$method=="simulated") cat(paste("Based on Nsim =",x$Nsim,"simulations\n\n"))
			ad.c <- x$ad.c
	    		print(signif(ad.c,5))
		}

    		if(x$test.name == "van der Waerden scores" | 
		   x$test.name == "normal scores" |
		   x$test.name == "Kruskal-Wallis" ){ 
    			nx <- length(x$qn.list)
        		if(x$method=="simulated") cat(paste("Based on Nsim =",x$Nsim,"simulations\n\n"))
	   	 	for(i in 1:nx){
				cat(paste("for data set",i,"we get\n"))
    				print(signif(x$qn.list[[i]],5))
            			cat("\n")
    			}
			cat("Combined Criterion: QN.combined =",QN.name,"\n")
			cat("\n")
        		if(x$method=="simulated") cat(paste("Based on Nsim =",x$Nsim,"simulations\n\n"))
	    		print(signif(x$qn.c,5))
		}
    		if (x$warning) {
        		cat("\n\nWarning: At least one sample size is less than 5,\n")
			cat("  asymptotic p-values may not be very accurate.\n")
    		}
		cat("\n")
		invisible(x)
  	}
  	if(x$test.name == "2 x t Contingency Table"){
   	 	cat(paste("\n      Kruskal-Wallis Test for 2 x",x$t,"Contingency Table\n\n"))
   		if(x$method=="simulated") cat(paste("      Based on Nsim =", x$Nsim,"simulations\n\n"))
		print(signif(x$KW.cont,5))
		cat("\n")
		invisible(x)
	}
  	if(x$test.name == "Combined 2 x t Contingency Tables"){
   	 	cat("\n   Combined Kruskal-Wallis Tests for 2 x t Contingency Tables\n\n")
   		if(x$method=="simulated") cat(paste("      Based on Nsim =", x$Nsim,"simulations\n\n"))
		nx <- length(x$kw.list)
		for( i in 1:nx){
				cat(paste("for data set",i,"we get\n"))
    				print(signif(x$kw.list[[i]],5))
            			cat("\n")
		}
		if(nx>3) KW.name=paste("KW.1","...",paste("KW.",k,sep=""),sep="+")
    		if(nx==2)KW.name=paste("KW.1+KW.2")
    		if(nx==3)KW.name=paste("KW.1+KW.2+KW.3")
		cat("Combined Criterion: KW.combined =",KW.name,"\n")
			cat("\n")
        		if(x$method=="simulated") cat(paste("Based on Nsim =",x$Nsim,"simulations\n\n"))
	    		print(signif(x$kw.c,5))
			cat("\n")
		invisible(x)
	}
  	if(x$test.name == "Steel"){
                print(signif(x$st,5))
		invisible(x)
	}
	if(x$test.name == "Steel.bounds"){
	cat("\nSteel Multiple Comparison Confidence Bounds for Shift Parameters\nBased on Wilcoxon Tests, k Treatments against a Common Control\n\n")
		cat("size of control sample: ",x$n0,"\n")
		if(length(x$ns) > 1){
			cat("sizes of treatment samples: ",paste(x$ns,collapse=", "),"\n")
		}else{
			cat("size of treatment sample: ",x$ns,"\n")
		}
		if(x$n.ties > 0){
			cat("number of ties: ",x$n.ties,"\n")
			cat("intervals should be widened on each end by the rounding epsilon\n")
			cat("to conservatively maintain the stated joint confidence level\n")
			    
		}
		

		cat("\nconservative bounds based on asymptotics\n\n")
		print(x$bounds[[1]])
		
		cat("\nbounds based on asymptotics,\n")
		cat("with level closest to nominal\n\n")
		print(x$bounds[[2]])
		
		if(x$method=="simulated"){
			cat("\nconservative bounds based on simulation\n\n")
			print(x$bounds[[3]])
			
			cat("\nbounds based on simulation,\n")
			cat("with level closest to nominal\n\n")
			print(x$bounds[[4]])
			
		}
		invisible(x)
	}
        
	
}

contingency2xt.comb <-
       function (..., method=c("asymptotic","simulated","exact"),
                  dist=FALSE,Nsim=10000) 
{
#################################################################
# This function contingency2xt.comb combines several 2 x t 
# contingency table analyses over M blocks (possibly different t 
# across blocks) by adding the Kruskal-Wallis test criteria for
# each of the blocks.
# This follows the same pattern as generally combinging 
# Kruskal-Wallis tests across blocks as suggested in 
# Lehmann, E.L. (2006), Nonparametrics, Statistical Methods Based 
# on Ranks, Ch. 6, Sec. 5D.
# See also the documentation of contingency2xt for the analysis
# of a single 2 x t table of counts.
# This combined version tests the hypothesis that all the 
# hypotheses underlying the individual contingency tables are 
# true simultaneously and is relevant when randomizations or 
# samples are independent from block to block
#
# Input: ...  
#        can take the form of several lists, 
#        say L.1,...,L.M, where list L.i contains 
#        2 sample vectors of length t > 1 each, but the
#	    t may vary from list to list,	
#
#        or a single list of such lists.
#
#	method 
#       can take one of three values "asymptotic", "simulated", 
#       and "exact", 
#	which determines the mode of P-value calculation.
#       The asymptotic P-value, based on the chi-square 
#       approximation, is always returned.
#       The simulated P-value simulates counts for the tables 
#       conditioned on the observed marginal totals 
#       (see contingency2xt), doing this independently across
#       tables, and computing the corresponding 
#       Kruskal-Wallis statistics KW.i, for i = 1,...,M.
#       Adding the KW.i yields the combined statistic
#       KW.combined. 
#   	   This is repeated Nsim times and the simulated P-value is
#       the proportion of these values that are >= the observed
#       combined value.
#       The exact P-value should only be attempted for small M
#       and small marginal totals and requires that Nsim be set
#       to >= the total number of KW.combined enumerations. 
#       Otherwise Nsim simulations are run to get a simulated 
#       P-value, as described above.
#       As example consider: M=2 with t.1 = 3 columns in the
#       first table and row totals m.1 = 40, n.1[2] = 60, while
#       the second table has t.2 = 4 columns, with row totals 
#       m.2 = 30 and n.2 = 25.
#       Then we would have choose(40+2,2) = 861 possible counts
#       for table 1
#       and choose(30+3,3) = 5456 counts for table 2, thus 
#       861*5456 = 4697616 possible count configurations for both
#       tables jointly. Thus one should choose Nsim >=  4697616.
#       However, the ultimate distributions KW.combined may have 
#       far fewer unique values.
#
#     dist
#       takes values FALSE (default) or TRUE, where TRUE enables
#       the return of the simulated or exact distributions of 
#       KW.combined. Otherwise NULL is returned for both versions
#
#     Nsim 
#       = 10000 (default), number of simulations as discussed 
#       above.
#       
#       
#
# An example:
# contingency2xt.comb(list(c(15,12,25),c(12,5,7)), 
#                     list(c(12,6,4),c(6,12,3)),
#                     method="exact",dist=F, Nsim=1e6)
# produces the outcome below.
#################################################################
# 
#   Combined Kruskal-Wallis Tests for 2 x t Contingency Tables
#
# for data set 1 we get
#       observed KW   asympt. P-value     exact P-Value 
#         3.4539090         0.1778252         0.1786888 
#
# for data set 2 we get
#       observed KW   asympt. P-value     exact P-Value 
#         4.0259740         0.1335890         0.1673558 
# 
# Combined Criterion: KW.combined = KW.1+KW.2 
# 
#      KW.combined  asympt. P-value    exact P-Value 
#        7.4798830        0.1125996        0.1099804 
#
#
#################################################################
# For
# out <- contingency2xt.comb(list(c(15,12,25),c(12,5,7)), 
#                            list(c(12,6,4),c(6,12,3)),
#                            method="exact",dist=F, Nsim=1e6)
# we get the object out of class ksamples with the following 
# components
#  names(out)
# [1] "test.name"  "t"  "M" "kw.list"   "kw.c"  "null.dist"
# [7] "method"    "Nsim"  
# where 
# test.name 	= "Combined 2 x t Contingency Tables"
# t		= vector giving the number of columns for each table
# M  		= number of tables
# kw.list 	= list of M vectors holding the results for each of 
#            the tests corresponding to the M blocks
# kw.c  	= 2 (or 3) vector containing the observed KW.combined, 
#            asymptotic P-value, (and simulated or exact P-value) 
#            for the combined test.
# null.dist 	= L x 2 matrix (if dist = TRUE), with first 
#                 column holding the unique, simulated or fully 
#                 numerated KW statistics, and the second column 
#		       holding the corresponding relative frequencies 
#                 or probabilities.
#                 If dist = FALSE we get null.dist = NULL.
# method  	  one of the following values: "asymptotic",
#              "simulated", "exact" as it was ultimately used.
# Nsim  		number of simulations used, when applicable.
#
# Fritz Scholz, April 2012
#################################################################

# the following converts individual data set lists into a list of
# such, if not already in this form.
if(length(list(...)) == 1) {
	if(is.list(...) & is.list(...[[1]])){
        	data.sets <- list(...)[[1]]}else{
       		stop("you need more than 1 block of data sets\n")
	}
   	}else {
        	data.sets <- list(...)
	} 
# end of data.sets list conversion
method <- match.arg(method)
M <- length(data.sets) # number of data sets
if(M < 2) 
	stop("To combine test results you must have at least two data sets.")
tvec <- numeric(M)
ncomb <- 1
for(i in 1:M){
	n.sample <- sapply(data.sets[[i]], length) 
        if(n.sample[1] != n.sample[2])  stop("Not all count vectors have the same length.")
        tvec[i] <- n.sample[1]
	m <- sum(data.sets[[i]][[1]])
	ncomb <- ncomb * choose(m+tvec[i] - 1,m) 
}
if(ncomb > Nsim & method == "exact") method <- "simulated"	

# Initializing output objects
KWobs <- 0
null.dist <- NULL
kw.list <- list()

if(method == "asymptotic"){
	dist0 <- FALSE
	for(i in 1:M){
		out <- contingency2xt(data.sets[[i]][[1]],data.sets[[i]][[2]],
			method=method,dist=dist0,tab0=FALSE,Nsim=Nsim)
		kw.list[[i]] <- out$KW.cont
		KWobs <- KWobs + out$KW.cont[1] 
                  # aggregates combined KW stats
	}
}
# the following loops aggregate the (estimated or exact)
# convolution distribution of the combined KW statistics
if(method == "simulated"){
	dist0 <- TRUE
	for(i in 1:M){
		out <- contingency2xt(data.sets[[i]][[1]],data.sets[[i]][[2]],
			method=method,dist=dist0,tab0=FALSE,Nsim=Nsim)
		if(i == 1){
			null.dist <- out$null.dist
			}else{
				null.dist <- null.dist + out$null.dist 
			}
		kw.list[[i]] <- out$KW.cont
		KWobs <- KWobs + out$KW.cont[1] 
               # aggregates combined KW stats 
	}
}
if(method == "exact"){
	dist0 <- TRUE
	for(i in 1:M){
		out <- contingency2xt(
                   data.sets[[i]][[1]],data.sets[[i]][[2]],
			   method=method,dist=dist0,Nsim=Nsim)
		if(i == 1){
			null.dist <- out$null.dist
			}else{
				null.dist <-   
                           conv(null.dist[,1],null.dist[,2],
					out$null.dist[,1],out$null.dist[,2])
			}
		kw.list[[i]] <- out$KW.cont
		KWobs <- KWobs + out$KW.cont[1] 
               # aggregates combined KW stats 
	}
}


# get exact or simulated P-value
if(method == "simulated"){
	pval <- sum(null.dist >= KWobs)/Nsim
}
if(method == "exact"){
	pval <- sum(null.dist[null.dist[,1] >= KWobs,2])
}

# get asymptotic P-value
pval.asympt <- 1-pchisq(KWobs, sum(tvec) - M)

    if(method=="asymptotic"){
     	kw.c <- c(KWobs,pval.asympt)
	}else{
		kw.c <- c(KWobs,pval.asympt,pval)
	}
    if(method=="asymptotic"){
		names(kw.c) <- c("KW.combined"," asympt. P-value")
    }
    if(method=="exact"){
		names(kw.c) <- 
              c("KW.combined"," asympt. P-value","exact P-Value")
    }
    if(method=="simulated"){
		names(kw.c) <- 
              c("KW.combined"," asympt. P-value","sim. P-Value")
    }
if(dist==FALSE){
	null.dist <- NULL
}
if(method== "simulated" & dist==TRUE){
	out <- table(round(null.dist,6))
        null.dist <-  
           cbind(as.numeric(names(out)),as.numeric(out)/Nsim)
	  dimnames(null.dist) <- list(NULL,c("KW","prob"))
}

object <- list(test.name =paste(
            "Combined 2 x t Contingency Tables"), 
            t = tvec, M = M, kw.list = kw.list, kw.c = kw.c, 
            null.dist = null.dist, method = method, Nsim = Nsim)
class(object) <-  "kSamples"
object

}

pp.kSamples <- function(x){
if(!inherits(x,"kSamples", TRUE)) stop('input not of class "kSamples"\n')
if(x$test.name == "Anderson-Darling"){
		if(is.null(x$null.dist1)) stop("the null.dist1 & null.dist2 components of input are NULL\n")
	}else{
		if(is.null(x$null.dist)) stop("the null.dist component of input is NULL\n")
}

if(x$test.name == "Anderson-Darling"){
	version <- "0"
	while(substr(version,1,1) != "1" & substr(version,1,1) != "2"){
		version <- readline("choose version of test statistic: 1 or 2\n")
	} 
	if(names(x)[[2]]=="k") {
		dist <- x[[8+as.numeric(version)]]}else{
		dist <- x[[12+as.numeric(version)]]
	}
	if(!is.null(dist)){
		xy <- table(dist)
		N <- length(dist)
		if(names(x)[[2]]=="k") {
			xx <- (as.numeric(names(xy))-(x$k-1))/x$sig
			px <- ad.pval(xx,x$k-1,as.numeric(version))}else{
			xx <- (as.numeric(names(xy))-(x$mu.c))/x$sig.c
			px <- ad.pval(xx,x$mu.c,as.numeric(version))
		}
		px.exact <- rev(cumsum(rev(as.numeric(xy)))/N)
		m <- min(px,px.exact)
		M <- max(px,px.exact)
		par(pty="s")
		if(names(x)[[2]]=="k") titlex <- paste(x$test.name,"k-sample test, version", version)
		if(names(x)[[2]]=="M") titlex <- paste("combined", x$test.name,"k-sample tests, version", version)
		plot(px.exact,px,log="xy",xlim=c(m,M), ylim=c(m,M), main= titlex,
		xlab=paste(x$method,"significance probability"),ylab="asymptotic significance probability",pch=16,cex=.5)
		text(.055,m,".05",adj=0)
		abline(0,1,lty=2)
		abline(h=.05,lty=2)
		abline(v=.05,lty=2)
		text(.012,.5,".01",adj=0)
		abline(h=.01,lty=2)
		abline(v=.01,lty=2)
		if(x$method=="simulated"){ 
			text(m,M,paste("Nsim =",x$Nsim),adj=0)}else{
		        text(m,M,paste("Ncomb =",length(dist)),adj=0)
		}
	}else{
		cat("no distribution data\n")
	}
}
if(x$test.name == "Kruskal-Wallis" |
   	x$test.name == "van der Waerden scores" |
   	x$test.name == "normal scores"){
	dist <- x$null.dist

	if(!is.null(dist)){
		xy <- table(dist)
		N <- length(dist)
		xx <- as.numeric(names(xy))
		if(names(x)[[2]] == "k"){
			px <- 1-pchisq(xx,x$k - 1)}else{
			df <- sum(sapply(x$n.samples,length))-length(x$n.samples)
			px <- 1-pchisq(xx,df)
		}
		px.exact <- rev(cumsum(rev(as.numeric(xy)))/N)
		m <- min(px,px.exact)
		M <- max(px,px.exact)
		par(pty="s")
		if(names(x)[[2]]=="k") titlex <- paste(x$test.name,"k-sample test")
		if(names(x)[[2]]=="M") titlex <- paste("combined", x$test.name,"k-sample tests")
		plot(px.exact,px,log="xy",xlim=c(m,M), ylim=c(m,M), main= titlex,
			xlab=paste(x$method,"significance probability"),
			ylab="asymptotic significance probability",pch=16,cex=.5)
		text(.055,m,".05",adj=0)
		abline(0,1,lty=2)
		abline(h=.05,lty=2)
		abline(v=.05,lty=2)
		text(.012,.5,".01",adj=0)
		abline(h=.01,lty=2)
		abline(v=.01,lty=2)
		if(x$method=="simulated"){ 
			text(m,M,paste("Nsim =",x$Nsim),adj=0)}else{
	    	    text(m,M,paste("Ncomb =",length(dist)),adj=0)
		}
	}else{
		cat("no distribution data\n")
	}
}

if(x$test.name == "Jonckheere-Terpstra"){
	dist <- x$null.dist

	if(!is.null(dist)){
		xy <- table(dist)
		N <- length(dist)
		xx <- as.numeric(names(xy))
		mu <- x$JT[2]
		sig <- x$JT[3]
		px <- 1-pnorm(xx,mu,sig)
		px.exact <- rev(cumsum(rev(as.numeric(xy)))/N)
		m <- min(px,px.exact)
		M <- max(px,px.exact)
		par(pty="s")
		titlex <- paste(x$test.name,"k-sample test")
		plot(px.exact,px,log="xy",xlim=c(m,M), 
			ylim=c(m,M), main= titlex,
			xlab=paste(x$method,"significance probability"),
			ylab="asymptotic significance probability",pch=16,cex=.5)
		text(.055,m,".05",adj=0)
		abline(0,1,lty=2)
		abline(h=.05,lty=2)
		abline(v=.05,lty=2)
		text(.012,.5,".01",adj=0)
		abline(h=.01,lty=2)
		abline(v=.01,lty=2)
		if(x$method=="simulated"){ 
			text(m,M,paste("Nsim =",x$Nsim),adj=0)}else{
	    	    text(m,M,paste("Ncomb =",length(dist)),adj=0)
		}
	}else{
		cat("no distribution data\n")
	}
}

if(x$test.name == "2 x t Contingency Table" | 
   	x$test.name == "Combined 2 x t Contingency Tables"){
	dist <- x$null.dist
	if(!is.null(dist)){
		xx <- dist[,1]
	        px.exact <- rev(cumsum(rev(dist[,2])))
		N <- length(dist[,1])
		if(names(x)[[3]] == "M"){
			M <- x$M
	    		tvec <- x$t
			df <- sum(tvec)-M 
		}else{
			df <- x$t -1
		}
		px <- pchisq(xx,df,lower.tail=FALSE)
		m <- min(px,px.exact)
		M <- max(px,px.exact)
		plot(px.exact,px,log="xy",xlim=c(m,M), ylim=c(m,M), main= x$test.name,
		xlab=paste(x$method,"significance probability"),ylab="asymptotic significance probability",pch=16,cex=.5)
		text(.055,m,".05",adj=0)
		abline(0,1,lty=2)
		abline(h=.05,lty=2)
		abline(v=.05,lty=2)
		text(.008,.5,".01",adj=1)
		abline(h=.01,lty=2)
		abline(v=.01,lty=2)
		if(x$method=="simulated"){ 
			text(m,M,paste("Nsim =",x$Nsim),adj=0)}else{
		        text(m,M,paste("Nsupp =",length(dist[,1])),adj=0)
		}
	}else{
		cat("no distribution data\n")
	}
}
if(x$test.name == "Steel"){
	dist <- x$null.dist

	if(!is.null(dist)){
		xy <- table(dist)
		N <- length(dist)
		xx <- as.numeric(names(xy))
		tau <- x$tau
		sig <- x$sig
		sig0 <- x$sig0
		ni <- x$ns[-1]
		px <- Steelnormal0(sig0,sig,tau,xx,ni, alternative= x$alternative)
		px.exact <- rev(cumsum(rev(as.numeric(xy)))/N)
		m <- min(px,px.exact)
		M <- max(px,px.exact)
		par(pty="s")
		titlex <- paste(x$test.name,"k-sample test")
		plot(px.exact,px,log="xy",xlim=c(m,M), 
			ylim=c(m,M), main= titlex,
			xlab=paste(x$method,"significance probability"),
			ylab="asymptotic significance probability",pch=16,cex=.5)
		text(.055,m,".05",adj=0)
		abline(0,1,lty=2)
		abline(h=.05,lty=2)
		abline(v=.05,lty=2)
		text(.012,.5,".01",adj=0)
		abline(h=.01,lty=2)
		abline(v=.01,lty=2)
		if(x$method=="simulated"){ 
			text(m,M,paste("Nsim =",x$Nsim),adj=0)}else{
	    	    text(m,M,paste("Ncomb =",length(dist)),adj=0)
		}
	}else{
		cat("no distribution data\n")
	}
}

}

# Project:  2 * t Contingency Table
# adapted from Angie Zhu's getTable2xtNull.R
# Filename: Contingency2xt.R
# Last modified: 03.23.2012

# This function computes or simulates the null distribution of
# the Kruskal-Wallis statistics in a 2 x t contingency table.  

# Treatment |   1   2   ...    t  | Total
# ---------------------------------------
# Response  |                     |
#    a      | A_1  A_2   ...  A_t | m
#    b      | B_1  B_2   ...  B_t | n
# ---------------------------------------
# Total     | d_1  d_2   ...  d_t | N

# Arguments:  
#		Avec: integer vector of length tnum, containing the 
#                "a" reponses
#		Bvec: integer vector of length tnum, containing the 
#                "b" reponses
#		method: "asymptotic", "simulated", "exact", 
#                  indicating the method of p-value calculation,
#		dist: FALSE (default) means that the exact or
#                estimated null distribution is not requested, 
#                otherwise it is. 
#          tab0: TRUE (default), when method = "simulated",
#			 the null distribution is given in tabular form,
#                when tab0 = FALSE and method = "simulated", the
#                null distribution is given as a single vector 
#                of all simulated values. 
#		Nsim: number of simulations to run, default 10^6
#
#
# Output: a list of class "kSamples" with components
#        test.name  = "2 x t Contingency Table"
#                t  =  number of columns in the table
#          KW.cont  = 2 (or 3 vector) containing the observed KW
#                     value, its asymptotic (and the simulated 
#                     or exact) P-value
#          null.dist  = a 2 x M matrix giving the M unique 
#                       ordered KW values of the null 
#                       distribution in the first column, and 
#                       the corresponding probabilities 
#                       (simulated or exact) in the second 
#                       column. When method = "simulated" and 
# 			        tab0 = FALSE null.dist consists of a
#                       single vector of all simulated values.
#			When dist = NULL or method = "asymptotic", only 
#                           NULL is returned.
#        method = the method used
#        Nsim = number of simulations employed


contingency2xt <- function(Avec, Bvec, method=c("asymptotic","simulated","exact"), 
           		dist = FALSE, tab0 = TRUE, Nsim=1e6) {
    	method <- match.arg(method)
	m <- sum(Avec)
	n <- sum(Bvec)
    	tnum <- length(Avec)
	N <- m + n
	dvec <- Avec + Bvec
	KW.obs <- (N * (N - 1) / (m * n)) * (sum(Avec^2 / dvec) - 
                  	m^2/N )
	pValue.asy <- 1 - pchisq(KW.obs,tnum-1)
	if(dist == TRUE) Nsim <- min(Nsim,1e8) 
	# limits the size of null.dist
	# whether method = "exact" or = "simulated"
	if(method == "asymptotic"){
		KW.cont <- c(KW.obs,pValue.asy)
		null.dist <- NULL
	}else{
		ncomb <- choose(m + tnum - 1, tnum - 1)
		if (ncomb <= Nsim && method == "exact") { 
			if(dist){ 
					ans <- numeric(2+2*ncomb)
				}else{
					ans <- numeric(2)
			}
			out <- .C("contingency2xtExact0",
					Avec = as.integer(Avec), Bvec = as.integer(Bvec),
					tnum = as.integer(tnum), 
					ncomb = as.integer(ncomb),
					getDist = as.integer(dist),
                                	ans = as.double(ans), PACKAGE = "kSamples")
				KW.obs <- out$ans[1]
				KW.obs <- (N * (N - 1) / (m * n)) * (KW.obs - 
                                 	m^2 / N) 
				KW.obs <- round(KW.obs, 6)
       				pValue <- out$ans[2]		
				if(dist){	
					out <- out$ans[-(1:2)]
					out <- matrix(out, nrow=ncomb, ncol=2, byrow=FALSE)
					out <- out[out[, 2] > 0, ]
					prob <- out[ , 2]
					KW <- (N * (N - 1) / (m * n)) * (
                                    		out[ , 1] - m^2 / N) 
					KW <- round(KW, 6)
					KW.u <- sort(unique(KW))
					k.u <- length(KW.u)
					prob.u <- sapply(1:k.u, function(j) {
		 					sum( prob[KW == KW.u[j]] ) })
					null.dist <- cbind(KW.u,prob.u)
                    			dimnames(null.dist) <- list(NULL,c("KW","prob"))
				}else{
					null.dist <- NULL
				}
				KW.cont <- c(KW.obs,pValue.asy,pValue)
		}else{
			method <- "simulated"
		}
		if( method == "simulated" ){
			if( Nsim < 100) Nsim <- 100
			if(dist){ 
					ans <- numeric(2+Nsim)
				}else{
					ans <- numeric(2)
			}
			out <- .C("contingency2xtSim0",
					Avec = as.integer(Avec), Bvec = as.integer(Bvec),
					tnum = as.integer(tnum), 
					Nsim = as.integer(Nsim),
					getDist = as.integer(dist),
                                	ans = as.double(ans))	
			KW.obs <- out$ans[1]
			KW.obs <- (N * (N - 1) / (m * n)) * (KW.obs - m^2 / N) 
			KW.obs <- round(KW.obs, 6)
      			pValue <- out$ans[2]		
			if(dist){
				KW <- (N * (N - 1) / (m * n)) * 
                           		(round(out$ans[3:(Nsim+2)],6) - m^2 / N)
				if(tab0==TRUE){
					tab <- table(KW)/Nsim
					null.dist <-
                                   cbind(as.numeric(names(tab)),
                                            as.vector(tab))
                    		dimnames(null.dist) <- 
                             list(NULL,c("KW","prob"))}else{
					null.dist <- KW
				}
			}else{
				null.dist <- NULL
			}
			KW.cont <- c(KW.obs,pValue.asy,pValue)
		}
	}
    if(method=="asymptotic"){
		names(KW.cont) <- c("observed KW",
                               "  asympt. P-value")
    }
    if(method=="exact"){
		names(KW.cont) <- c("observed KW",
                           "  asympt. P-value","exact P-Value")
    }
    if(method=="simulated"){
		names(KW.cont) <- c("observed KW",
                           "  asympt. P-value","sim. P-Value")
    }
    object <- list(test.name =paste("2 x t Contingency Table"),
                     t = tnum, KW.cont = KW.cont, 
                     null.dist = null.dist, 
				method = method, Nsim = Nsim)
    class(object) <- "kSamples"
    object
}

listmake <- function (y=1:10,g=as.factor(c(1,1,1,2,2,2,2,2,2,3)),b=as.factor(c(1,1,1,1,3,2,2,2,3,1)))
{
data.sets <- lapply(levels(b), 
					function(blv){
						gg <- as.factor(as.numeric(unlist(g[b==blv])))
						lapply(levels(gg),
							function(glv){
						     return(y[b == blv & g == glv])
							}
						)
					}
				)
data.sets
}

pjt <- function (x, nn) 
{
    dist <- Harding(nn)
    if (is.nan(dist[1])) 
        stop("can't compute due to numerical instability\n")
    nn <- sort(as.integer(nn))
    nvec <- rev(cumsum(rev(nn)))
    k <- length(nn)
    L <- sum(nn[1:(k - 1)] * nvec[2:k])
    dist <- cumsum(dist)
    x <- floor(x)
    pos <- match(x, 0:L)
    d <- numeric(length(x))
    d[x > L] <- 1
    d[!is.na(pos)] <- dist[pos[!is.na(pos)]]
    d
}

qjt <- function (p, nn) 
{
    dist <- Harding(nn)
    if (is.nan(dist[1])) 
        stop("can't compute due to numerical instability\n")
    nn <- sort(as.integer(nn))
    nvec <- rev(cumsum(rev(nn)))
    k <- length(nn)
    L <- sum(nn[1:(k - 1)] * nvec[2:k])
    dist <- c(0, cumsum(dist))
    x <- c(-Inf, 0:L)
    out <- findInterval(p, dist)
    out[p %in% dist] <- out[p %in% dist] - 1
    x[out + 1]
}

jt.test <-
function (..., data = NULL,
		method=c("asymptotic","simulated","exact"),
		dist=FALSE,Nsim=10000) 
{
#############################################################################
# This function "jt.test" tests whether k samples (k>1) come from a common
# continuous distribution, using the Jonckheere-Terpstra rank test. See Lehmann (2006),
# Nonparametrics, Statistical Methods Based on Ranks.
# The test rejects the null hypothesis of no effect when JT is too large,
# i.e., a positive trend in the samples (in the order given) seems indicated.
#
# Ties are handled by using midranks.
# While the asymptotic P-value is always returned, there is the option 
# to get a P-value estimate based on Nsim simulations or an exact value based 
# on the full enumeration distribution, provided method = "exact" is chosen
# and the number of full enumerations is <= the Nsim specified.
# If the latter is not the case, simulation is used with the indicated Nsim.
# These simulated or exact P-values are appropriate under the continuity 
# assumption or, when ties are present, they are still appropriate
# conditionally on the tied rank pattern, provided randomization took 
# place in allocating subjects to the respective samples, i.e., also
# under random sampling from a common discrete parent population.
# However, under ties the results are only meaningful conditionally given
# the observed tie pattern.
#
#
# 
# Inputs:
#       	...:	can either be a sequence of k (>1) sample vectors,
#
#				or a list of k (>1) sample vectors,
#
#                               or y, g, where y contains the concatenated
#				samples and g is a factor which by its levels
#				identifies the samples in y,
#
#                               or a formula y ~ g with y and g as in previous case.
#
#				
#               data:   data frame with variables usable in formula input, default = NULL.
#           
#				
#			method: takes values "asymptotic", "simulated", or "exact".
#				The value "asymptotic" causes calculation of P-values
#            		        using the asymptotic normal approximation, always done.
#
#				The value "simulated" causes estimation of P-values
#				by randomly splitting the pooled data into
#				samples of sizes ns[1], ..., ns[k], where
#  				ns[i] is the size of the i-th sample vector,
#				and n = ns[1] + ... + ns[k] is the pooled sample size.
#				For each such random split the JT statistic is 
#				computed. This is repeated Nsim times and the proportions
#				of simulated values >= the actually observed JT value
#				is reported as P-value estimate.
#
#                   		The value "exact" enumerates all n!/(ns[1]! * ... * ns[k])
#                   		splits of the pooled sample and computes the JT statistic
#				for each such split. 
#				The proportion of all enumerated JT statistics
# 				that are >= the actually observed JT value
#				is reported as exact (conditional) P-value.
#
#			dist: 	= FALSE (default) or TRUE, TRUE causes the simulated
#					or fully enumerated vector of the QN statstic to be returned
#     					as null.dist. The length of this vector should not exceed 1e8.
#
#			Nsim: 	number of simulations to perform, 
#					for method = "exact" to take hold, it needs to be >=
#					the number of all possible splits of the pooled
#					data into samples of sizes ns[1], ..., ns[k], where
#  					ns[i] is the size of the i-th sample vector.
#
# When there are NA's among the sample values they are removed,
# with a warning message indicating the number of NA's.
# It is up to the user to judge whether such removals make sense.
#
# An example:
# x1 <- c(1,2)
# x2 <- c(1.5,2.1)
# x3 <- c(1.9,3.1)
# jt.test(x1,x2,x3,method="exact",Nsim=90)
# or 
# jt.test(list(x1,x2,x3),method="exact",Nsim=90)
# which produces the output below.
#############################################################################
# Jonckheere-Terpstra k-sample test.
#
# Number of samples:  3
# Sample sizes:  2, 2, 2
# Number of ties: 0
#
# Null Hypothesis: All samples come from a common population.
# Alternative: Samples indicate a positive trend.
#
#   test statistic               mu              sig  asympt. P-value 
#        9.0000000        6.0000000        2.5166115        0.1166151 
#    exact P-Value 
#       0.1666667 
# 
# 
# Warning: At least one sample size is less than 5,
#   asymptotic p-values may not be very accurate.

#############################################################################
# In order to get the output list, call 
# JT.out <- jt.test(list(x1,x2,x3),method="exact",dist=T,Nsim=100000)
# then JT.out is of class "kSamples" and has components 
# > names(JT.out)
# [1] "test.name" "k"         "ns"        "N"         "n.ties"    "JT"       
# [7] "warning"   "null.dist" "method"    "Nsim"    
#
# where
# test.name = "Jonckheere-Terpstra"
# k = number of samples being compared
# ns = vector of the k sample sizes ns[1],...,ns[k]
# N = ns[1] + ... + ns[k] total sample size
# n.ties = number of ties in the combined set of all n observations
# JT =  4 (or 5) vector containing the JT statistics, its mean and standard 
# deviation, its asymptotic P-value, (and its exact or simulated P-value). 
# warning = logical indicator, warning = TRUE indicates that at least  
#		one of the sample sizes is < 5.    
# null.dist is a vector of simulated values of the JT statistic
# 		or the full enumeration of such values.
#		This vector is given when dist = TRUE is specified, 
# 		otherwise null.dist = NULL is returned.
# method = one of the following values: "asymptotic", "simulated", "exact"
# 			as it was ultimately used.
# Nsim = number of simulations used, when applicable.
#
# The class "kSamples" causes JT.out to be printed in a special output
# format when invoked simply as: > JT.out
# An example was shown above.
#
# Fritz Scholz, August 2015
#
#################################################################################
JTmusig <- function (rx,ns) 
{
# this function computes the mean and standard deviation of the 
# JT statistic when ties are present.
	N <- length(rx)
	dvec <- as.vector(table(rx))
	X1 <- N*(N-1)*(2*N+5)
	X2 <- sum(ns*(ns-1)*(2*ns+5))
	X3 <- sum(dvec*(dvec-1)*(2*dvec+5))
	A1 <- (X1-X2-X3)/72
	A2 <- sum(ns*(ns-1)*(ns-2))*sum(dvec*(dvec-1)*(dvec-2))
	A2 <- A2/(36*N*(N-1)*(N-2))
	A3 <- sum(ns*(ns-1))*sum(dvec*(dvec-1))/(8*N*(N-1))
	sig <- sqrt(A1+A2+A3)
	nmat <- outer(ns,ns,"*")
	mu <- sum(nmat[upper.tri(nmat)])/2
	list(mu=mu,sig=sig)
}
##############################################################


	samples <- io(..., data = data)
	method <- match.arg(method)
	out <- na.remove(samples)
	na.t <- out$na.total
	if( na.t > 1) print(paste("\n",na.t," NAs were removed!\n\n"))
	if( na.t == 1) print(paste("\n",na.t," NA was removed!\n\n"))
	samples <- out$x.new
	k <- length(samples)
	if (k < 2) stop("Must have at least two samples.")
	ns <- sapply(samples, length)
	n <- sum(ns)
	if (any(ns == 0)) stop("One or more samples have no observations.")
	rx <- rank(unlist(samples))
	JTobs <- 0
	pval <- 0
	L <- length(unique(rx)) # to count ties
	if(dist == TRUE) Nsim <- min(Nsim,1e8) 
	# limits the size of null.dist	
	ncomb <- 1
	np <- n
    	for(i in 1:(k-1)){
		ncomb <- ncomb * choose(np,ns[i])
        	np <- np-ns[i]
	}
	# it is possible that ncomb overflows to Inf
	if( method == "exact" & Nsim < ncomb ) {
		method <- "simulated"
    	}
	if( method == "exact" & dist == TRUE ) nrow <- ncomb
   	if( method == "simulated" & dist == TRUE ) nrow <- Nsim

	if( method == "simulated" ) ncomb <- 1 # don't need ncomb anymore
	if(method == "asymptotic"){
		Nsim <- 1
		dist <- FALSE
	}
	useExact <- FALSE
	if(method == "exact") useExact <- TRUE
	if(dist == TRUE){
		JTvec <- numeric(nrow)
	}else{
    		JTvec <- 0
	}

	out <- .C("JTtest", pval=as.double(pval),
			Nsim=as.integer(Nsim), k=as.integer(k), 
			rx=as.double(rx), ns=as.integer(ns), 
            		useExact=as.integer(useExact),
			getJTdist=as.integer(dist),
			ncomb=as.double(ncomb),JTobs=as.double(JTobs),
			JTvec = as.double(JTvec), PACKAGE = "kSamples")
	JTobs <- out$JTobs
	musig <- JTmusig(rx,ns)
	mu <- musig$mu
	sig <- musig$sig
	pval <- out$pval
	if(dist){
		JTvec <- round(out$JTvec,8)
		}else{
		JTvec <- NULL
	}
	pval.asympt <- 1-pnorm((JTobs - mu)/sig)
	if(method=="asymptotic"){
		JT <- c(JTobs,mu,sig,pval.asympt)
	}else{
		JT <- c(JTobs,mu,sig,pval.asympt,pval)
	}
	if(method=="asymptotic"){
		names(JT) <- c("test statistic","mu","sig"," asympt. P-value")
	}
	if(method=="exact"){
		names(JT) <- c("test statistic","mu","sig"," asympt. P-value","exact P-Value")
	}
	if(method=="simulated"){
		names(JT) <- c("test statistic","mu","sig"," asympt. P-value","sim. P-Value")
	}
	warning <- FALSE
	if(min(ns) < 5) warning <- TRUE
	if(dist == FALSE) null.dist <- NULL
	test.name <- "Jonckheere-Terpstra"

	object <- list(test.name = test.name,
		k = k, ns = ns, N = n, n.ties = n - L,
		JT = JT, warning = warning, null.dist = JTvec,
		method=method, Nsim=Nsim)
    class(object) <- "kSamples"
    object

}

SteelConfInt <-function(..., data = NULL, conf.level=.95,
		alternative=c("less","greater","two.sided"),
		method = c("asymptotic","exact","simulated"), Nsim = 10000){

pmaxWilcox <- function(x,ns){
#----------------------------------------------------------------------------------
# computes the CDF of the maximum of standarized Wilcoxon statistics for each 
# element in x, using the asymptotic approximation.
# Standardized Wilcoxon statistics are considered for sample sizes ns[1] and 
# each of ns[i], i=2,..., length(ns)
#----------------------------------------------------------------------------------
	if(length(ns) < 2) return("ns has length < 2\n")
	n1 <- ns[1]
	ni <- ns[-1]
	s <- length(ni)
	f1 <- sqrt(1+ni/(n1+1))
	f2 <- sqrt(ni/(n1+1))
	fx <- function(z,U,f1,f2){
 		fxx <- 1
		for(i in 1:s){
			fxx <- fxx * pnorm(U*f1[i]-z*f2[i])
		}
		fxx*dnorm(z)
	}
	N <- length(x)
	px <- numeric(N)
	for(j in 1:N){
		px[j] <- integrate(f=fx,-Inf,Inf,U=x[j],f1=f1,f2=f2)$value
	}
px
}
# end of pmaxWilcox
qmaxWilcox <- function(p,ns){
#----------------------------------------------------------
# computes the p-quantile of the CDF provided by pmaxWilcox
# not vectorized over p
#----------------------------------------------------------
	if(length(ns) < 2) return("ns has length < 2\n")
	fx <- function(z,ns,p){
 		pmaxWilcox(z,ns)-p
	}
	a <- qnorm(p)-2
	b <- qnorm(p)+2
	while(fx(a,ns,p) > 0) a <- a-1
	while(fx(b,ns,p) < 0) b <- b+1
	uniroot(fx,c(a,b),ns,p)$root
}
# end of qmaxWilcox
ProbWilcox <- function(U,n1,ni){
#-----------------------------------------------------------------------
# computes the asymptotic joint probability P(W.i <= U[i], i=1,\ldots,s)
# where W.i is the Mann-Whitney statistic for sample sizes n1 and ni[i], 
# for i=1,...,s, and the same sample is involved for the first sample
# of size n1 in all Mann-Whitney statistics.
#-----------------------------------------------------------------------
	s <- length(ni)
	if(s < 1) return("ni has length < 1\n")
        if(length(U) != s) return(paste("U does not have length",s,"\n"))
	f1 <- sqrt(1+ni/(n1+1))
	f2 <- sqrt(ni/(n1+1))
	fx <- function(z,U,f1,f2){
 		fxx <- 1
		for(i in 1:s){
			fxx <- fxx * pnorm(U[i]*f1[i]-z*f2[i])
		}
		fxx*dnorm(z)
	}
	px <- integrate(f=fx,-Inf,Inf,U=U,f1=f1,f2=f2)$value
	px
}
# end of ProbWilcox
qdiscrete <- function (x, gam) 
{
#------------------------------------------------------------
# x is assumed to be a sorted numeric vector and 0 < gam < 1
# This function finds cm and cp such that 
# cm is the largest value such that the proportion of 
# x values <= cm is <= gam. cm is set to -Inf, when the 
# highest achievable proportion <= gam is zero.
# cp is the smallest value such that the proportion of 
# x values <= cp is >= gam
#------------------------------------------------------------
    if (gam <= 0 | gam >= 1) 
        stop(paste("conf.level not in (0,1)","\n"))
    N <- length(x)
    gamN <- gam * N
    im <- floor(gamN)
    ip <- ceiling(gamN)
    cm <- -Inf
    cp <- x[ip]
    if(im > 0){ 
	if(x[im] < x[ip]){ cm <- x[im]}else{ 
		if(sum(x <= x[im]) <= gamN){cm <- x[im]}else{
		 	if(x[im] > x[1]) cm <- max(x[x<x[im]])
		}
	}
    }
    list(cm = cm, cp = cp)
}
# end of qdiscrete function
samples <- io(..., data = data)
alternative <- match.arg(alternative)
method <- match.arg(method)
out <- na.remove(samples)
na.t <- out$na.total
if( na.t > 1) cat(paste("\n",na.t," NAs were removed!\n\n"))
if( na.t == 1) cat(paste("\n",na.t," NA was removed!\n\n"))
samples <- out$x.new
k <- length(samples)
if (k < 2) stop("Must have at least two samples.")
ns <- sapply(samples, length)
nsamp <- sum(ns)
if (any(ns == 0)) stop("One or more samples have no observations.")
x <- numeric(nsamp)
istart <- 0
for (i in 1:k){
	x[istart+(1:ns[i])] <- samples[[i]]
	istart <- istart + ns[i]		
}
if(method != "asymptotic"){
	if(!is.loaded("SteelConf")) dyn.load("SteelConf.so")
}
if(alternative=="two.sided"){gam <- 1-(1-conf.level)/2 }else{gam <- conf.level}
if(length(x) != sum(ns)) return("sample sizes don't agree with length of data vector\n")
k <- length(ns)
s <- k-1
# number of treatments
rx <- rank(x)
n1 <- ns[1]
ni <- ns[-1]
mu <- ni*n1/2 
	# vector of means for Mann-Whitney statistics
tau <- sqrt(n1*ni*(n1+ni+1)/12) 
	# vector of stand. deviations of Mann-Whitney statistics
	# in the case of continuous sampled populations (no ties).  
n.ties <- nsamp - length(unique(x))
# computing total number of combination splits
ncomb <- choose(nsamp,ns[1])
np <- nsamp-ns[1]
if(k>2){
	for(i in 2:(k-1)){
		ncomb <- ncomb * choose(np,ns[i])
        	np <- np-ns[i]
	}
}
if(method == "exact"){
	useExact <- TRUE}else{
        useExact <- FALSE
}
# calculation of coverage probabilities based on asymptotics
cgam <- qmaxWilcox(gam,ns)
if(alternative != "greater"){
	ellpU <- ceiling(tau*cgam+mu+1)
	ellpUx <- ellpU
        ellpUx[ellpUx > n1*ni] <- Inf
	ellmU <- floor(tau*cgam+mu+1)
	ellmUx <- ellmU
        ellmUx[ellmUx > n1*ni] <- Inf
	ellcU <- round(tau*cgam+mu+1)
	ellcUx <- ellcU
        ellcUx[ellcUx > n1*ni] <- Inf
	probpU <- ProbWilcox((ellpUx-1-mu)/tau,n1,ni)
	probmU <- ProbWilcox((ellmUx-1-mu)/tau,n1,ni)
	probcU <- ProbWilcox((ellcUx-1-mu)/tau,n1,ni)
}
if(alternative != "less"){
	ellmL <- ceiling(n1*ni-tau*cgam-mu)
	ellmLx <- ellmL
	ellmLx[ellmLx < 1] <- -Inf
	ellpL <- floor(n1*ni-tau*cgam-mu)
	ellpLx <- ellpL
	ellpLx[ellpLx < 1] <- -Inf
	ellcL <- round(n1*ni-tau*cgam-mu)
	ellcLx <- ellcL
	ellcLx[ellcLx < 1] <- -Inf
	probpL <- ProbWilcox((n1*ni-ellpLx-mu)/tau,n1,ni)
	probmL <- ProbWilcox((n1*ni-ellmLx-mu)/tau,n1,ni)
	probcL <- ProbWilcox((n1*ni-ellcLx-mu)/tau,n1,ni)
}
# initializing asymptotic bounds
Lboundp <- rep(-Inf,s)
Uboundp <- rep(Inf,s)
Lboundc <- rep(-Inf,s)
Uboundc <- rep(Inf,s)
# initializing bounds based on exact or simulated null distribution
LboundXp <- rep(-Inf,s)
UboundXp <- rep(Inf,s)
LboundXc <- rep(-Inf,s)
UboundXc <- rep(Inf,s)

if(alternative != "greater"){
# determine ellUp and probUp for which probUp comes closest 
# to gam but >= gam among the three possibilities ellpU,
# ellcU, and ellmU, with corresponding probpU, probcU, probmU
	ellUp <- ellpU
	probUp <- probpU
        if(probcU >= gam & probcU < probUp){
		ellUp <- ellcU
                probUp <- probcU
	}
        if(probmU >= gam & probmU < probUp){
		ellUp <- ellmU
                probUp <- probmU
	}
	ellUc <- ellcU
	probUc <- probcU
# determine ellUc and probUc for which probUc comes closest 
# to gam among the three possibilities ellpU,
# ellcU, and ellmU, with corresponding probpU, probcU, probmU
	if(abs(probUc-gam)>abs(probpU-gam)){
		ellUc <- ellpU
		probUc <- probpU
	}
	if(abs(probUc-gam)>abs(probmU-gam)){
		ellUc <- ellmU
		probUc <- probmU
	}

}
if(alternative != "less"){
# determine ellLp and probLp for which probLp comes closest 
# to gam but >= gam among the three possibilities ellpL,
# ellcL, and ellmL, with corresponding probpL, probcL, probmL
		ellLp <- ellpL
		probLp <- probpL
           	if(probcL >= gam & probcL < probLp){
			ellLp <- ellcL
                 	probLp <- probcL
		}
           	if(probmL >= gam & probmL < probLp){
			ellLp <- ellmL
                 	probLp <- probmL
		}
# determine ellLc and probLc for which probLc comes closest 
# to gam among the three possibilities ellpL,
# ellcL, and ellmL, with corresponding probpL, probcL, probmL
		ellLc <- ellcL
		probLc <- probcL
		if(abs(probLc-gam)>abs(probpL-gam)){
			ellLc <- ellpL
			probLc <- probpL
		}
		if(abs(probLc-gam)>abs(probmL-gam)){
			ellLc <- ellmL
			probLc <- probmL
		}
}
if(alternative == "less"){
	achieved.confidence.p <- probUp
	achieved.confidence.c <- probUc
}
if(alternative == "greater"){
	achieved.confidence.p <- probLp
	achieved.confidence.c <- probLc
}
if(alternative == "two.sided"){
	achieved.confidence.p <- probUp+probLp-1
	achieved.confidence.c <- probUc+probLc-1
}


# end of asymptotic treatment
# calculation based on exact enumeration or simulation
if(method != "asymptotic"){
	if((Nsim < ncomb & method == "exact") |  method == "simulated"){
		method <- "simulated"
		useExact <- FALSE
	   	nrow <- Nsim * s
	}else{
		nrow <- ncomb * s
	}
	MannWhitneyStats <- numeric(nrow)

	out <- .C("SteelConf", Nsim=as.integer(Nsim), k=as.integer(k), 
		rx=as.double(rx), ns=as.integer(ns), 
            	useExact=as.integer(useExact),
		MannWhitneyStats = as.double(MannWhitneyStats), PACKAGE = "kSamples")

	MannWhitneyStats <- matrix(out$MannWhitneyStats,ncol=s,byrow=T)
	maxstandMannWhitneyStats <- apply(matrix(
		(out$MannWhitneyStats-mu)/tau,ncol=s,byrow=T),1,max)
	if(method=="simulated"){
# taking advantage (only in the simulated case) of the equality of the full
# distributions of 
# -minstandMannWhitneyStats and maxstandMannWhitneyStats. 
	minstandMannWhitneyStats <- apply(matrix(
		(out$MannWhitneyStats-mu)/tau,ncol=s,byrow=T),1,min)
        maxstandMannWhitneyStats <- sort(c(maxstandMannWhitneyStats,
		-minstandMannWhitneyStats))
	}else{
		maxstandMannWhitneyStats <- sort(maxstandMannWhitneyStats)
	}
	cmp <- qdiscrete(maxstandMannWhitneyStats,gam)
	cm <- cmp$cm
	cp <- cmp$cp
	ellhat <- ceiling(tau*cp+mu)
	elltilde <- floor(tau*cm+mu)
	ellest <- round(tau*(cm+cp)/2+mu)
	selhat <- MannWhitneyStats[,1] <= ellhat[1]
	seltilde <- MannWhitneyStats[,1] <= elltilde[1]
	selest <- MannWhitneyStats[,1] <= ellest[1]
	if(s >1){
		for(i in 2:s){
			selhat <- selhat & (MannWhitneyStats[,i] <= ellhat[i])
			seltilde <- seltilde & (MannWhitneyStats[,i] <= elltilde[i])
			selest <- selest & (MannWhitneyStats[,i] <= ellest[i])
		}
	}
	p.ellhat <- mean(selhat)
	p.elltilde <- mean(seltilde)
	p.ellest <- mean(selest)
	if(alternative == "less"){
		ellhatU <- ellhat+1
		elltildeU <- elltilde+1
		ellestU <- ellest+1
	}
	if(alternative == "greater"){
		ellhatL <- n1*ni-ellhat
		elltildeL <- n1*ni-elltilde
		ellestL <- n1*ni-ellest
	}
	if(alternative == "two.sided"){
		ellhatU <- ellhat+1
		elltildeU <- elltilde+1
		ellestU <- ellest+1
		ellhatL <- n1*ni-ellhat
		elltildeL <- n1*ni-elltilde
		ellestL <- n1*ni-ellest
		p.ellhat <- 1-2*(1-p.ellhat)
		p.elltilde <- 1-2*(1-p.elltilde)
		p.ellest <- 1-2*(1-p.ellest)
# reset gam to original confidence level for later 
# refinement comparisons.
		gam <- conf.level
	}
	if(alternative != "greater"){
# determine ellUXp and probXp for which probXp comes closest 
# to gam but >= gam among the three possibilities ellhatU,
# elltildeU, and ellestU, with corresponding p.ellhat, p.elltilde, p.ellest
		ellUXp <- ellhatU
		probXp <- p.ellhat
        	if(p.elltilde >= gam & p.elltilde < probXp){
			ellUXp <- elltildeU
        	        probXp <- p.elltilde
		}
        	if(p.ellest >= gam & p.ellest < probXp){
			ellUXp <- ellestU
        	        probXp <- p.ellest
		}
		ellUXc <- ellhatU
		probXc <- p.ellhat
# determine ellUXc and probXc for which probXc comes closest 
# to gam among the three possibilities ellhatU,
# elltildeU, and ellestU, with corresponding p.ellhat, p.elltilde, p.ellest
		if(abs(probXc-gam)>abs(p.elltilde-gam)){
			ellUXc <- elltildeU
			probXc <- p.elltilde
		}
		if(abs(probXc-gam)>abs(p.ellest-gam)){
			ellUXc <- ellestU
			probXc <- p.ellest
		}
	}
	if(alternative != "less"){
# determine ellLXp and probXp for which probXp comes closest 
# to gam but >= gam among the three possibilities ellhatL,
# elltildeL, and ellestL, with corresponding p.ellhat, p.elltilde, p.ellest
		ellLXp <- ellhatL
		probXp <- p.ellhat
	        if(p.elltilde >= gam & p.elltilde < probXp){
			ellLXp <- elltildeL
	                probXp <- p.elltilde
		}
	        if(p.ellest >= gam & p.ellest < probXp){
			ellLXp <- ellestL
	                probXp <- p.ellest
		}
		ellLXc <- ellhatL
		probXc <- p.ellhat
# determine ellLXc and probXc for which probXc comes closest 
# to gam among the three possibilities ellhatL,
# elltildeL, and ellestL, with corresponding p.ellhat, p.elltilde, p.ellest
		if(abs(probXc-gam)>abs(p.elltilde-gam)){
			ellLXc <- elltildeL
			probXc <- p.elltilde
		}
		if(abs(probXc-gam)>abs(p.ellest-gam)){
			ellLXc <- ellestL
			probXc <- p.ellest
		}
	}
}
if(alternative != "greater"){
	istart <- n1
	for( i in 1:s ){
		D <- sort(outer(x[(istart+1):(istart+ni[i])],x[1:n1],"-"))
		if(ellUp[i] > n1*ni[i]){ 
			Uboundp[i] <- Inf}else{
			Uboundp[i] <- D[ellUp[i]]
		}
		if(ellUc[i] > n1*ni[i]){ 
			Uboundc[i] <- Inf}else{
			Uboundc[i] <- D[ellUc[i]]
		}
		if(method != "asymptotic"){
			if(ellUXp[i] > n1*ni[i]){ 
				UboundXp[i] <- Inf}else{
				UboundXp[i] <- D[ellUXp[i]]
			}
			if(ellUXc[i] > n1*ni[i]){ 
				UboundXc[i] <- Inf}else{
				UboundXc[i] <- D[ellUXc[i]]
			}
		}
		istart <- istart + ni[i]
	}
}
if(alternative != "less"){
	istart <- n1
	for( i in 1:s ){
		D <- sort(outer(x[(istart+1):(istart+ni[i])],x[1:n1],"-"))
		if(ellLp[i] < 1){
			Lboundp[i] <- -Inf}else{	
			Lboundp[i] <- D[ellLp[i]]
		}
		if(ellLc[i] < 1){
			Lboundc[i] <- -Inf}else{	
			Lboundc[i] <- D[ellLc[i]]
		}
		if(method != "asymptotic"){
			if(ellLXp[i] < 1){
				LboundXp[i] <- -Inf}else{	
				LboundXp[i] <- D[ellLXp[i]]
			}
			if(ellLXc[i] < 1){
				LboundXc[i] <- -Inf}else{	
				LboundXc[i] <- D[ellLXc[i]]
			}
		}
		istart <- istart + ni[i]
	}
}


Delta <- paste("Delta_",1:s,sep="")
LBound <- Lboundp
UBound <- Uboundp
LBoundc <- Lboundc
UBoundc <- Uboundc
levelA <- c(round(achieved.confidence.p,6),rep("",length(LBound)-1))
levelB <- c(round(achieved.confidence.c,6),rep("",length(LBound)-1))

outA<- data.frame(L=LBound,U=UBound,level=levelA,row.names=Delta)
outB<- data.frame(L=LBoundc,U=UBoundc,level=levelB,row.names=Delta)
if(alternative=="greater"){
	ellUc <- NA
	ellUp <- NA
	ellUXc <- NA
	ellUXp <- NA
}
if(alternative=="less"){
	ellLc <- NA
	ellLp <- NA
	ellLXc <- NA
	ellLXp <- NA
}
i.LU <- cbind(ellLp,ellUp,ellLc,ellUc)
dimnames(i.LU) <- list(NULL,c("i.L","i.U","i.Lc","i.Uc"))
i.LUX <- NA
if(method=="asymptotic"){
out <- list(conservative.bounds.asymptotic = outA,
	closest.bounds.asymptotic=outB)
}else{
	i.LUX <- cbind(ellLXp,ellUXp,ellLXc,ellUXc)
	dimnames(i.LUX) <- list(NULL,c("i.L","i.U","i.Lc","i.Uc"))
	levelAX <- c(round(probXp,6),rep("",length(LboundXp)-1))
	levelBX <- c(round(probXc,6),rep("",length(LboundXp)-1))
	outAX <- data.frame(L=LboundXp,U=UboundXp,level=levelAX,row.names=Delta)
	outBX <- data.frame(L=LboundXc,U=UboundXc,level=levelBX,row.names=Delta)

	if(method == "simulated"){
		
		out <- list(conservative.bounds.asymptotic = outA, 
			closest.bounds.asymptotic=outB,
			conservative.bounds.simulated = outAX, 
			closest.bounds.simulated=outBX)
	}
	if(method == "exact"){
		out <- list(conservative.bounds.asymptotic = outA, 
			closest.bounds.asymptotic=outB,
			conservative.bounds.exact = outAX, 
			closest.bounds.exact=outBX)
	}
}

outx <- list(test.name = "Steel.bounds",
	 	n1 = n1, ns = ni, N = nsamp,n.ties=n.ties,
		bounds=out,method=method,Nsim=Nsim,i.LU=i.LU,i.LUX=i.LUX)
class(outx) <- "kSamples"
outx		
}

Harding <-
function (nn) 
{
nn <- sort(as.integer(nn))
nvec <- rev(cumsum(rev(nn)))
k <- length(nn)
L1 <- sum(nn[1:(k-1)]*nvec[2:k])+1
freq <- double(L1)

out <- .C("Harding0", k=as.integer(k), L1=as.integer(L1),
		nn=as.integer(nn), nvec=as.integer(nvec),
		freq=as.double(freq), PACKAGE = "kSamples")

freq <- out$freq

if(is.nan(sum(freq)) ||
        abs(sum(freq)-1) > .0000001) freq <- NaN

freq
}

io <-
function(...,data=NULL) 
{
  # This function can take a bunch of numeric sample vectors 
  # in ..., or a list of such, or a formula that specifies a
  # a response (e.g., y), grouped by a factor, e.g., g, of same
  # length as y, via y ~ g. 
  # This breaks down y into the desired samples, one sample 
  # for each factor level. 
    xlist <- list(...)
    if(is(xlist[[1L]], "formula")) {
	cl <- match.call()  # gets a copy of the current call
    	mf <- cl 
    	mf[[1L]] <- as.name("model.frame") 
    	mf <- eval(mf,parent.frame())
    # mf is a data frame consisting of response 
    # and the other model variable
    	mt <- attr(mf, "terms")
    	y <- model.response(mf, "numeric") # response vector
    	fname <- attr(mt,"term.labels") 
    # fname contains the names of explanatory variables
    	if(length(fname) != 1) {
      	stop("Please specify only one term in the formula")
    	}
    	fvec <- as.factor(mf[, fname])
    # fvec contains the values of the single explanatory variable
    # as a factor
    samples <- lapply(levels(fvec), 
                      function(flvl) {
                        return(y[which(fvec == flvl)])} )
  } else {
    # tests whether ... is a list, when not a formula
    	if (is.list(xlist[[1]])) {
      	samples <- xlist[[1]]
    	} else {
	if( all(unlist(lapply(xlist,FUN=is.numeric))) == FALSE) stop("improper input for ...\n")
        samples <- lapply(xlist,as.numeric)
     }
    
  }
  if(length(samples) < 2) stop("fewer than 2 samples\n")
  if(all(unlist(lapply(samples,FUN=is.numeric))) != TRUE) stop("improper input for ...\n")
  lapply(samples,as.numeric)
  
}

Steelnormal0 <- function(sig0,sig,tau,U,ni,
		alternative=c("greater","two-sided")){
# this function computes the normal approximation of the p-value for the Steel test,
# based on the sizes ni = c(n1,...,nk) of the k treatment samples
# based on the observed Steel statistic U
# sig0, sig and tau are parameters required for the power evaluation. 
alternative <- match.arg(alternative)

k <- length(ni)
if(alternative=="greater"){ 
	funz <- function(z,k,sig0,sig,tau,S,ni){
		fac <- 1
		for(i in 1:k){
			fac <- fac * pnorm((S*tau[i]-ni[i]*sig0*z)/sig[i])
		}
		dnorm(z)*fac
	}
	N <- length(U)
	pval <- numeric(N)
	for(i in 1:N){
		pval[i] <- 1-integrate(funz,-Inf,Inf,k,sig0,sig,tau,U[i],ni)$value
	}
}


if(alternative=="two-sided"){ 

	funz <- function(z,k,sig0,sig,tau,S,ni){
			fac <- 1
			for(i in 1:k){
				fac <- fac * (pnorm((S*tau[i]-ni[i]*sig0*z)/sig[i])-
					pnorm((-S*tau[i]-ni[i]*sig0*z)/sig[i]))
			}
			dnorm(z)*fac
	}
	N <- length(U)
	pval <- numeric(N)
	for(i in 1:N){
		pval[i] <- 1-integrate(funz,-Inf,Inf,k,sig0,sig,tau,U[i],ni)$value
	}
}
pval
}

ad.test.combined <-
function (..., data = NULL, method = c("asymptotic","simulated","exact"),
	dist = FALSE, Nsim = 10000) 
{
#############################################################################
# This function ad.test.combined combines several Anderson-Darling 
# K-sample test statistics AD.m, m = 1,...,M, into one overall test 
# statistic AD.combined as suggested in Section 8 of Scholz F.W. and 
# Stephens M.A. (1987), K-sample Anderson-Darling Tests,
# Journal of the American Statistical Association, Vol 82, No. 399, 
# pp. 918-924.
# See also the documentation of ad.test for the comparison of a single 
# set of K samples.
# Each application of the Anderson-Darling K-sample test can be 
# based on a different K > 1. This combined version tests the hypothesis 
# that all the hypotheses underlying the individual K-sample tests are 
# true simultaneously.
# The individual K-sample test hypothesis is that all samples from 
# the m-th group come from a common population. However, that population 
# may be different from one individual K-sample situation to the next. 
# Such a combined test is useful in 
#
# 1) examining intra-laboratory measurement equivalence, when samples from 
#    the same material or batch are compared for M laboratories and such
#    comparisons are made for samples from several different materials or
#    batches and one assessment over all materials/batches is desired.
#
# 2) analyzing treatment effects in randomized complete or incomplete 
#    block designs.
#
# When there are NA's among the sample values they are removed,
# with a warning message indicating the number of NA's.
# It is up to the user to judge whether such removals make sense.
#
# Input: ...  
#        can take the form of several lists, 
#        say L.1,...,L.M, where list L.i contains 
#        K.i sample vectors of respective sizes n.i[1], ..., n.i[K.i] 
#        (n.i[j] > 4 is recommended)
#
#        or a single list of such lists
#
#        or a data frame with 3 columns, the first column representing 
#	 the responses y, the second column a factor g the levels of 
#  	 which are used to indicate the samples within each block, and 
#	 the third column a factor b indicating the block
#
#        or a formula y ~ g | b with y, g, b as in the previous situation,
#        where y, g, b may be variable names in a provided data frame, say dat,
#	 supplied via data = dat,
#
#        or just the three vectors y, g, b in this order with same meaning.
#
# data 
#  	an optional data frame containing the variable names used in formula input, 
#       default is NULL, in which case the used variables must exist in the calling
#       environment.
#
# method 
#       can take one of three values "asymptotic", "simulated",
#		and "exact", which determines the mode of P-value calculation.
#       The asymptotic P-value is always returned.
#       The simulated P-value simulates splits of the pooled samples in 
#		the i-th list L.i into K.i samples of sizes n.i[1], ..., n.i[K.i],
#       computing the corresponding AD statistic AD.i (both versions),
#       doing this independently for i = 1,...,M and adding the AD.i
#       to get the combined statistic AD.comb. This is repeated Nsim 
#       times and the simulated P-value is the proportion of these 
#       values that are >= the observed combined value.
#       The exact P-value should only be attempted for small M and small
#       sample sizes and requires that Nsim be set to >= the total number
#       of AD.comb enumerations. Otherwise Nsim simulations are run
#       to get a simulated P-value, as described above.
#       As example consider: M=2 with K.1 = 2, n.1[1] = 5, n.1[2] = 6,
#		K.2 = 2, n.2[1] = 5, n.2[2] = 7, then we would have
#       choose(11,5) = 462 splits of the first set of pooled samples
#       and choose(12,5) = 792 splits of the second set of pooled samples
#       and thus 462 * 792 = 365904 combinations of AD.1+AD.2 = AD.comb.
#       Thus we would need to set Nsim >= 365904 to enable exact 
#       exact enumeration evaluation of the P-value. Since these enumerated
# 		values of AD.comb need to be held inside R in a single vector,
#  		we are limited by the object size in R. In simulations the length
#       of the simulated vector of AD.comb is only Nsim and is manageable.
#
# dist
#       takes values FALSE (default) or TRUE, where TRUE enables the 
#		return of the simulated or exact vectors of generated values
#       for both versions of AD.comb.
#       Otherwise NULL is returned for both versions
#
# Nsim  = 10000 (default), number of simulations as discussed above.
#       
#       
#
# An example:
# x1 <- c(1, 3, 2, 5, 7), x2 <- c(2, 8, 1, 6, 9, 4), and 
# x3 <- c(12,  5,  7,  9, 11)
# y1 <- c(51, 43, 31, 53, 21, 75) and y2 <- c(23, 45, 61, 17, 60)
# then 
# set.seed(2627)
# ad.test.combined(list(x1,x2,x3),list(y1,y2),method="simulated",Nsim=100000) 
# or equivalently ad.test.combined(list(list(x1,x2,x3),list(y1,y2)),
#	method="simulated",Nsim=100000)
# produces the outcome below.
##########################################################################
# 
# Combination of Anderson-Darling K-Sample Tests.
# 
# Number of data sets = 2 
# 
# Sample sizes within each data set:
# Data set 1 :  5 6 5
# Data set 2 :  6 5
# Total sample size per data set: 16 11 
# Number of unique values per data set: 11 11 
# 
# AD.i = Anderson-Darling Criterion for i-th data set
# Means: 2 1 
# Standard deviations: 0.92837 0.64816 
# 
# T.i = (AD.i - mean.i)/sigma.i
#  
# Null Hypothesis:
# All samples within a data set come from a common distribution.
# The common distribution may change between data sets.
# 
# Based on Nsim = 1e+05 simulations
# for data set 1 we get
#               AD   T.AD  asympt. P-value  sim. P-value
# version 1: 3.316 1.4176         0.088063       0.09868
# version 2: 3.510 1.6286         0.070278       0.09115
# 
# for data set 2 we get
#                 AD     T.AD  asympt. P-value  sim. P-value
# version 1: 0.37267 -0.96786          0.96305       0.94529
# version 2: 0.33300 -1.02930          0.98520       0.93668
# 
#
# Combined Anderson-Darling Criterion: AD.comb = AD.1+AD.2 
# Mean = 3    Standard deviation = 1.13225 
# 
# T.comb = (AD.comb - mean)/sigma
# 
# Based on Nsim = 1e+05 simulations
#            AD.comb    T.comb  asympt. P-value  sim. P-value
# version 1: 3.68867 0.6082333        0.2205062       0.23733
# version 2: 3.84300 0.7445375        0.1902867       0.21825
#
#
###############################################################################
# For out.ad.combined <- ad.test.combined(list(x1,x2,x3),list(y1,y2))
# or out.ad.combined <- ad.test.combined(list(list(x1,x2,x3),list(y1,y2)))
# we get the object out.ad.combined of class ksamples with the following 
# components
# > names(out.ad.combined)
# > names(ad.c.out)
#  [1] "test.name"  "M"          "n.samples"  "nt"         "n.ties"    
#  [6] "ad.list"    "mu"         "sig"        "ad.c"       "mu.c"      
# [11] "sig.c"      "warning"    "null.dist1" "null.dist2" "method"    
# [16] "Nsim" 
# where 
# test.name = "Anderson-Darling"
# M = number of sets of samples being compared
# n.samples = is a list of the vectors giving the sample sizes for each 
#             set of samples being compared
# nt = vector of total sample sizes involved in each of the M comparisons
# n.ties = vector of lenth M giving the number of ties in each comparison group
# ad.list = list of M data frames for the results for each of the test results
#           corresponding to the M block
#
# mu = vector of means of the M AD statistics
# sig = vector of standard deviations of the M AD statistics
# ad.c = 2 x 3 (or 2 x 4) matrix containing the AD statistics, 
#		standardized AD statistics, its asymptotic P-value, 
#      	(and its exact or simulated P-value), for version 1 in the first row 
#		and for version 2 in the second row.
# mu.c = mean of the combined AD statistic
# sig.c = standard deviation of the combined AD statistic
# warning = logical indicator, warning = TRUE when at least one of 
#           the sample sizes is < 5.
# null.dist1 enumerated or simulated values of AD statistic, version 1
# null.dist2 enumerated or simulated values of AD statistic, version 2
# method the method that was used: "asymptotic", "simulated", "exact".
# Nsim the number of simulations that were used.
#
# Fritz Scholz, August 2012
#####################################################################################
convvec <- function(x1,x2){
#----------------------------------------------------
# R routine for calling convvec in conv.c
# created 02.05.2012  Fritz Scholz
#----------------------------------------------------
n1 <- length(x1)
n2 <-length(x2)
n <- n1*n2
x <- numeric(n)
 out <- .C("convvec",  x1=as.double(x1), n1=as.integer(n1), 
					x2=as.double(x2),n2=as.integer(n2), 
					x=as.double(x),n=as.integer(n), PACKAGE= "kSamples")
out$x
}


# the following converts individual data sets into a list of such,
# if not already in this form.

data.sets <- io2(...,data=data)
data.sets <- test.list(data.sets)

# end of data.sets list conversion

method <- match.arg(method)
n.sizes <- NULL
M <- length(data.sets) # number of data sets (blocks)

n.data <- sapply(data.sets, length) 
		# gets vector of number of samples in each component of data.sets
n.samples <- list() # intended to contain vectors of sample sizes for 
                    # for each respective data set.
na.t <- 0 # intended to tally total of NA cases
ncomb <- 1 # intended to hold the total number of evaluations 
           # of the full convolution distribution, to check
           # whether exact computation is reasonable. 

for(i in 1:M){
	out <- na.remove(data.sets[i])
	na.t<- na.t + out$na.total
	data.sets[i] <- out$x.new # replace data set i with the one that has
                              # NA's removed
	n.sample <- sapply(data.sets[[i]], length) 
				# contains sample sizes for i-th data set
	n.sizes <- c(n.sizes, n.sample) 
				# accumulates all sample size, warning purpose only
	if(any(n.sample==0))
		stop(paste("one or more samples in data set", i,
                       "has no observations"))
	n.samples[[i]] <- n.sample 
	N <- sum(n.sample) # total observations in i-th data set
    	k <- length(n.sample) # number of samples in i-th data set

    # updating ncomb
   	ncomb <- ncomb * choose(N,n.sample[1])
	for(j in 2:k){
		N <- N-n.sample[j-1]
		ncomb <- ncomb * choose(N,n.sample[j])
	} # end of ncomb update for data set i
}
Nsim <- min(Nsim,1e7) 

if(ncomb > Nsim & method == "exact") method <- "simulated"	

if( na.t > 1) cat(paste("\n",na.t," NAs were removed!\n\n"))
if( na.t == 1) cat(paste("\n",na.t," NA was removed!\n\n"))

warning <- min(n.sizes) < 5 # set warning flag for caution on
                            # trusting asymptotic p-values


# Initializing output objects
AD <- 0
sig <- NULL
n.ties <- NULL
nt <- NULL
mu <- NULL
ad.list <- list()
mu.c <- 0
dist1 <- NULL
dist2 <- NULL
if(method == "asymptotic"){dist0 <- FALSE}else{dist0 <- TRUE}
# the following loop aggregates the (estimated or exact)
# convolution distribution of the combined AD statistic versions
for(i in 1:M){
	out <- ad.test(data.sets[[i]],method=method,dist=dist0,Nsim=Nsim)
	if(dist0==TRUE){
	    if(i == 1){
			dist1 <- out$null.dist1
			dist2 <- out$null.dist2
		}else{
			if(method=="simulated"){
				dist1 <- dist1+out$null.dist1
				dist2 <- dist2+out$null.dist2
			}else{
				dist1 <- convvec(dist1,out$null.dist1)
				dist2 <- convvec(dist2,out$null.dist2)
			}
		}
	}
	ad.list[[i]] <- out$ad
	sig.i <- out$sig
	mu <- c(mu, length(data.sets[[i]])-1)
	AD.i <- out$ad[,1]
	sig <- c(sig, sig.i) # accumulated stand. dev.'s of AD stats
	AD <- AD+AD.i # aggregates combined AD stats (version 1 and 2)
	mu.c <- mu.c + length(data.sets[[i]]) - 1 # aggregates mean of combined AD stats
	n.ties <- c(n.ties, out$n.ties)
	nt <- c(nt, sum(out$ns)) # accumulates total observations in data sets
}
AD <- as.numeric(AD)
# get exact or simulated P-value
if(dist0==T){
    	nrow <- length(dist1)
	ad.pval1.dist <- sum(dist1 >= AD[1])/nrow
	ad.pval2.dist <- sum(dist2 >= AD[2])/nrow
}
sig.c <- sqrt(sum(sig^2)) # standard deviation of combined AD stats
if(sig.c >0){
	tc.obs <- (AD - mu.c)/sig.c # standardized values of AD stats
	}else{
	tc.obs <- NA
}

# get asymptotic P-value
if(sig.c >0){
	ad.pval1 <- ad.pval(tc.obs[1], mu.c,1)
	ad.pval2 <- ad.pval(tc.obs[2], mu.c,2)
	}else{
	ad.pval1 <- 1
	ad.pval2 <- 1
}

    if(method == "asymptotic"){
		ad.c <- matrix(c(signif(AD[1],7),signif(tc.obs[1],7),round(ad.pval1,7),
           signif(AD[2],7),signif(tc.obs[2],7), round(ad.pval2,7)),
           byrow=TRUE, ncol=3)
		dimnames(ad.c) <- list(c("version 1:","version 2:"),
						c("AD.comb","T.comb"," asympt. P-value"))
    }
    if(method == "exact"){
		ad.c <- matrix(c(signif(AD[1],7),signif(tc.obs[1],7),round(ad.pval1,7),
		   round(ad.pval1.dist,7),
           signif(AD[2],7),signif(tc.obs[2],7), round(ad.pval2,7),
		   round(ad.pval2.dist,7)),byrow=TRUE, ncol=4)
		dimnames(ad.c) <- list(c("version 1:","version 2:"),
						c("AD.comb","T.comb"," asympt. P-value"," exact P-value"))
    }
    if(method == "simulated"){
		ad.c <- matrix(c(signif(AD[1],7),signif(tc.obs[1],7),round(ad.pval1,7),
		   round(ad.pval1.dist,7),
           signif(AD[2],7),signif(tc.obs[2],7), round(ad.pval2,7),
		   round(ad.pval2.dist,7)),byrow=TRUE, ncol=4)

		dimnames(ad.c) <- list(c("version 1:","version 2:"),
						c("AD.comb","T.comb"," asympt. P-value"," sim. P-value"))
    }

if(dist==FALSE){
	dist1 <- NULL
	dist2 <- NULL
}


object <- list(test.name ="Anderson-Darling",
				M=M, n.samples=n.samples, nt=nt, n.ties=n.ties, ad.list=ad.list,
           		mu=mu, sig=sig, ad.c = ad.c, mu.c=mu.c,
           		sig.c=round(sig.c,5), warning=warning,null.dist1=dist1,
				null.dist2=dist2,method=method,Nsim=Nsim)
class(object) <-  "kSamples"
object
}

\name{SteelConfInt}
\alias{SteelConfInt}
\title{
Simultaneous Confidence Bounds Based on Steel's Multiple Comparison Wilcoxon Tests
}
\description{
This function inverts pairwise Wilcoxon tests, comparing a common control sample with each of
several treatment samples to provide simultaneous confidence bounds
for the respective shift parameters by which the sampled treatment populations may differ
from the control population. It is assumed that all samples are independent 
and
that the sampled distributions are continuous to avoid ties.
The joint coverage probability for all bounds/intervals is calculated, estimated, or approximated, see
Details. For treatment of ties also see Details.
}
\usage{
SteelConfInt(\dots, data = NULL, conf.level = 0.95, 
	alternative = c("less", "greater", "two.sided"), 
     	method = c("asymptotic", "exact", "simulated"), Nsim = 10000)
}
\arguments{
	\item{\dots}{
		Either several sample vectors, say 
		\eqn{x_1, \ldots, x_k}, 
		with \eqn{x_i} containing \eqn{n_i} sample values.
		\eqn{n_i > 4} is recommended for reasonable asymptotic 
		\eqn{P}-value calculation. The pooled sample size is denoted 
		by \eqn{N=n_1+\ldots+n_k}. The first vector serves as control sample,
		the others as treatment samples.

		or a list of such sample vectors.

		or a formula y ~ g, where y contains the pooled sample values 
		and g (same length as y) is a factor with levels identifying 
		the samples to which the elements of y belong. The lowest factor level
		corresponds to the control sample, the other levels to treatment samples. 
	}

    	\item{data}{= an optional data frame providing the variables in formula y ~ g.
		}

    \item{conf.level}{\code{= 0.95} (default) the target joint confidence level for all bounds/intervals.

		\code{0 < conf.level < 1}.
    }

    \item{alternative}{= \code{c("less", "greater", "two.sided")}, where \code{"less"} results in
		simultaneous upper confidence bounds for all shift parameters and any negative upper bound
		should lead to the rejection of the null hypothesis of all shift parameters 
		being zero or positive in favor of at least one being less than zero.

		\code{"greater"} results in simultaneous lower confidence bounds for all shift parameters 
		and any positive lower bound should lead to the rejection of the null hypothesis of all
		shift parameters being zero or negative in favor of at least one being greater 
		than zero.


		\code{"two.sided"} results in simultaneous confidence intervals for all shift parameters 
		and any interval not straddling zero should lead to the rejection of the null hypothesis 
		of all shift parameters being zero in favor of at least one being different from zero.
    }

    \item{method}{= \code{c("asymptotic", "exact", "simulated")}, where

		\code{"asymptotic"} uses only an asymptotic normal approximation 
		to approximate the achieved joint coverage probability. 
		This calculation is always done.

		\code{"exact"} uses full enumeration of all sample splits to obtain the exact achieved 
		joint coverage probability (see Details).
     		It is used only when \code{Nsim} is at least as large as the number of full enumerations. 
		Otherwise, \code{method} reverts to \code{"simulated"} using the given \code{Nsim}.

		\code{"simulated"} uses \code{Nsim} simulated random splits of the 
		pooled samples into samples of sizes \eqn{n_1, \ldots, n_k}, 
		to estimate the achieved joint coverage probability.


    }

    \item{Nsim}{\code{= 10000} (default), number of simulated sample splits to use.	
		It is only used when \code{method = "simulated"},
		or when \code{method = "exact"} reverts to \code{method =}
		\code{"simulated"}, as previously explained.
    }

}
\details{The first sample is treated as control sample with sample size \eqn{n_1}. The remaining 
\eqn{s=k-1} samples are treatment samples.
Let \eqn{W_{1i}, i=2,\ldots,k} denote the respective Wilcoxon statistics comparing the common control sample (index 1)
with each of the \eqn{s} treatment samples (indexed by \eqn{i}). 
For each comparison of control and treatment \eqn{i}
sample
only the observations of the two samples involved are ranked.
By \eqn{W_i=W_{1i}-n_i(n_i+1)/2} we denote
the corresponding Mann-Whitney test statistic.
Furthermore, let \eqn{D_{i(j)}} denote the \eqn{j}-th ordered value (ascending order) of the \eqn{n_1n_i}
paired differences between the observations in treatment sample \eqn{i} and those of the control
sample. By simple extension of results in Lehmann (2006), pages 87 and 92, the following equations hold, 
relating the null distribution of the 
Mann-Whitney statistics and the joint coverage probabilities of the \eqn{D_{i(j_i)}} for any set of
\eqn{j_1,\ldots,j_s} with \eqn{1\le j_i \le n_1 n_i}.
\deqn{P_\Delta(\Delta_i \le D_{i(j_i)}, i=2,\ldots,k)=P_0(W_i\le j_i -1, i=2,\ldots,k)}
and
\deqn{P_\Delta(\Delta_i \ge D_{i(j_i)}, i=2,\ldots,s)=P_0(W_{i}\le n_1 n_i -j_i, i=2,\ldots,k)}
where \eqn{P_\Delta} refers to the distribution under \eqn{\Delta=(\Delta_2,\ldots,\Delta_k)}
and \eqn{P_0} refers to the joint null distribution of the \eqn{W_i} when all sampled 
distributions are the same and continuous. There are \eqn{k-1} indices \eqn{j_i} that can be manipulated
to affect the achieved confidence level. To limit the computational complexity
standardized versions of the \eqn{W_i}, i.e.,  \eqn{(W_i-\mu_i)/\tau_i} with 
\eqn{\mu_i} and \eqn{\tau_i} representing mean and standard deviation of \eqn{W_i},
are used to choose a common value for \eqn{(j_i -1-\mu_i)/\tau_i}  (satisfying the 
\eqn{\gamma} level) from the multivariate normal approximation 
for the \eqn{W_i} (see Miller (1981) and Scholz (2016)), and reduce that 
to integer values for \eqn{j_i}, rounding up, rounding down, and rounding to the nearest integer. These
integers  \eqn{j_i} are then used in approximating the actual joint probabilities
\eqn{P_0(W_i\le j_i -1, i=2,\ldots,k)}, and from these three coverage probabilities 
the one that is closest to the nominal confidence level \eqn{\gamma} and \eqn{\ge \gamma}
and also also the one that is closest without the restriction \eqn{\ge \gamma} are chosen.

When \code{method = "exact"} or \code{= "simulated"} is specified, the same process
is used, using either the fully enumerated exact distribution of \eqn{W_i, i=2,\ldots,k} (based on a recursive 
version of Chase's sequence as presented in Knuth (2011)) for all sample splits,
or the simulated distribution of \eqn{W_i, i=2,\ldots,k}. However, since these distributions are discrete
the starting point before rounding up is the smallest quantile such that the proportion of distribution values less 
or equal to it is at least \eqn{\gamma}. The starting point before rounding down is the highest quantile such that 
the proportion of distribution values less 
or equal to it is at most \eqn{\gamma}. The third option of rounding to the closest integer is performed using 
the average of the first two.

Confidence intervals are constructed by using upper and lower confidence bounds, each with
same confidence level of \eqn{(1+\gamma)/2}.

When the original sample data appear to be rounded, and especially when there are ties,
one should widen the computed intervals or bounds by the rounding \eqn{\epsilon}, as illustrated
in Lehmann (2006), pages 85 and 94. For example, when all sample values appear to end in one of \eqn{.0, .2, .4, .6, .8},
the rounding \eqn{\epsilon} would be \eqn{.2}. Ultimately, this is a judgment call for the user. Such widening
of intervals will make the actually achieved confidence level \eqn{\ge} the stated achieved level. 
}
\value{
A list of class \code{kSamples} with components 
\item{test.name}{\code{"Steel.bounds"}}
\item{n1}{the control sample size \eqn{= n_1}}
\item{ns}{vector \eqn{(n_2,\ldots,n_k)} of the \eqn{s=k-1} treatment sample sizes}
\item{N}{size of the pooled sample \eqn{= n_1+\ldots+n_k}}
\item{n.ties}{number of ties in the pooled sample}
\item{bounds}{a list of data frames. When \code{method = "asymptotic"} is specified, the list
consists of two data frames named \code{conservative.bounds.asymptotic}
and \code{closest.bounds.asymptotic}. Each data frame consists of \eqn{s} rows
corresponding to the \eqn{s} shift parameters \eqn{\Delta_i} and three columns,
the first column giving the lower bound, the second column the upper bound, while
the first row of the third column states the computed confidence level by asymptotic 
approximation, applying jointly to all \eqn{s} sets of bounds. For one-sided bounds
the corresponding other bound is set to \code{Inf} or \code{-Inf}, respectively.

In case of \code{conservative.bounds.asymptotic} the achieved asymptotic confidence level is 
targeted to be \eqn{\ge} \code{conf.level}, but closest to it among three possible choices (see Details).

In the case of \code{closest.bounds.asymptotic} the achieved level is targeted to
be closest to \code{conf.level}.

When \code{method = "exact"} or \code{method = "simulated"}
is specified the list consists in addition of two further data frames named either

\code{conservative.bounds.exact} and 
\code{closest.bounds.exact} or

\code{conservative.bounds.simulated} and 
\code{closest.bounds.simulated}. 

In either case the structure and meaning
of these data frames parallels that of the \code{"asymptotic"} case. 
}
\item{method}{the \code{method} used.}
\item{Nsim}{the number of simulations used.}
\item{j.LU}{an \eqn{s} by 4 matrix giving the indices \eqn{j_i} used for computing the bounds
\eqn{D_{i(j_i)}} for \eqn{\Delta_i, i=1,\ldots, s}.
}
}

\references{
Knuth, D.E. (2011), \emph{The Art of Computer Programming, Volume 4A 
Combinatorial Algorithms Part 1}, Addison-Wesley

Lehmann, E.L. (2006),
\emph{Nonparametrics, Statistical Methods Based on Ranks, Revised First Edition},
Springer Verlag.

Miller, Rupert G., Jr. (1981), \emph{Simultaneous Statistical Inference, Second Edition},
Springer Verlag, New York.

Scholz, F.W. (2023), "On Steel's Test with Ties", \url{https://arxiv.org/abs/2308.05873}
}
\section{warning}{\code{method = "exact"} should only be used with caution.
Computation time is proportional to the number of enumerations. 
Experiment with \code{\link{system.time}} and trial values for
\code{Nsim} to get a sense of the required computing time.}


\examples{
z1 <- c(103, 111, 136, 106, 122, 114)
z2 <- c(119, 100,  97,  89, 112,  86)
z3 <- c( 89, 132,  86, 114, 114, 125)
z4 <- c( 92, 114,  86, 119, 131,  94)
set.seed(2627)
SteelConfInt(list(z1,z2,z3,z4),conf.level=0.95,alternative="two.sided", 
   method="simulated",Nsim=10000)
# or with same seed
# SteelConfInt(z1,z2,z3,z4,conf.level=0.95,alternative="two.sided", 
#   method="simulated",Nsim=10000)
}

\keyword{nonparametric}
\keyword{htest}
\keyword{design}

\name{Steel.test}
\alias{Steel.test}
\title{
Steel's Multiple Comparison Wilcoxon Tests
}
\description{
This function uses pairwise Wilcoxon tests, comparing a common control sample with each of
several treatment samples, in a multiple comparison fashion. The experiment wise
significance probabity is calculated, estimated, or approximated, when
testing
the hypothesis that all independent samples arise
from a common unspecified distribution, or that treatments have no effect when assigned
randomly to the given subjects.
}
\usage{
Steel.test(\dots, data = NULL, 
	method = c("asymptotic", "simulated", "exact"),
	alternative = c("greater","less","two-sided"),
	continuity.corr = TRUE,
	dist = FALSE, Nsim = 10000)
}
\arguments{
	\item{\dots}{
		Either several sample vectors, say 
		\eqn{x_1, \ldots, x_k}, 
		with \eqn{x_i} containing \eqn{n_i} sample values.
		\eqn{n_i > 4} is recommended for reasonable asymptotic 
		\eqn{P}-value calculation. The pooled sample size is denoted 
		by \eqn{N=n_1+\ldots+n_k}. The first vector serves as control sample,
		the others as treatment samples.

		or a list of such sample vectors.

		or a formula y ~ g, where y contains the pooled sample values 
		and g (same length as y) is a factor with levels identifying 
		the samples to which the elements of y belong. The lowest factor level
		corresponds to the control sample, the other levels to treatment samples.
	}

    	\item{data}{= an optional data frame providing the variables in formula y ~ g or y, g input
		}

    \item{method}{= \code{c("asymptotic","simulated","exact")}, where

		\code{"asymptotic"} uses only an asymptotic normal approximation 
		to approximate the \eqn{P}-value, This calculation is always done.

		\code{"simulated"} uses \code{Nsim} simulated standardized
		Steel statistics based on random splits of the 
		pooled samples into samples of sizes 
		\eqn{n_1, \ldots, n_k}, to estimate the \eqn{P}-value.

		\code{"exact"} uses full enumeration of all sample splits with resulting 
		standardized Steel statistics to obtain the exact \eqn{P}-value. 
     		It is used only when \code{Nsim} is at least as large as the number
 		\deqn{ncomb = \frac{N!}{n_1!\ldots n_k!}}{N!/(n_1!\ldots n_k!)}
		of full enumerations. Otherwise, \code{method} reverts to 
		\code{"simulated"} using the given \code{Nsim}. It also reverts
		to \code{"simulated"} when \eqn{ncomb > 1e8} and \code{dist = TRUE}.
    }
    \item{alternative}{= \code{c("greater","less","two-sided")}, where for \code{"greater"} the 
		maximum of the pairwise standardized Wilcoxon test statistics is used and 
                a large maximum value is judged significant.
		For \code{"less"} the minimum of the pairwise standardized Wilcoxon test 
		statistics is used and a low minimum value is judged significant.
		For \code{"two-sided"} the maximum of the absolute pairwise standardized Wilcoxon test 
		statistics is used and a large maximum value is judged significant.
		
    }
    \item{continuity.corr}{\code{= TRUE}(default) or \code{FALSE}. If \code{TRUE}, a step .25 or .5
                is added to or subtracted from the observed Mann-Whitney statistic for
                the compared samples, depending on existence of ties in those two samples or not,
                when calculating the normal approximation for the adjusted \eqn{P}-value.
    }
    \item{dist}{\code{= FALSE} (default) or \code{TRUE}. If \code{TRUE}, the 
		simulated or fully enumerated null distribution vector \code{null.dist} 
		is returned for the Steel test statistic, as chosen via \code{alternative}.
		Otherwise, \code{NULL} is returned. When \code{dist = TRUE} then 
		\code{Nsim <- min(Nsim, 1e8)}, to limit object size.
    }
	\item{Nsim}{\code{= 10000} (default), number of simulation sample splits to use.	
		It is only used when \code{method = "simulated"},
		or when \code{method = "exact"} reverts to \code{method =}
		\code{ "simulated"}, as previously explained.
	}
}
\details{
The Steel criterion uses the Wilcoxon test statistic in the pairwise comparisons of the 
common control sample with each of the treatment samples. These statistics are used in 
standardized form, using the means and standard deviations as they apply conditionally
given the tie pattern in the pooled data, see Scholz (2023). This conditional treatment allows for 
correct usage in the presence of ties and is appropriate either when the samples are independent
and come from the same distribution (continuous or not) or when treatments are assigned 
randomly among the total of \code{N} subjects. However, in the case of ties the significance probability
has to be viewed conditionally given the tie pattern.

The Steel statistic is used to test the hypothesis that the samples all come 
from the same but unspecified distribution function \eqn{F(x)}, or, under random treatment 
assigment, that the treatments have no effect. The significance probability is the probability
of obtaining test results as extreme or more extreme than the observed test statistic,
when testing for the possibility of a treatment effect under any of the treatments.

For small sample sizes exact (conditional) null distribution
calculations are possible (with or without ties), based on a recursively extended
version of Algorithm C (Chase's sequence) in Knuth (2011), which allows the 
enumeration of all possible splits of the pooled data into samples of
sizes of \eqn{n_1, \ldots, n_k}, as appropriate under treatment randomization. This 
is done in C, as is the simulation of such splits.

NA values are removed and the user is alerted with the total NA count.
It is up to the user to judge whether the removal of NA's is appropriate.

}
\value{
A list of class \code{kSamples} with components 
\item{test.name}{\code{"Steel"}}
\item{alternative}{ "greater", "less", or "two-sided"}
\item{k}{number of samples being compared, including the control sample as the first one}
\item{ns}{vector \eqn{(n_1,\ldots,n_k)} of the \eqn{k} sample sizes}
\item{N}{size of the pooled sample \eqn{= n_1+\ldots+n_k}}
\item{n.ties}{number of ties in the pooled sample}
\item{st}{2 (or 3) vector containing the observed standardized Steel statistic, 
its asymptotic \eqn{P}-value, 
(its simulated or exact \eqn{P}-value)}
\item{warning}{logical indicator, \code{warning = TRUE} when at least one 
\eqn{n_i < 5}
}
\item{null.dist}{simulated or enumerated null distribution vector
of the test statistic. It is \code{NULL} when \code{dist = FALSE} or when
\code{method = "asymptotic"}.
}
\item{method}{the \code{method} used.}
\item{Nsim}{the number of simulations used.}
\item{W}{vector 
\eqn{(W_{12},\ldots, W_{1k})}
of Mann-Whitney statistics comparing each  observation under treatment \eqn{i (> 1)} 
against each observation of the control sample.
}
\item{Wstand}{standardized version of \code{W}, using \code{mu} and \code{tau} below.}
\item{pval.asympt.adj}{vector of adjusted \eqn{P}-values for each component of \code{Wstand} based the normal approximation for the Steel statistic. The smallest of these also serves as 
approximate \eqn{P}-value for the Steel test.}
\item{pval.adj}{vector of adjusted \eqn{P}-values for each component of \code{Wstand} based the exact or 
simulated distribution of \code{Wstand} if method \code{method = "simulated"} or \code{"exact"} is used.
The smallest of these also serves as simulated or exact \eqn{P}-value for the Steel test.}
\item{mu}{mean vector \eqn{(\mu_1,\ldots,\mu_{k-1})=(n_1n_2/2,\ldots,n_1n_k/2)} of \code{W}.} 
\item{tau}{vector \eqn{(\tau_1,\ldots,\tau_{k-1})} of standard deviations of \code{W}.} 

\item{sig0}{standard deviation used in calculating the significance probability
of the maximum (minimum) of (absolute) standardized Mann-Whitney statistics, see Scholz (2023).}
\item{sig}{vector 
\eqn{(\sigma_1,\ldots, \sigma_k)}
of standard deviations used in calculating the significance probability
of the maximum (minimum) of (absolute)  standardized Mann-Whitney statistics, see Scholz (2023).}

}
\references{
Knuth, D.E. (2011), \emph{The Art of Computer Programming, Volume 4A 
Combinatorial Algorithms Part 1}, Addison-Wesley

Lehmann, E.L. (2006),
\emph{Nonparametrics, Statistical Methods Based on Ranks, Revised First Edition},
Springer Verlag.

Scholz, F.W. (2023), "On Steel's Test with Ties", \url{https://arxiv.org/abs/2308.05873}
}
\section{warning}{\code{method = "exact"} should only be used with caution.
Computation time is proportional to the number of enumerations. 
Experiment with \code{\link{system.time}} and trial values for
\code{Nsim} to get a sense of the required computing time.
In most cases
\code{dist = TRUE} should not be used, i.e.,  
when the returned distribution objects 
become too large for R's work space.}


\examples{
z1 <- c(103, 111, 136, 106, 122, 114)
z2 <- c(119, 100,  97,  89, 112,  86)
z3 <- c( 89, 132,  86, 114, 114, 125)
z4 <- c( 92, 114,  86, 119, 131,  94)
y <- c(z1, z2, z3, z4)
g <- as.factor(c(rep(1, 6), rep(2, 6), rep(3, 6), rep(4, 6)))
set.seed(2627)
Steel.test(list(z1, z2, z3, z4), method = "simulated", 
  alternative = "less", continuity.corr = FALSE, Nsim = 1000)
# or with same seed
# Steel.test(z1, z2, z3, z4,method = "simulated", 
#   alternative = "less", continuity.corr = FALSE, Nsim = 1000)
# or with same seed
# Steel.test(y ~ g, method = "simulated", 
#   alternative = "less", continuity.corr = FALSE, Nsim=1000)
}

\keyword{nonparametric}
\keyword{htest}
\keyword{design}

\name{pp.kSamples}
\alias{pp.kSamples}
\title{
Upper Tail Probability Plots for Objects of Class kSamples
}
\description{
This function plots upper tail probabilities of the limiting distribution 
against the corresponding exact or simulated probabilities, both on a log-scale.
}
\usage{
pp.kSamples(x) 
}
\arguments{
  	\item{x}{an object of class \code{kSamples}}
	}

\details{Objects of class \code{kSamples} are produced by any of the following functions

\code{\link{ad.test}} Anderson-Darling k-sample test.

\code{\link{ad.test.combined}} Combined Anderson-Darling k-sample tests.

\code{\link{qn.test}} \eqn{QN} rank scores test.

\code{\link{qn.test.combined}} Combined \eqn{QN} rank scores tests.

\code{\link{contingency2xt}} test for \eqn{2 * t} contingency table.

\code{\link{contingency2xt.comb}} test for the combination of \eqn{2 * t} contingency tables.

\code{\link{jt.test}} Jonckheere-Terpstra test.

\code{\link{Steel.test}} Steel test. This will work only for alternative = "greater" or "two-sided".
The approximation quality for "less" is the same as for "greater".

The command \code{pp.kSamples(x)} for an object of class \code{kSamples}
will only produce a plot when the object \code{x} contains 
non-NULL entries for the null distribution. The purpose of this function is to give the user
a sense of the asymptotic distribution accuracy.
}


\seealso{
\code{\link{ad.test}},
\code{\link{ad.test.combined}},
\code{\link{qn.test}},
\code{\link{qn.test.combined}},

\code{\link{contingency2xt}},
\code{\link{contingency2xt.comb}}
\code{\link{jt.test}}
\code{\link{Steel.test}}
}

\examples{
qn.out <- qn.test(c(1,3,7,2,9),c(1,4,6,11,2),test="KW",
		method="simulated",dist=TRUE,Nsim=1000) 
pp.kSamples(qn.out)
}

\keyword{nonparametric}
\keyword{htest}

\name{ad.pval}
\alias{ad.pval}
\title{
\eqn{P}-Value for the Asymptotic Anderson-Darling Test Distribution
}
\description{
This function computes upper tail probabilities for the limiting distribution of the 
standardized Anderson-Darling test statistic.
}
\usage{
ad.pval(tx,m,version=1) 
}
\arguments{
  	\item{tx}{
		a vector of desired thresholds \eqn{\ge 0}
	}
    \item{m}{ 
		The degrees of freedom for the asymptotic standardized Anderson-Darling test statistic
    }
	\item{version}{\code{= 1} (default) if \eqn{P}-value for version 1 of the 
		test statistic is desired, otherwise the version 2 \eqn{P}-value is calculated.
	}
}
\details{
Extensive simulations (sampling from a common continuous distribution)
were used to extend the range of the asymptotic 
\eqn{P}-value calculation from the original \eqn{[.01,.25]} in Table 1 of the reference paper
to 36 quantiles corresponding to \eqn{P}  = .00001, .00005, .0001, .0005, .001, .005, .01, .025, 
.05, .075, .1, .2, .3, .4, .5, .6, .7, .8, .9, .925, .95, .975, .99, .9925, .995, .9975, .999,
.99925, .9995, .99975, .9999, .999925, .99995, .999975, .99999. Note that the entries of the original Table 1
were obtained by using the first 4 moments of the asymptotic distribution and a
Pearson curve approximation.

Using \code{ad.test},
1 million replications of the standardized \eqn{AD} statistics with sample sizes
\eqn{n_i=500}{n.i=500}, \eqn{i=1,\ldots,k} were run for \eqn{k=2,3,4,5,7} (\eqn{k=2} was done twice).
These values of \eqn{k} correspond to degrees of freedom
\eqn{m=k-1=1,2,3,4,6} in the asymptotic distribution. The random variable described by this 
distribution is denoted by \eqn{T_m}.
The actual variances (for \eqn{n_i=500}) agreed fairly well with the asymptotic variances.

Using the convolution nature of the asymptotic distribution, the performed simulations
were exploited to result in an effective simulation of 2 million cases, except for 
\eqn{k=11}, i.e., \eqn{m=k-1=10}, for which the asymptotic distribution of 
\eqn{T_{10}}
was approximated by the sum of the \eqn{AD} statistics for \eqn{k=7} and \eqn{k=5}, 
for just the 1 million cases run for each \eqn{k}. 

The interpolation of tail 
probabilities \eqn{P}
for any desired \eqn{k} is done in two stages. First, a spline in \eqn{1/\sqrt{m}}{1/sqrt(m)} is
fitted to each of the 36 quantiles obtained for \eqn{m=1,2,3,4,6,8,10,\infty} to obtain 
the corresponding interpolated quantiles for the \eqn{m} in question. 

Then a spline is fitted
to the \eqn{\log((1-P)/P)}{log((1-P)/P)} as a function of these 36 interpolated quantiles. This latter
spline is used to determine the tail probabilities \eqn{P} for the 
specified threshold \code{tx}, corresponding to either \eqn{AD} 
statistic version. The above procedure is based on simulations for either version
of the test statistic,
appealing to the same limiting distribution.
}
\value{
a vector of upper tail probabilities corresponding to \code{tx}
}
\references{
Scholz, F. W. and Stephens, M. A. (1987), K-sample Anderson-Darling Tests, 
\emph{Journal of the American Statistical Association}, 
\bold{Vol 82, No. 399}, 918--924. 
}



\seealso{
\code{\link{ad.test}},
\code{\link{ad.test.combined}}
}

\examples{
ad.pval(tx=c(3.124,5.65),m=2,version=1)
ad.pval(tx=c(3.124,5.65),m=2,version=2)
}

\keyword{nonparametric}
\keyword{htest}

\name{ShorelineFireEMS}
\alias{ShorelineFireEMS}
\docType{data}
\title{
Shoreline Fire and EMS Turnout Times
}
\description{
This data set gives turnout response times for Fire and EMS (Emergency Medical Services) 
dispatch calls to the Shoreline, WA, Fire Department in 2006. The turnout time refers to 
time elapsed between the emergency call dispatch and the crew leaving the fire station,
or signaling that they are on their way while being on route already. The latter scenario
may explain the bimodal distribution character.
}
\usage{
data(ShorelineFireEMS)
}

\format{
  	A list of two sublists \code{$EMSTOT} and \code{$FireTOT}, 
	each with 4 vector components \code{$ST57}, \code{$ST63}, \code{$ST64}, and
	\code{$ST65} respectively, giving the turnout times (in seconds) (for EMS and Fire)
	at fire stations ST57, ST63, ST64, and ST65.
}
\note{These data sets are provided to illustrate usage of \code{ad.test} and \code{qn.test} 
and their combined versions in testing for performance equivalence across fire stations.
}
\source{
Thanks to Michael Henderson and the Fire Fighters
and Paramedics of the Shoreline Fire Department in Washington State.
}

\examples{
data(ShorelineFireEMS)
boxplot(ShorelineFireEMS$EMSTOT,xlab="Station", ylab="seconds",
	main="EMS Turnout Time")
boxplot(ShorelineFireEMS$FireTOT,xlab="Station", ylab="seconds",
	main="Fire Turnout Time")
}
\keyword{datasets}

\name{JT.dist}
\alias{djt}
\alias{pjt}
\alias{qjt}
\title{
Null Distribution of the 
Jonckheere-Terpstra k-Sample Test Statistic
}
\description{
The
Jonckheere-Terpstra k-sample test statistic JT is defined
as \eqn{JT = \sum_{i<j} W_{ij}} where
\eqn{W_{ij}}{W_{ij}} is the Mann-Whitney statistic comparing 
samples \eqn{i} and \eqn{j}, indexed in the order 
of the stipulated increasing alternative. 
It is assumed that there are no ties
in the pooled samples.

This function uses Harding's algorithm as far as computations
are possible without becoming unstable.
}
\usage{
djt(x, nn)

pjt(x, nn)

qjt(p, nn)
}
\arguments{
  	\item{x}{a numeric vector, typically integers}
		
	\item{nn}{a vector of integers, representing the sample sizes
		in the order stipulated by the alternative}

	\item{p}{a vector of probabilities}

}
\details{While Harding's algorithm is mathematically correct, 
it is problematic in its computing implementation. 
The counts become very large and normalizing 
them by combinatorials leads to significance loss. 
When that happens the functions return an error message: 
can't compute due to numerical instability.
This tends to happen when the total number of sample values 
becomes too large.
That depends also on the way the sample sizes are allocated. 
}
\value{
For \code{djt} it is a vector 
\eqn{p = (p_1,\ldots,p_n)}
giving the values of 
\eqn{p_i = P(JT = x_i)}, where \code{n} is the length
of the input \code{x}.

For \code{pjt} it is a vector 
\eqn{P = (P_1,\ldots,P_n)}
giving the values of 
\eqn{P_i = P(JT \leq x_i)}{P_i = P(JT <= x_i)}.

For \code{qjt} is a vecto r \eqn{x = (x_1,\ldots,x_n)},where \eqn{x_i}
is the smallest \eqn{x}
such that \eqn{P(JT \leq x) \geq p_i}{P(JT <= x) >= p_i}.


}


\references{
Harding, E.F. (1984), An Efficient, Minimal-storage Procedure for
Calculating the Mann-Whitney U, Generalized U and Similar Distributions,
\emph{Appl. Statist.} \bold{33} No. 1, 1-6.


Jonckheere, A.R. (1954), A Distribution Free \emph{k}-sample Test against Ordered 
Alternatives,
\emph{Biometrika}, \bold{41}, 133-145.

Lehmann, E.L. (2006),
\emph{Nonparametrics, Statistical Methods Based on Ranks, Revised First Edition},
Springer Verlag.

Terpstra, T.J. (1952), The Asymptotic Normality and Consistency of Kendall's Test against Trend, when Ties are Present in One Ranking,
\emph{Indagationes Math.} \bold{14}, 327-333.
}




\examples{
djt(c(-1.5,1.2,3), 2:4)
pjt(c(2,3.4,7), 3:5)
qjt(c(0,.2,.5), 2:4)
}
\keyword{nonparametric}
\keyword{htest}
\keyword{design}

\name{contingency2xt}
\alias{contingency2xt}
\title{
Kruskal-Wallis Test for the 2 x t Contingency Table 
}
\description{
This function uses the Kruskal-Wallis criterion to test
the hypothesis of no association between the counts 
for two responses
"A" and "B" across t categories.
}
\usage{
contingency2xt(Avec, Bvec, 
	method = c("asymptotic", "simulated", "exact"), 
	dist = FALSE, tab0 = TRUE, Nsim = 1e+06)
}
\arguments{
  	\item{Avec}{
		vector of length \eqn{t} giving the counts \eqn{A_1,\ldots, A_t}
		for response "A" according to \eqn{t} categories. 
		\eqn{m = A_1 + \ldots + A_t}.
	}
  	\item{Bvec}{
		vector of length \eqn{t} giving the counts \eqn{B_1,\ldots, B_t}
		for response "B" according to \eqn{t} categories. 
		\eqn{n = B_1 + \ldots + B_t = N-m}.
	}
    \item{method}{= \code{c("asymptotic","simulated","exact")}, where

		\code{"asymptotic"} uses only an asymptotic chi-square approximation 
		with \eqn{t-1} degrees of freedom to approximate the \eqn{P}-value. 
		This calculation is always done.

		\code{"simulated"} uses \code{Nsim} simulated counts for \code{Avec} and 
		\code{Bvec} with the observed marginal totals, \code{m, n, d = Avec+Bvec}, 
		to estimate the \eqn{P}-value.

		\code{"exact"} enumerates all counts for \code{Avec} and \code{Bvec} with 
		the observed marginal totals to get an exact \eqn{P}-value. It is used only 
		when \code{Nsim} is at least as large as the number \code{choose(m+t-1,t-1)}
		of full enumerations. 
		Otherwise, \code{method} reverts to \code{"simulated"} using the given \code{Nsim}.
    }
	\item{dist}{\code{FALSE} (default) or \code{TRUE}. If \code{dist = TRUE}, the distribution of the
		simulated or fully enumerated Kruskal-Wallis test statistics is
		returned as \code{null.dist}, if \code{dist = FALSE} the value
		of \code{null.dist} is \code{NULL}.
		The coice \code{dist = TRUE} also limits \code{Nsim <- min(Nsim,1e8)}.
	}
	\item{tab0}{\code{TRUE} (default) or \code{FALSE}. If \code{tab0 = TRUE}, the null distribution
		is returned in 2 column matrix form when 
		\code{method = "simulated"}. When \code{tab0 = FALSE} the simulated null distribution 
		is returned as a vector of all simulated values of the test statistic.
	}
	\item{Nsim}{\code{=10000} (default), number of simulated \code{Avec} splits to use.	
		It is only used when \code{method = "simulated"},
		or when \code{method = "exact"} reverts to \code{method =}
		\code{"simulated"}, as previously explained.
	}

}
\details{
For this data scenario the Kruskal-Wallis criterion is
\deqn{K.star = \frac{N(N-1)}{mn}(\sum\frac{A_i^2}{d_i}-\frac{m^2}{N})}{K.star = 
N(N-1)/(mn) (\sum A_i^2/d_i-m^2/N)}
with \eqn{d_i=A_i+B_i}, treating "A" responses
as 1 and "B" responses as 2, and using midranks as explained in Lehmann (2006), Chapter 5.3.

For small sample sizes exact null distribution
calculations are possible, based on Algorithm C (Chase's sequence) in Knuth (2011),
which allows the enumeration of all possible splits of \eqn{m} into counts
\eqn{A_1,\ldots, A_t} such that 
\eqn{m = A_1 + \ldots + A_t},
followed by the calculation of the statistic 
\eqn{K.star} for each such split. 
Simulation of \eqn{A_1,\ldots, A_t} uses the probability model (5.35) in Lehmann (2006)
to successively generate hypergeometric counts \eqn{A_1,\ldots, A_t}.
Both these processes, enumeration and simulation, are done in C.
}
\value{
A list of class \code{kSamples} with components 
\item{test.name}{\code{"2 x t Contingency Table"}}
\item{t}{number of classification categories}
\item{KW.cont}{2 (3) vector giving the observed KW statistic, its asymptotic 
\eqn{P}-value (and simulated or exact \eqn{P}-value) }
\item{null.dist}{simulated or enumerated null distribution 
of the test statistic. It is given as an \code{M} by 2 matrix,
where the first column (named \code{KW}) gives the \code{M} unique ordered 
values of the Kruskal-Wallis 
statistic and the second column (named \code{prob}) gives the corresponding (simulated or exact)
probabilities. 

This format of \code{null.dist} is returned when \code{method = "exact"} 
and \code{dist} \code{= TRUE} or when \code{method =}\code{ "simulated"} 
and \code{dist = TRUE} and \code{tab0} \code{= TRUE} are specified.

For \code{method =} \code{"simulated"}, \code{dist = TRUE}, and 
\code{tab0 = FALSE} the null distribution \code{null.dist} is returned as the vector of
all simulated test statistic values. This is used in \code{\link{contingency2xt.comb}}
in the simulation mode.

\code{null.dist = NULL} is returned 
when \code{dist = FALSE} or when \code{method =}
\code{"asymptotic"}. }
\item{method}{the \code{method} used.}
\item{Nsim}{the number of simulations.}
}

\section{warning}{\code{method = "exact"} should only be used with caution.
Computation time is proportional to the number of enumerations. In most cases
\code{dist = TRUE} should not be used, i.e.,  
when the returned distribution objects 
become too large for R's work space.}

\references{
Knuth, D.E. (2011), \emph{The Art of Computer Programming, Volume 4A 
Combinatorial Algorithms Part 1}, Addison-Wesley


Kruskal, W.H. (1952), A Nonparametric Test for the Several Sample Problem,
\emph{The Annals of Mathematical Statistics},
\bold{Vol 23, No. 4}, 525-540

Kruskal, W.H. and Wallis, W.A. (1952), Use of Ranks in One-Criterion Variance Analysis,
\emph{Journal of the American Statistical Association}, 
\bold{Vol 47, No. 260}, 583--621. 

Lehmann, E.L. (2006), \emph{Nonparametrics, Statistical Methods Based on Ranks},
Revised First Edition, 
Springer, New York.

}




\examples{
contingency2xt(c(25,15,20),c(16,6,18),method="exact",dist=FALSE,
	tab0=TRUE,Nsim=1e3)
}

\keyword{nonparametric}
\keyword{htest}

\name{qn.test.combined}
\alias{qn.test.combined}
\title{
Combined Rank Score k-Sample Tests
}
\description{
This function combines several independent rank score \eqn{k}-sample tests
into one overall test of the hypothesis that the independent samples within 
each block come from a common unspecified distribution, while the common
distributions may vary from block to block.  
}
\usage{
qn.test.combined(\dots, data = NULL, test = c("KW", "vdW", "NS"),
	method = c("asymptotic", "simulated", "exact"),
	dist = FALSE, Nsim = 10000)
}
\arguments{
  	\item{\dots}{
		Either a sequence of several lists, say \eqn{L_1, \ldots, L_M} (\eqn{M > 1})
		where list \eqn{L_i} contains \eqn{k_i > 1}  sample vectors of respective 
		sizes \eqn{n_{i1}, \ldots, n_{ik_i}}, 
		where \eqn{n_{ij} > 4} is recommended
		for reasonable asymptotic \eqn{P}-value calculation. 
		\eqn{N_i=n_{i1}+\ldots+n_{ik_i}}
		is the pooled sample size for block \eqn{i},

		or a list of such lists,

		or a formula, like y ~ g | b, where y is a numeric response vector,
		g is a factor with levels indicating different treatments and
		b is a factor indicating different blocks; y, g, b have same length. 
		y is split separately for each block level into separate samples 
		according to the g levels. The same g level may occur in different 
		blocks. The variable names may correspond to variables in an optionally 
		supplied data frame via the data = argument.
	}
    	\item{data}{= an optional data frame providing the variables in formula input
		}
	\item{test}{= \code{c("KW", "vdW", "NS")}, 

		where \code{"KW"} uses scores \code{1:N} (Kruskal-Wallis test)

		\code{"vdW"} uses van der Waerden scores, \code{qnorm( (1:N) / (N+1) )}

		\code{"NS"} uses normal scores, i.e., expected values of 
		standard normal order statistics,
		invoking function \code{normOrder} of \code{package SuppDists (>=1.1-9.4)}

		For the above scores \eqn{N} changes from block to block and represents the total 
		pooled sample size \eqn{N_i}{N.i} for block \eqn{i}. 
	}
    \item{method}{=\code{c("asymptotic","simulated","exact")}, where

		\code{"asymptotic"} uses only an asymptotic chi-square approximation for the  \eqn{P}-value.
		The adequacy of asymptotic \eqn{P}-values for use with moderate sample sizes
		may be checked with \code{method = "simulated"}.

		\code{"simulated"} uses simulation to get \code{Nsim} simulated \eqn{QN} statistics for each
		block of samples, adding them component wise across blocks to get \code{Nsim} 
		combined values, and compares these with the observed combined value to 
		get the estimated \eqn{P}-value.

		\code{"exact"} uses full enumeration of the test statistic value 
		for all sample splits of the pooled samples within each block.
		The test statistic vectors for each block are added 
		(each component against each component, as in the R \code{outer(x,y,} \code{"+")} command) 
		to get the convolution enumeration for the combined test statistic.
		This "addition" is done one block at a time. 
		It is possible only for small problems, and is attempted only when \code{Nsim}
		is at least the (conservatively maximal) length 
		\deqn{\frac{N_1!}{n_{11}!\ldots n_{1k_1}!}\times\ldots\times \frac{N_M!}{n_{M1}!\ldots n_{Mk_M}!}}
		of the final distribution vector, were \eqn{N_i = n_{i1}+\ldots+n_{ik_i}} 
		is the sample size of the pooled samples for the i-th block. Otherwise, it reverts to the 
		simulation method using the provided \code{Nsim}.
	}
	\item{dist}{\code{FALSE} (default) or \code{TRUE}. If \code{TRUE}, the 
		simulated or fully enumerated convolution vector \code{null.dist} is returned for the 
		\eqn{QN} test statistic. 

		Otherwise, \code{NULL} is returned.
	}
	\item{Nsim}{\code{= 10000} (default), number of simulation splits to use within 
		each block of samples. It is only used when \code{method =} \code{"simulated"}
		or when \code{method =} \code{"exact"} reverts to \code{method =} \code{"simulated"}, 
		as previously explained.
		Simulations are independent across blocks, using \code{Nsim} for each block.
	}

}
\details{
The rank score \eqn{QN} criterion \eqn{QN_i}{QN.i} for the \eqn{i}-th block of 
\eqn{k_i}{k.i} samples, 
is used to test the hypothesis that the samples in the \eqn{i}-th block all come 
from the same but unspecified continuous distribution function \eqn{F_i(x)}{F.i(x)}.
See \code{\link{qn.test}} for the definition of the \eqn{QN} criterion
and the exact calculation of its null distribution.

The combined \eqn{QN} criterion \eqn{QN_{\rm comb} = QN_1 + \ldots + QN_M}{%
QN.comb = QN.1 + \ldots + QN.M}
is used to simultaneously test whether the samples 
in block i come from the same continuous distribution function \eqn{F_i(x)}{F.i(x)}. 
However, the unspecified common distribution function \eqn{F_i(x)}{F.i(x)} may change 
from block to block. 

The \eqn{k} for each block of \eqn{k}
independent samples may change from block to block.

The asymptotic approximating chi-square distribution has 
\eqn{f = (k_1-1)+\ldots+(k_M-1)}{f = (k.1-1)+\ldots+(k.M-1)} degrees of freedom.

NA values are removed and the user is alerted with the total NA count.
It is up to the user to judge whether the removal of NA's is appropriate.

The continuity assumption can be dispensed with if we deal with 
independent random samples, or if randomization was used in allocating
subjects to samples or treatments, independently from block to block, and if
the asymptotic, simulated or exact \eqn{P}-values are viewed conditionally, given the tie patterns
within each block. Under such randomization any conclusions 
are valid only with respect to the blocks of subjects that were randomly allocated.
In case of ties the average rank scores are used across tied observations within each block.
}
\value{
A list of class \code{kSamples} with components 
\item{test.name}{\code{"Kruskal-Wallis"}, \code{"van der Waerden scores"}, or

\code{"normal scores"}}
\item{M}{number of blocks of samples being compared}
\item{n.samples}{list of \code{M} vectors, each vector giving the sample sizes for 
each block of samples being compared}
\item{nt}{vector of length \code{M} of total sample sizes involved in each of the 
\code{M} comparisons of \eqn{k_i}{k.i} samples, respectively}
\item{n.ties}{vector giving the number of ties in each the \code{M}
comparison blocks}
\item{qn.list}{list of \code{M} matrices giving the \code{qn} results 
from \code{qn.test}, applied to the samples in each of
the \code{M} blocks}
\item{qn.c}{2 (or 3) vector containing the observed 
\eqn{QN_{\rm comb}}{QN.comb}, asymptotic \eqn{P}-value, 
(simulated or exact \eqn{P}-value).}
\item{warning}{logical indicator, \code{warning = TRUE} when at least one 
\eqn{n_{ij} < 5}{n.ij < 5}.}
\item{null.dist}{simulated or enumerated null distribution of the 
\eqn{QN_{\rm comb}}{QN.comb} statistic.
It is \code{NULL} when \code{dist = FALSE} or when \code{method = "asymptotic"}.}
\item{method}{The \code{method} used.}
\item{Nsim}{The number of simulations used for each block of samples.}
}
\references{
Lehmann, E.L. (2006), \emph{Nonparametric, Statistical Methods Based on Ranks}, Springer Verlag, New York. Ch. 6, Sec. 5D.
}

\note{
These tests are useful in analyzing treatment effects of shift nature in randomized 
(incomplete) block experiments.
}
\seealso{
\code{\link{qn.test}}
}
\examples{
## Create two lists of sample vectors.
x1 <- list( c(1, 3, 2, 5, 7), c(2, 8, 1, 6, 9, 4), c(12, 5, 7, 9, 11) )
x2 <- list( c(51, 43, 31, 53, 21, 75), c(23, 45, 61, 17, 60) )
# and a corresponding data frame datx1x2
x1x2 <- c(unlist(x1),unlist(x2))
gx1x2 <- as.factor(c(rep(1,5),rep(2,6),rep(3,5),rep(1,6),rep(2,5)))
bx1x2 <- as.factor(c(rep(1,16),rep(2,11)))
datx1x2 <- data.frame(A = x1x2, G = gx1x2, B = bx1x2)

## Run qn.test.combined.
set.seed(2627)
qn.test.combined(x1, x2, method = "simulated", Nsim = 1000) 
# or with same seed
# qn.test.combined(list(x1, x2), method = "simulated", Nsim = 1000)
# or qn.test.combined(A~G|B,data=datx1x2,method="simulated",Nsim=1000)
}



\keyword{nonparametric}
\keyword{htest}
\keyword{design}

\name{jt.test}
\alias{jt.test}

\title{
Jonckheere-Terpstra k-Sample Test for Increasing Alternatives
}
\description{
The
Jonckheere-Terpstra k-sample test statistic JT is defined
as \eqn{JT = \sum_{i<j} W_{ij}} where
\eqn{W_{ij}} is the Mann-Whitney statistic comparing 
samples \eqn{i} and \eqn{j}, indexed in the order 
of the stipulated increasing alternative. 
There may be ties in the pooled samples.
}
\usage{
jt.test(\dots, data = NULL, method=c("asymptotic","simulated","exact"),
		dist = FALSE, Nsim = 10000) 
}
\arguments{
	\item{\dots}{
		Either several sample vectors, say 
		\eqn{x_1, \ldots, x_k}, 
		with \eqn{x_i} containing \eqn{n_i} sample values.
		\eqn{n_i > 4} is recommended for reasonable asymptotic 
		\eqn{P}-value calculation. The pooled sample size is denoted 
		by \eqn{N=n_1+\ldots+n_k}. The order of samples should be
		as stipulated under the alternative

		or a list of such sample vectors,

		or a formula y ~ g, where y contains the pooled sample values 
		and g (same length as y) is a factor with levels identifying 
		the samples to which the elements of y belong, the factor levels
		reflecting the order under the stipulated alternative,

	}

    	\item{data}{= an optional data frame providing the variables in formula y ~ g.
		}
	
    	\item{method}{= \code{c("asymptotic","simulated","exact")}, where

		\code{"asymptotic"} uses only an asymptotic normal \eqn{P}-value approximation.

		\code{"simulated"} uses \code{Nsim} simulated \eqn{JT} statistics based on random splits of the 
		pooled samples into samples of sizes 
		\eqn{n_1, \ldots, n_k}, to estimate the \eqn{P}-value.

		\code{"exact"} uses full enumeration of all sample splits with 
		resulting \eqn{JT} statistics to obtain the exact \eqn{P}-value. 
		It is used only when \code{Nsim} is at least as large as the number
		\deqn{ncomb = \frac{N!}{n_1!\ldots n_k!}}{N!/(n_1!\ldots n_k!)}
		of full enumerations. Otherwise, \code{method}
		reverts to \code{"simulated"} using the given \code{Nsim}. It also reverts
		to \code{"simulated"} when \eqn{ncomb > 1e8} and \code{dist = TRUE}.
    	}
	\item{dist}{\code{= FALSE} (default) or \code{TRUE}. If \code{TRUE}, the 
		simulated or fully enumerated distribution vector \code{null.dist}
		is returned for the JT test statistic. Otherwise, \code{NULL} is returned. 
		When \code{dist = TRUE} then \code{Nsim <- min(Nsim, 1e8)}, 
		to limit object size.
	}
	\item{Nsim}{\code{= 10000} (default), number of simulation sample splits to use.	
		It is only used when \code{method = "simulated"},
		or when \code{method = "exact"} reverts to \code{method =}
		\code{ "simulated"}, as previously explained.
	}

}
\details{
The JT statistic 
is used to test the hypothesis that the samples all come 
from the same but unspecified continuous distribution function \eqn{F(x)}.
It is specifically aimed at alternatives where the sampled distributions
are stochastically increasing.

NA values are removed and the user is alerted with the total NA count.
It is up to the user to judge whether the removal of NA's is appropriate.

The continuity assumption can be dispensed with, if we deal with 
independent random samples, or if randomization was used in allocating
subjects to samples or treatments, and if we view
the simulated or exact \eqn{P}-values conditionally, given the tie pattern
in the pooled samples. Of course, under such randomization any conclusions 
are valid only with respect to the group of subjects that were randomly allocated
to their respective samples.
The asymptotic \eqn{P}-value calculation is valid provided all sample sizes become large.
}
\value{
A list of class \code{kSamples} with components 
\item{test.name}{\code{"Jonckheere-Terpstra"}}
\item{k}{number of samples being compared}
\item{ns}{vector \eqn{(n_1,\ldots,n_k)} of the \eqn{k} sample sizes}
\item{N}{size of the pooled sample \eqn{= n_1+\ldots+n_k}}
\item{n.ties}{number of ties in the pooled sample}
\item{qn}{4 (or 5) vector containing the observed \eqn{JT}, its mean and standard deviation
and its asymptotic \eqn{P}-value, 
(and its simulated or exact \eqn{P}-value)}
\item{warning}{logical indicator, \code{warning = TRUE} when at least one 
\eqn{n_i < 5}{n.i < 5}}
\item{null.dist}{simulated or enumerated null distribution 
of the test statistic. It is \code{NULL} when \code{dist = FALSE} or when
\code{method = "asymptotic"}.}
\item{method}{the \code{method} used.}
\item{Nsim}{the number of simulations used.}

}


\references{
Harding, E.F. (1984), An Efficient, Minimal-storage Procedure for
Calculating the Mann-Whitney U, Generalized U and Similar Distributions,
\emph{Appl. Statist.} \bold{33} No. 1, 1-6.


Jonckheere, A.R. (1954), A Distribution Free \emph{k}-sample Test against Ordered 
Alternatives,
\emph{Biometrika}, \bold{41}, 133-145.

Lehmann, E.L. (2006),
\emph{Nonparametrics, Statistical Methods Based on Ranks, Revised First Edition},
Springer Verlag.

Terpstra, T.J. (1952), The Asymptotic Normality and Consistency of Kendall's Test against Trend, when Ties are Present in One Ranking,
\emph{Indagationes Math.} \bold{14}, 327-333.
}




\examples{
x1 <- c(1,2)
x2 <- c(1.5,2.1)
x3 <- c(1.9,3.1)
yy <- c(x1,x2,x3)
gg <- as.factor(c(1,1,2,2,3,3))
jt.test(x1, x2, x3,method="exact",Nsim=90)
# or 
# jt.test(list(x1, x2, x3), method = "exact", Nsim = 90)
# or
# jt.test(yy ~ gg, method = "exact", Nsim = 90)
}
\keyword{nonparametric}
\keyword{htest}
\keyword{design}

\name{ad.test.combined}
\alias{ad.test.combined}
\title{
Combined Anderson-Darling k-Sample Tests
}
\description{
This function combines several independent Anderson-Darling \eqn{k}-sample tests
into one overall test of the hypothesis that the independent samples within 
each block come from a common unspecified distribution, while the common
distributions may vary from block to block.  Both versions of the 
Anderson-Darling test statistic are provided.
}
\usage{
ad.test.combined(\dots, data = NULL,
	method = c("asymptotic", "simulated", "exact"),
	dist = FALSE, Nsim = 10000)
}
\arguments{
  	\item{\dots}{
		Either a sequence of several lists, say \eqn{L_1, \ldots, L_M} (\eqn{M > 1})
		where list \eqn{L_i} contains \eqn{k_i > 1}  sample vectors of respective 
		sizes \eqn{n_{i1}, \ldots, n_{ik_i}}, 
		where \eqn{n_{ij} > 4} is recommended
		for reasonable asymptotic \eqn{P}-value calculation. 
		\eqn{N_i=n_{i1}+\ldots+n_{ik_i}}
		is the pooled sample size for block \eqn{i},

		or a list of such lists,

		or a formula, like y ~ g | b, where y is a numeric response vector,
		g is a factor with levels indicating different treatments and
		b is a factor indicating different blocks; y, g, b are or equal length. 
		y is split separately for each block level into separate samples 
		according to the g levels. The same g level may occur in different blocks. 
		The variable names may correspond to variables in an optionally supplied 
		data frame via the data = argument,
	}
    \item{data}{= an optional data frame providing the variables in formula input
		}
    \item{method}{= \code{c("asymptotic","simulated","exact")}, where

		\code{"asymptotic"} uses only an asymptotic \eqn{P}-value approximation, reasonable 
		for P in [0.00001, .99999], linearly extrapolated via 
		\eqn{\log(P/(1-P))}{log(P/(1-P))} outside 
		that range. See \code{\link{ad.pval}} for details. 		
		The adequacy of the asymptotic \eqn{P}-value calculation may be checked using 
		\code{\link{pp.kSamples}}.


		\code{"simulated"} uses simulation to get \code{Nsim} simulated \eqn{AD} statistics 
		for each block of samples, adding them across blocks component wise to get \code{Nsim} 
		combined values. These are compared with the observed combined value to obtain the 
		estimated	\eqn{P}-value.

		\code{"exact"} uses full enumeration of the test statistic values 
		for all sample splits of the pooled samples within each block.
		The test statistic vectors for the first 2 blocks are added 
		(each component against each component, as in the R \code{outer(x,y,} \code{"+")} command) 
		to get the convolution enumeration for the combined test statistic. The resulting
		vector is convoluted against the next block vector in the same fashion, and so on.
		It is possible only for small problems, and is attempted only when \code{Nsim}
		is at least the (conservatively maximal) length 
		\deqn{\frac{N_1!}{n_{11}!\ldots n_{1k_1}!}\times\ldots\times 
			\frac{N_M!}{n_{M1}!\ldots n_{Mk_M}!}}
		of the final distribution vector. Otherwise, it reverts to the 
		simulation method using the provided \code{Nsim}.
    	}
	\item{dist}{\code{FALSE} (default) or \code{TRUE}. If \code{TRUE}, the 
		simulated or fully enumerated convolution vectors 
		\code{null.dist1} and \code{null.dist2} are returned for the respective
		test statistic versions. Otherwise, \code{NULL} is returned for each.
	}
	\item{Nsim}{\code{= 10000} (default), number of simulation splits to use within 
		each block of samples. It is only used when \code{method = "simulated"}
		or when \code{method =} \code{"exact"} reverts to \code{method = "simulated"}, 
		as previously explained. Simulations are independent across blocks, 
		using \code{Nsim} for each block. \code{Nsim} is limited by \code{1e7}.
	}

}
\details{
If \eqn{AD_i} is the Anderson-Darling criterion for the i-th block of 
\eqn{k_i} samples, 
its standardized test statistic is 
\eqn{T_i = (AD_i - \mu_i)/\sigma_i}, with 
\eqn{\mu_i} and
\eqn{\sigma_i} representing mean and standard deviation of 
\eqn{AD_i}. This statistic 
is used to test the hypothesis that the samples in the i-th block all come 
from the same but unspecified continuous distribution function \eqn{F_i(x)}.

The combined Anderson-Darling criterion is 
\eqn{AD_{comb}=AD_1 + \ldots + AD_M} and 
\eqn{T_{comb} = } \eqn{(AD_{comb} - \mu_c)/\sigma_c} is the standardized form, 
where \eqn{\mu_c=\mu_1+\ldots+\mu_M} and \eqn{\sigma_c =
\sqrt{\sigma_1^2 +\ldots+\sigma_M^2}} 
represent the mean and standard deviation of \eqn{AD_{comb}}.
The statistic \eqn{T_{comb}} is used to simultaneously 
test whether the samples 
in each block come from the same continuous distribution function 
\eqn{F_i(x), i=1,\ldots,M}. 
The unspecified common distribution function \eqn{F_i(x)} may change 
from block to block. According to the reference article, two versions
of the test statistic and its corresponding combinations are provided.

The \eqn{k_i} for each block of \eqn{k_i}
independent samples may change from block to block.

NA values are removed and the user is alerted with the total NA count.
It is up to the user to judge whether the removal of NA's is appropriate.

The continuity assumption can be dispensed with if we deal with 
independent random samples, or if randomization was used in allocating
subjects to samples or treatments, independently from block to block, and if we view
the simulated or exact \eqn{P}-values conditionally, given the tie patterns
within each block. Of course, under such randomization any conclusions 
are valid only with respect to the blocks of subjects that were randomly allocated.
The asymptotic \eqn{P}-value calculation assumes distribution continuity. No adjustment 
for lack thereof is known at this point. The same comment holds for the means
and standard deviations of respective statistics.
}
\value{
A list of class \code{kSamples} with components 
\item{test.name}{\eqn{=} \code{"Anderson-Darling"}}
\item{M}{number of blocks of samples being compared}
\item{n.samples}{list of \code{M} vectors, each vector giving the sample sizes for 
each block of samples being compared}
\item{nt}{\eqn{= (N_1,\ldots,N_M)}}
\item{n.ties}{vector giving the number of ties in each the \code{M}
comparison blocks}
\item{ad.list}{list of \code{M} matrices giving the \code{ad} results 
for \code{ad.test} applied to the samples in each of
the \code{M} blocks}
\item{mu}{vector of means of the \eqn{AD} statistic for the  \code{M} blocks}
\item{sig}{vector of standard deviations of the \eqn{AD} statistic for the  \code{M} blocks}
\item{ad.c}{2 x 3 (2 x 4) matrix containing 
\eqn{AD_{comb}, T_{comb}}, asymptotic \eqn{P}-value, 
(simulated or exact \eqn{P}-value), for each version of the combined test statistic,
version 1 in row 1 and version 2 in row 2}
\item{mu.c}{mean of \eqn{AD_{comb}}}
\item{sig.c}{standard deviation of \eqn{AD_{comb}}}
\item{warning}{logical indicator, \code{warning = TRUE} when at least one 
\eqn{n_{ij} < 5}}
\item{null.dist1}{simulated or enumerated null distribution of version 1 
of \eqn{AD_{comb}}}
\item{null.dist2}{simulated or enumerated null distribution of version 2 
of \eqn{AD_{comb}}}
\item{method}{the \code{method} used.}
\item{Nsim}{the number of simulations used for each block of samples.}
}
\references{
Scholz, F. W. and Stephens, M. A. (1987), K-sample Anderson-Darling Tests, 
\emph{Journal of the American Statistical Association}, 
\bold{Vol 82, No. 399}, 918--924. 
}

\note{
This test is useful in analyzing treatment effects in randomized 
(incomplete) block experiments and in examining performance 
equivalence of several laboratories when presented with different 
test materials for comparison.
}
\seealso{
\code{\link{ad.test}}, \code{\link{ad.pval}}
}
\examples{
## Create two lists of sample vectors.
x1 <- list( c(1, 3, 2, 5, 7), c(2, 8, 1, 6, 9, 4), c(12, 5, 7, 9, 11) )
x2 <- list( c(51, 43, 31, 53, 21, 75), c(23, 45, 61, 17, 60) )
# and a corresponding data frame datx1x2
x1x2 <- c(unlist(x1),unlist(x2))
gx1x2 <- as.factor(c(rep(1,5),rep(2,6),rep(3,5),rep(1,6),rep(2,5)))
bx1x2 <- as.factor(c(rep(1,16),rep(2,11)))
datx1x2 <- data.frame(A = x1x2, G = gx1x2, B = bx1x2)

## Run ad.test.combined.
set.seed(2627)
ad.test.combined(x1, x2, method = "simulated", Nsim = 1000) 
# or with same seed
# ad.test.combined(list(x1, x2), method = "simulated", Nsim = 1000)
# ad.test.combined(A~G|B,data=datx1x2,method="simulated",Nsim=1000)


}

\keyword{nonparametric}
\keyword{htest}
\keyword{design}

\name{kSamples-package}
\alias{kSamples-package}
\alias{kSamples}
\docType{package}
\title{
The Package kSamples Contains Several Nonparametric K-Sample Tests and 
their Combinations over Blocks
}
\description{
The k-sample Anderson-Darling, Kruskal-Wallis, normal score and van der Waerden score tests
are used to test the hypothesis that k samples of 
sizes \eqn{n_1, \ldots, n_k}
come from a common continuous distribution \eqn{F} that is otherwise unspecified. They are rank tests.
Average rank scores are used in case of ties. 
While \code{\link{ad.test}} is consistent against all alternatives, \code{\link{qn.test}}
tends to be sensitive mainly to shifts between samples.
The combined versions of these tests,
\code{\link{ad.test.combined}} and
\code{\link{qn.test.combined}}, are 
used to simultaneously test such hypotheses across several blocks of samples.
The hypothesized common distributions and the number k of samples for each block of samples
may vary from block to block. 

The Jonckheere-Terpstra test addresses the same hypothesis as above but is sensitive to increasing
alternatives (stochastic ordering).

Also treated is the analysis of 2 x t contingency tables
using the Kruskal-Wallis criterion and its extension to blocks.
 
Steel's simultaneous comparison test of a common control sample with \eqn{s=k-1} treatment samples
using pairwise Wilcoxon tests for each control/treatment pair is provided, and also 
the simultaneous confidence bounds of treatment shift effects resulting from the inversion of these tests
when sampling from continuous populations.

Distributional aspects are handled asymptotically in all cases, and by choice also
via simulation or exact enumeration.
While simulation is always an option, exact calculations
are only possible for small sample sizes and only when few samples are involved. These exact
calculations can be done with or without ties in the pooled samples, based on a recursively extended
version of Algorithm C (Chase's sequence) in Knuth (2011), which allows the 
enumeration of all possible splits of the pooled data into samples of
sizes of \eqn{n_1, \ldots, n_k}, as appropriate under treatment randomization
or random sampling, when viewing tests conditionally given the observed tie pattern.
}


\author{
Fritz Scholz and Angie Zhu

Maintainer: Fritz Scholz <fscholz@u.washington.edu>
}
\references{
Hajek, J., Sidak, Z., and Sen, P.K. (1999), \emph{Theory of Rank Tests (Second Edition)}, Academic Press.

Knuth, D.E. (2011), \emph{The Art of Computer Programming, Volume 4A 
Combinatorial Algorithms Part 1}, Addison-Wesley

Kruskal, W.H. (1952), A Nonparametric Test for the Several Sample Problem,
\emph{The Annals of Mathematical Statistics},
\bold{Vol 23, No. 4}, 525-540

Kruskal, W.H. and Wallis, W.A. (1952), Use of Ranks in One-Criterion Variance Analysis,
\emph{Journal of the American Statistical Association}, 
\bold{Vol 47, No. 260}, 583--621. 

Lehmann, E.L. (2006), \emph{Nonparametrics, Statistical Methods Based on Ranks},
Revised First Edition, 
Springer, New York.

Scholz, F.W. (2023), "On Steel's Test with Ties", \url{https://arxiv.org/abs/2308.05873}

Scholz, F. W. and Stephens, M. A. (1987), K-sample Anderson-Darling Tests, \emph{Journal of the American Statistical Association}, 
\bold{Vol 82, No. 399}, 918--924. 
}


\keyword{nonparametric}
\keyword{htest}
\keyword{design}

\name{contingency2xt.comb}
\alias{contingency2xt.comb}
\title{
Combined Kruskal-Wallis Tests for the 2 x t Contingency Tables
}
\description{
This function uses the Kruskal-Wallis criterion to test
the hypothesis of no association between the counts 
for two responses
"A" and "B" across t categories and across \eqn{M} blocks.
}
\usage{
contingency2xt.comb(\dots, 
	method = c("asymptotic", "simulated", "exact"), 
	dist = FALSE, Nsim = 10000)
}
\arguments{
  	\item{\dots}{ 
		Either several lists \eqn{L_1,\ldots,L_M}, each
		of two equal length vectors \eqn{A_i} and
		\eqn{B_i}, \eqn{i=1,\ldots,M}, of counts \eqn{\ge 0}, 
		where the common length \eqn{t_i} of \eqn{A_i} and
		\eqn{B_i} may vary from list to list

		or a list of \code{M} such lists
	}
    \item{method}{= \code{c("asymptotic","simulated","exact")}, where

		\code{"asymptotic"} uses only an asymptotic chi-square approximation 
		with \eqn{(t_1-1)+\ldots+(t_M-1)} degrees of freedom
		to approximate the \eqn{P}-value, This calculation is always done.

		\code{"simulated"} uses \code{Nsim} simulated counts for the two vectors
		\eqn{A_i} and \eqn{B_i} in list \eqn{L_i}, 
		with the observed marginal totals, \eqn{m_i=\sum A_i}, 
		\eqn{n_i = \sum B_i}, \eqn{d_i = A_i+B_i}. 
		It does this independently from list to list using the same \code{Nsim} each time, 
		adding the resulting Kruskal-Wallis criteria across lists
		to get \code{Nsim} such summed values to estimate the \eqn{P}-value.

		\code{"exact"} enumerates all counts for \eqn{A_i} and \eqn{B_i} with 
		the respective observed marginal totals to get an exact distribution for each list. 
		These distributions are then convolved to obtain the \eqn{P}-value.
		It is used only when \code{Nsim} is at least as large as the product across blocks
		of the number \code{choose(m+t-1,t-1)} of full enumerations per block, where
		\eqn{t = t_1,\ldots, t_M}.
		Otherwise, \code{method} reverts to \code{"simulated"} using the given \code{Nsim}.
    }
	\item{dist}{\code{FALSE} (default) or \code{TRUE}. If \code{TRUE}, the 
		simulated or fully enumerated null distribution \code{null.dist} is returned
		for the Kruskal-Wallis test statistic. Otherwise \code{null.dist = NULL} is returned.
	}
	\item{Nsim}{\code{=10000} (default), number of simulated \eqn{A_i} splits to use per block.
		It is only used when \code{method = "simulated"},
		or when \code{method = "exact"} reverts to \code{method = "simulated"}, as previously explained.
	}

}
\details{
For details on the calculation of the Kruskal-Wallis criterion and its exact or simulated
distribution for each block see \code{\link{contingency2xt}}.
}
\value{
A list of class \code{kSamples} with components 
\item{test.name}{\code{"Combined 2 x t Contingency Tables"}}
\item{t}{vector giving the number of classification categories per block}
\item{M}{number of blocked tables}
\item{kw.list}{a list of the \code{KW.cont} output componenents from 
\code{\link{contingency2xt}} for each of the blocks}
\item{null.dist}{simulated or enumerated null distribution 
of the combined test statistic. It is given as an \code{L} by 2 matrix,
where the first column (named \code{KW}) gives the \code{L} unique ordered 
values of the combined Kruskal-Wallis 
statistic and the second column (named \code{prob}) gives the corresponding (simulated or exact)
probabilities.

\code{null.dist = NULL} is returned when \code{dist = FALSE} or when 
\code{method =} \code{"asymptotic"}.}
\item{method}{the \code{method} used.}
\item{Nsim}{the number of simulations.}
}
\note{
The required level for \code{Nsim} in order for \code{method = "exact"}
to be carried out, is conservative, but there is no transparent way to get a 
better estimate. The actual dimension \code{L} of the realized \code{null.dist}
will typically be much smaller, since the distribution is compacted to
its unique support values.
}

\section{warning}{\code{method = "exact"} should only be used with caution.
Computation time is proportional to the number of enumerations. In most cases
\code{dist = TRUE} should not be used, i.e.,  
when the returned distribution objects 
become too large for R's work space.}



\examples{
out <- contingency2xt.comb(list(c(25,15,20),c(16,6,18)),
list(c(12,4,5),c(13,8,9)),method = "simulated", dist=FALSE, Nsim=1e3)
}

\keyword{nonparametric}
\keyword{htest}

\name{qn.test}
\alias{qn.test}
\title{
Rank Score k-Sample Tests
}
\description{
This function uses the \eqn{QN} criterion (Kruskal-Wallis, van der Waerden scores, normal scores) to test
the hypothesis that \eqn{k} independent samples arise
from a common unspecified distribution.
}
\usage{
qn.test(\dots, data = NULL, test = c("KW", "vdW", "NS"), 
	method = c("asymptotic", "simulated", "exact"),
	dist = FALSE, Nsim = 10000)
}
\arguments{
	\item{\dots}{
		Either several sample vectors, say 
		\eqn{x_1, \ldots, x_k}, 
		with \eqn{x_i} containing \eqn{n_i} sample values.
		\eqn{n_i > 4} is recommended for reasonable asymptotic 
		\eqn{P}-value calculation. The pooled sample size is denoted 
		by \eqn{N=n_1+\ldots+n_k},

		or a list of such sample vectors,

		or a formula y ~ g, where y contains the pooled sample values 
		and g (same length as y) is a factor with levels identifying 
		the samples to which the elements of y belong. 
	}

    	\item{data}{= an optional data frame providing the variables in formula y ~ g.
		}
	\item{test}{= \code{c("KW", "vdW", "NS")}, where

		\code{"KW"} uses scores \code{1:N} (Kruskal-Wallis test)

		\code{"vdW"} uses van der Waerden scores, \code{qnorm( (1:N) / (N+1) )}

		\code{"NS"} uses normal scores, i.e., expected standard normal order statistics,
		invoking function \code{normOrder} of \code{package SuppDists (>=1.1-9.4)}
	}
    	\item{method}{= \code{c("asymptotic","simulated","exact")}, where

		\code{"asymptotic"} uses only an asymptotic chi-square approximation 
		with \code{k-1} degrees of freedom to approximate the \eqn{P}-value. 
		This calculation is always done.

		\code{"simulated"} uses \code{Nsim} simulated \eqn{QN} statistics based on random 
		splits of the pooled samples into samples of sizes 
		\eqn{n_1, \ldots, n_k}, to estimate the \eqn{P}-value.

		\code{"exact"} uses full enumeration of all sample splits with resulting 
		\eqn{QN} statistics to obtain the exact \eqn{P}-value. 
     		It is used only when \code{Nsim} is at least as large as the number
		\deqn{ncomb = \frac{N!}{n_1!\ldots n_k!}}{N!/(n_1!\ldots n_k!)} of
		full enumerations. Otherwise, \code{method}
		reverts to \code{"simulated"} using the given \code{Nsim}. It also reverts
		to \code{"simulated"} when \eqn{ncomb > 1e8} and \code{dist = TRUE}.
    	}
	\item{dist}{\code{FALSE} (default) or \code{TRUE}. If \code{TRUE}, the 
		simulated or fully enumerated null distribution vector \code{null.dist} 
		is returned for the \eqn{QN} test statistic. Otherwise, \code{NULL} 
		is returned. When \code{dist = TRUE} then \code{Nsim <- min(Nsim, 1e8)}, 
		to limit object size.
	}
	\item{Nsim}{\code{= 10000} (default), number of simulation sample splits to use.	
		It is only used when \code{method = "simulated"},
		or when \code{method = "exact"} reverts to \code{method =}
		\code{ "simulated"}, as previously explained.
	}
}
\details{
The \eqn{QN} criterion based on rank scores \eqn{v_1,\ldots,v_N} is
\deqn{QN=\frac{1}{s_v^2}\left(\sum_{i=1}^k \frac{(S_{iN}-n_i \bar{v}_{N})^2}{n_i}\right)}
where \eqn{S_{iN}} is the sum of rank scores for the \eqn{i}-th sample and 
\eqn{\bar{v}_N} and 
\eqn{s_v^2} are sample mean and sample variance (denominator \eqn{N-1})
of all scores.

The statistic \eqn{QN} is used to test the hypothesis that the samples all come 
from the same but unspecified continuous distribution function \eqn{F(x)}.
\eqn{QN} is always adjusted for ties by averaging the scores of tied observations.

Conditions for the asymptotic approximation (chi-square  with \eqn{k-1} degrees of freedom)  
can be found in Lehmann, E.L. (2006), Appendix Corollary 10, or in 
Hajek, Sidak, and Sen (1999), Ch. 6, problems 13 and 14.

For small sample sizes exact null distribution
calculations are possible (with or without ties), based on a recursively extended
version of Algorithm C (Chase's sequence) in Knuth (2011), which allows the 
enumeration of all possible splits of the pooled data into samples of
sizes of \eqn{n_1, \ldots, n_k}, as appropriate under treatment randomization. This 
is done in C, as is the simulation.

NA values are removed and the user is alerted with the total NA count.
It is up to the user to judge whether the removal of NA's is appropriate.

The continuity assumption can be dispensed with, if we deal with 
independent random samples from any common distribution, 
or if randomization was used in allocating
subjects to samples or treatments, and if 
the asymptotic, simulated or exact \eqn{P}-values are viewed conditionally, given the tie pattern
in the pooled sample. Under such randomization any conclusions 
are valid only with respect to the subjects that were randomly allocated
to their respective treatment samples.
}
\value{
A list of class \code{kSamples} with components 
\item{test.name}{\code{"Kruskal-Wallis"}, \code{"van der Waerden scores"}, or

\code{"normal scores"}}
\item{k}{number of samples being compared}
\item{ns}{vector \eqn{(n_1,\ldots,n_k)} of the \eqn{k} sample sizes}
\item{N}{size of the pooled samples \eqn{= n_1+\ldots+n_k}}
\item{n.ties}{number of ties in the pooled sample}
\item{qn}{2 (or 3) vector containing the observed \eqn{QN}, its asymptotic \eqn{P}-value, 
(its simulated or exact \eqn{P}-value)}
\item{warning}{logical indicator, \code{warning = TRUE} when at least one 
\eqn{n_i < 5}}
\item{null.dist}{simulated or enumerated null distribution 
of the test statistic. It is \code{NULL} when \code{dist = FALSE} or when
\code{method = "asymptotic"}.}
\item{method}{the \code{method} used.}
\item{Nsim}{the number of simulations used.}
}

\section{warning}{\code{method = "exact"} should only be used with caution.
Computation time is proportional to the number of enumerations. 
Experiment with \code{\link{system.time}} and trial values for
\code{Nsim} to get a sense of the required computing time.
In most cases
\code{dist = TRUE} should not be used, i.e.,  
when the returned distribution objects 
become too large for R's work space.}

\references{
Hajek, J., Sidak, Z., and Sen, P.K. (1999), \emph{Theory of Rank Tests (Second Edition)}, Academic Press.

Knuth, D.E. (2011), \emph{The Art of Computer Programming, Volume 4A 
Combinatorial Algorithms Part 1}, Addison-Wesley

Kruskal, W.H. (1952), A Nonparametric Test for the Several Sample Problem,
\emph{The Annals of Mathematical Statistics},
\bold{Vol 23, No. 4}, 525-540

Kruskal, W.H. and Wallis, W.A. (1952), Use of Ranks in One-Criterion Variance Analysis,
\emph{Journal of the American Statistical Association}, 
\bold{Vol 47, No. 260}, 583--621. 

Lehmann, E.L. (2006),
\emph{Nonparametrics, Statistical Methods Based on Ranks, Revised First Edition},
Springer Verlag.
}


\seealso{
\code{\link{qn.test.combined}}
}

\examples{
u1 <- c(1.0066, -0.9587,  0.3462, -0.2653, -1.3872)
u2 <- c(0.1005, 0.2252, 0.4810, 0.6992, 1.9289)
u3 <- c(-0.7019, -0.4083, -0.9936, -0.5439, -0.3921)
yy <- c(u1, u2, u3)
gy <- as.factor(c(rep(1,5), rep(2,5), rep(3,5)))
set.seed(2627)
qn.test(u1, u2, u3, test="KW", method = "simulated", 
  dist = FALSE, Nsim = 1000)
# or with same seed
# qn.test(list(u1, u2, u3),test = "KW", method = "simulated", 
#  dist = FALSE, Nsim = 1000)
# or with same seed
# qn.test(yy ~ gy, test = "KW", method = "simulated", 
#  dist = FALSE, Nsim = 1000)
}

\keyword{nonparametric}
\keyword{htest}
\keyword{design}

\name{ad.test}
\alias{ad.test}
\title{
Anderson-Darling k-Sample Test
}
\description{
This function uses the Anderson-Darling criterion to test
the hypothesis that \eqn{k} independent samples with sample sizes  
\eqn{n_1,\ldots, n_k} arose
from a common unspecified distribution function \eqn{F(x)} and testing is 
done conditionally given the observed tie pattern. Thus this is a permutation test.
Both versions of the \eqn{AD} statistic are computed.
}
\usage{
ad.test(\dots, data = NULL, method = c("asymptotic", "simulated", "exact"),
	dist = FALSE, Nsim = 10000)
}
\arguments{
	\item{\dots}{
		Either several sample vectors, say 
		\eqn{x_1, \ldots, x_k}, 
		with \eqn{x_i} containing \eqn{n_i} sample values.
		\eqn{n_i > 4} is recommended for reasonable asymptotic 
		\eqn{P}-value calculation. The pooled sample size is denoted 
		by \eqn{N=n_1+\ldots+n_k},

		or a list of such sample vectors,

		or a formula y ~ g, where y contains the pooled sample values 
		and g is a factor (of same length as y) with levels identifying 
		the samples to which the elements of y belong. 
	}

    	\item{data}{= an optional data frame providing the variables in formula y ~ g.
		}

    	\item{method}{= \code{c("asymptotic","simulated","exact")}, where

		\code{"asymptotic"} uses only an asymptotic \eqn{P}-value approximation, reasonable 
		for P in [.00001, .99999] when all \eqn{n_i > 4}. 
		Linear extrapolation via \eqn{\log(P/(1-P))}{log(P/(1-P))}
		is used outside [.00001, .99999]. This calculation is always done.
		See \code{\link{ad.pval}} for details. 
		The adequacy of the asymptotic \eqn{P}-value calculation
	 	may be checked using \code{\link{pp.kSamples}}.

		\code{"simulated"} uses \code{Nsim} simulated \eqn{AD} statistics, based on random 
		splits of the pooled samples into samples of sizes 
		\eqn{n_1, \ldots, n_k}, to estimate the exact conditional \eqn{P}-value.

		\code{"exact"} uses full enumeration of all sample splits with 
		resulting \eqn{AD} statistics to obtain the exact conditional \eqn{P}-values. 
		It is used only when \code{Nsim} is at least as large as the number
		\deqn{ncomb = \frac{N!}{n_1!\ldots n_k!}}{N!/(n_1!\ldots n_k!)}
		of full enumerations. Otherwise, \code{method}
		reverts to \code{"simulated"} using the given \code{Nsim}. It also reverts
		to \code{"simulated"} when \eqn{ncomb > 1e8} and \code{dist = TRUE}.
        }
	\item{dist}{\code{= FALSE} (default) or \code{TRUE}. If \code{TRUE}, the 
		simulated or fully enumerated distribution vectors \code{null.dist1} and
		\code{null.dist2} are returned for the respective test statistic versions.
		Otherwise, \code{NULL} is returned. When \code{dist = TRUE} then 
		\code{Nsim <- min(Nsim, 1e8)}, to limit object size.
	}
	\item{Nsim}{\code{= 10000} (default), number of simulation sample splits to use.	
		It is only used when \code{method = "simulated"},
		or when \code{method = "exact"} reverts to \code{method =}
		\code{ "simulated"}, as previously explained.
	}

}
\details{
If \eqn{AD} is the Anderson-Darling criterion for the \eqn{k} samples, 
its standardized test statistic is \eqn{T.AD = (AD - \mu)/\sigma}, with 
\eqn{\mu = k-1} and
\eqn{\sigma} representing mean and standard deviation of \eqn{AD}. This statistic 
is used to test the hypothesis that the samples all come 
from the same but unspecified continuous distribution function \eqn{F(x)}.


According to the reference article, two versions
of the \eqn{AD} test statistic are provided.
The above mean and standard deviation are strictly
valid only for version 1 in the 
continuous distribution case.

NA values are removed and the user is alerted with the total NA count.
It is up to the user to judge whether the removal of NA's is appropriate.

The continuity assumption can be dispensed with, if we deal with 
independent random samples, or if randomization was used in allocating
subjects to samples or treatments, and if we view
the simulated or exact \eqn{P}-values conditionally, given the tie pattern
in the pooled samples. Of course, under such randomization any conclusions 
are valid only with respect to the group of subjects that were randomly allocated
to their respective samples.
The asymptotic \eqn{P}-value calculation assumes distribution continuity. No adjustment 
for lack thereof is known at this point. For details on the asymptotic 
\eqn{P}-value calculation see \code{\link{ad.pval}}.
}
\value{
A list of class \code{kSamples} with components 
\item{test.name}{\code{"Anderson-Darling"}}
\item{k}{number of samples being compared}
\item{ns}{vector of the \eqn{k} sample sizes \eqn{(n_1,\ldots,n_k)}}
\item{N}{size of the pooled sample \eqn{= n_1+\ldots+n_k}}
\item{n.ties}{number of ties in the pooled samples}
\item{sig}{standard deviations \eqn{\sigma} of version 1 of \eqn{AD} under the continuity assumption}
\item{ad}{2 x 3 (2 x 4) matrix containing \eqn{AD, T.AD}, asymptotic \eqn{P}-value, 
(simulated or exact \eqn{P}-value), for each version of the standardized test statistic \eqn{T},
version 1 in row 1, version 2 in row 2.}
\item{warning}{logical indicator, warning = TRUE when at least one 
\eqn{n_i < 5}{n.i < 5}}
\item{null.dist1}{simulated or enumerated null distribution of version 1 
of the test statistic, given as vector of all generated \eqn{AD} statistics.}
\item{null.dist2}{simulated or enumerated null distribution of version 2 
of the test statistic, given as vector of all generated \eqn{AD} statistics.}
\item{method}{The \code{method} used.}
\item{Nsim}{The number of simulations.}
}

\section{warning }{\code{method = "exact"} should only be used with caution.
Computation time is proportional to the number of enumerations. In most cases
\code{dist = TRUE} should not be used, i.e.,  
when the returned distribution vectors \code{null.dist1} and \code{null.dist2}
become too large for the R work space. These vectors are limited in length by 1e8. 
}


\note{
For small sample sizes and small \eqn{k} exact null distribution
calculations are possible (with or without ties), based on a recursively extended
version of Algorithm C (Chase's sequence) in Knuth (2011), Ch. 7.2.1.3, which allows the 
enumeration of all possible splits of the pooled data into samples of
sizes of \eqn{n_1, \ldots, n_k}, as appropriate under treatment randomization. The
enumeration and simulation are both done in C.
}

\note{
It has recently come to our attention that the Anderson-Darling test, originally 
proposed by Pettitt (1976) in the 2-sample case and generalized to k samples by
Scholz and Stephens, has a close relative created by Baumgartner et al (1998) in
the 2 sample case and populatized by Neuhaeuser (2012) with at least 6 papers 
among his cited references and generalized by Murakami (2006) to k samples.
}

\references{
Baumgartner, W., Weiss, P. and Schindler, H. (1998), A nonparametric test for the 
general two-sample problem, \emph{Bionetrics}, \bold{54}, 1129-1135.

Knuth, D.E. (2011), \emph{The Art of Computer Programming, Volume 4A 
Combinatorial Algorithms Part 1}, Addison-Wesley


Neuhaeuser, M. (2012), \emph{Nonparametric Statistical Tests, A Computational Approach},
CRC Press.

Murakami, H. (2006), A k-sample rank test based on modified Baumgartner statistic and
it power comparison, 
\emph{Jpn. Soc. Comp. Statist.}, \bold{19}, 1-13.

Murakami, H. (2012), Modified Baumgartner statistic for the two-sample and multisample 
problems: a numerical comparison.
\emph{J. of Statistical Comput. and Simul.}, \bold{82:5}, 711-728.

Pettitt, A.N. (1976), A two-sample Anderson_Darling rank statistic, \emph{Biometrika},
\bold{63}, 161-168.

Scholz, F. W. and Stephens, M. A. (1987), K-sample Anderson-Darling Tests, 
\emph{Journal of the American Statistical Association}, 
\bold{Vol 82, No. 399}, 918--924. 
}

\seealso{
\code{\link{ad.test.combined}}, \code{\link{ad.pval}}
}
\examples{
u1 <- c(1.0066, -0.9587,  0.3462, -0.2653, -1.3872)
u2 <- c(0.1005, 0.2252, 0.4810, 0.6992, 1.9289)
u3 <- c(-0.7019, -0.4083, -0.9936, -0.5439, -0.3921)
y <- c(u1, u2, u3)
g <- as.factor(c(rep(1, 5), rep(2, 5), rep(3, 5)))
set.seed(2627)
ad.test(u1, u2, u3, method = "exact", dist = FALSE, Nsim = 1000)
# or with same seed
# ad.test(list(u1, u2, u3), method = "exact", dist = FALSE, Nsim = 1000)
# or with same seed
# ad.test(y ~ g, method = "exact", dist = FALSE, Nsim = 1000)
}

\keyword{nonparametric}
\keyword{htest}
\keyword{design}

\name{conv}
\alias{conv}
\title{
Convolution of Two Discrete Distributions
}
\description{
This function convolutes two discrete distribution, each given by strictly increasing support vectors
and corresponding probability vectors.
}
\usage{
	conv(x1,p1,x2,p2)
}
\arguments{
  	\item{x1}{
		support vector of the first distribution, with strictly increasing elements.
	}
	\item{p1}{vector of probabilities corresponding to \code{x1}.
	}
  	\item{x2}{
		support vector of the second distribution, with strictly increasing elements.
	}
	\item{p2}{vector of probabilities corresponding to \code{x2}.
	}
}
\details{
The convolution is performed in C, looping through all paired sums, 
augmenting existing values or inserting them with an update of the 
corresponding probabilities.
}
\value{
A matrix with first column the new support vector and the second column the 
corresponding probability vector.
}



\examples{
x1 <- c(1,2,3.5)
p1 <- c(.2,.3,.5)
x2 <- c(0,2.3,3,4)
p2 <- c(.1,.3,.3,.3)


conv(x1,p1,x2,p2)
}
